@@ -31,6 +31,7 @@
 #include "H5DataSet.h"
 #include "H5File.h"
 #include "H5Alltypes.h"
+#include "H5private.h"		// for HDstrcpy
 
 // There are a few comments that are common to most of the functions
 // defined in this file so they are listed here.
@@ -249,7 +250,7 @@ DataSet CommonFG::openDataSet( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 void CommonFG::link( H5L_type_t link_type, const char* curr_name, const char* new_name ) const
 {
-    herr_t ret_value;
+    herr_t ret_value = -1;
 
     switch(link_type) {
         case H5L_TYPE_HARD:
@@ -265,8 +266,7 @@ void CommonFG::link( H5L_type_t link_type, const char* curr_name, const char* ne
             break;
     } /* end switch */
 
-   if( ret_value < 0 )
-   {
+   if( ret_value < 0 ) {
       throwException("link", "creating link failed");
    }
 }
@@ -322,7 +322,7 @@ void CommonFG::unlink( const H5std_string& name ) const
 ///\note
 ///		Exercise care in moving groups as it is possible to render
 ///		data in a file inaccessible with Group::move. Please refer
-///		to the Group Interface in the HDF5 User's Guide at:
+///		to the Group Interface in the HDF5 User's Guide for details at:
 /// http://www.hdfgroup.org/HDF5/doc/UG/UG_frame09Groups.html
 // Programmer	Binh-Minh Ribler - 2000
 // Modification
@@ -425,16 +425,37 @@ void CommonFG::getObjinfo( const H5std_string& name, H5G_stat_t& statbuf ) const
 //--------------------------------------------------------------------------
 H5std_string CommonFG::getLinkval( const char* name, size_t size ) const
 {
-   char* value_C = new char[size+1];  // temporary C-string for C API
+    H5L_info_t linkinfo;
+    char *value_C;	// value in C string
+    size_t val_size = size;
+    H5std_string value = "";
+    herr_t ret_value;
 
-   herr_t ret_value = H5Lget_val( getLocId(), name, value_C, size, H5P_DEFAULT );
-   if( ret_value < 0 )
-   {
-      throwException("getLinkval", "H5Lget_val failed");
-   }
-   H5std_string value = H5std_string( value_C );
-   delete []value_C;
-   return( value );
+    // if user doesn't provide buffer size, determine it
+    if (size == 0)
+    {
+	ret_value = H5Lget_info(getLocId(), name, &linkinfo, H5P_DEFAULT);
+	if( ret_value < 0 )
+	{
+	    throwException("getLinkval", "H5Lget_info to find buffer size failed");
+	}
+	val_size = linkinfo.u.val_size;
+    }
+
+    // if link has value, retrieve the value, otherwise, return null string
+    if (val_size > 0)
+    {
+	value_C = new char[val_size+1];  // temporary C-string for C API
+
+	ret_value = H5Lget_val(getLocId(), name, value_C, val_size, H5P_DEFAULT);
+	if( ret_value < 0 )
+	{
+	    throwException("getLinkval", "H5Lget_val failed");
+	}
+	value = H5std_string(value_C);
+	delete []value_C;
+    }
+    return(value);
 }
 
 //--------------------------------------------------------------------------
@@ -522,48 +543,6 @@ void CommonFG::removeComment(const H5std_string& name) const
    removeComment (name.c_str());
 }
 
-//--------------------------------------------------------------------------
-// Function:	CommonFG::getComment
-///\brief	Retrieves comment for the specified object.
-///\param	name  - IN: Name of the object
-///\return	Comment string
-///\exception	H5::FileIException or H5::GroupIException
-// Programmer	Binh-Minh Ribler - May 2005
-//	2007: QAK modified to use H5O APIs; however the first parameter is
-//		no longer just file or group, this function should be moved
-//		to another class to accommodate attribute, dataset, and named
-//		datatype. - BMR
-//--------------------------------------------------------------------------
-H5std_string CommonFG::getComment (const H5std_string& name) const
-{
-   size_t bufsize = 256;        // anticipating the comment's length
-   hid_t loc_id = getLocId();   // temporary variable
-
-   // temporary C-string for the object's comment
-   char* comment_C = new char[bufsize+1];
-   ssize_t ret_value = H5Oget_comment_by_name(loc_id, name.c_str(), comment_C, bufsize, H5P_DEFAULT);
-
-   // if the actual length of the comment is longer than the anticipated
-   // value, then call H5Oget_comment_by_name again with the correct value
-   if ((size_t)ret_value > bufsize)
-   {
-	bufsize = ret_value;
-	delete []comment_C;
-	comment_C = new char[bufsize+1];
-	ret_value = H5Oget_comment_by_name(loc_id, name.c_str(), comment_C, bufsize, H5P_DEFAULT);
-   }
-
-   // if H5Oget_comment_by_name returns SUCCEED, return the string comment,
-   // otherwise, throw an exception
-   if( ret_value < 0 )
-   {
-      throwException("getComment", "H5Oget_comment_by_name failed");
-   }
-   H5std_string comment = H5std_string(comment_C);
-   delete []comment_C;
-   return (comment);
-}
-
 //--------------------------------------------------------------------------
 // Function:	CommonFG::getComment
 ///\brief	Retrieves comment for the specified object and its comment's
@@ -580,19 +559,37 @@ H5std_string CommonFG::getComment (const H5std_string& name) const
 //--------------------------------------------------------------------------
 H5std_string CommonFG::getComment( const char* name, size_t bufsize ) const
 {
-   // temporary C-string for the object's comment
-   char* comment_C = new char[bufsize+1];
+   // bufsize is default to 256
+   // temporary variable
+   hid_t loc_id = getLocId();   // temporary variable
 
-   herr_t ret_value = H5Oget_comment_by_name( getLocId(), name, comment_C, bufsize, H5P_DEFAULT );
+   // temporary C-string for the object's comment; bufsize already including
+   // null character
+   char* comment_C = new char[bufsize];
+   ssize_t ret_value = H5Oget_comment_by_name(loc_id, name, comment_C, bufsize, H5P_DEFAULT);
 
-   // if H5Oget_comment_by_name returns SUCCEED, return the string comment
+   // if the actual length of the comment is longer than bufsize and bufsize
+   // was the default value, i.e., not given by the user, then call
+   // H5Oget_comment_by_name again with the correct value.
+   // If the call to H5Oget_comment_by_name returned an error, skip this block
+   // and throw an exception below.
+   if (ret_value >= 0 && (size_t)ret_value > bufsize && bufsize == 256)
+   {
+	size_t new_size = ret_value;
+	delete []comment_C;
+	comment_C = new char[new_size];	// new_size including null terminator
+	ret_value = H5Oget_comment_by_name(loc_id, name, comment_C, new_size, H5P_DEFAULT);
+   }
+
+   // if H5Oget_comment_by_name returns SUCCEED, return the string comment,
+   // otherwise, throw an exception
    if( ret_value < 0 )
    {
       throwException("getComment", "H5Oget_comment_by_name failed");
    }
    H5std_string comment = H5std_string(comment_C);
    delete []comment_C;
-   return( comment );
+   return (comment);
 }
 
 //--------------------------------------------------------------------------
@@ -1062,10 +1059,27 @@ H5std_string CommonFG::getObjnameByIdx(hsize_t idx) const
 ///		each time the group is opened.
 // Programmer	Binh-Minh Ribler - January, 2003
 //--------------------------------------------------------------------------
+ssize_t CommonFG::getObjnameByIdx(hsize_t idx, char* name, size_t size) const
+{
+   ssize_t name_len = H5Lget_name_by_idx(getLocId(), ".", H5_INDEX_NAME, H5_ITER_INC, idx, name, size, H5P_DEFAULT);
+   if(name_len < 0)
+   {
+      throwException("getObjnameByIdx", "H5Lget_name_by_idx failed");
+   }
+   return (name_len);
+}
+
+//--------------------------------------------------------------------------
+// Function:	CommonFG::getObjnameByIdx
+///\brief	This is an overloaded member function, provided for convenience.
+///		It differs from the above function in that it takes an
+///		\c H5std_string for \a name.
+// Programmer	Binh-Minh Ribler - January, 2003
+//--------------------------------------------------------------------------
 ssize_t CommonFG::getObjnameByIdx(hsize_t idx, H5std_string& name, size_t size) const
 {
    char* name_C = new char[size];
-   ssize_t name_len = H5Lget_name_by_idx(getLocId(), ".", H5_INDEX_NAME, H5_ITER_INC, idx, name_C, size, H5P_DEFAULT);
+   ssize_t name_len = getObjnameByIdx(idx, name_C, size);
    if(name_len < 0)
    {
       throwException("getObjnameByIdx", "H5Lget_name_by_idx failed");
@@ -1099,7 +1113,35 @@ H5G_obj_t CommonFG::getObjTypeByIdx(hsize_t idx) const
 // Function:	CommonFG::getObjTypeByIdx
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function because it also provides
-///		the returned object type in text.
+///		the returned object type in text (char*)
+///\param	idx       - IN: Transient index of the object
+///\param	type_name - IN: Object type in text
+///\return	Object type
+///\exception	H5::FileIException or H5::GroupIException
+// Programmer	Binh-Minh Ribler - May, 2010
+//--------------------------------------------------------------------------
+H5G_obj_t CommonFG::getObjTypeByIdx(hsize_t idx, char* type_name) const
+{
+   H5G_obj_t obj_type = H5Gget_objtype_by_idx(getLocId(), idx);
+   switch (obj_type)
+   {
+	case H5G_LINK: HDstrcpy(type_name, "symbolic link"); break;
+	case H5G_GROUP: HDstrcpy(type_name, "group"); break;
+	case H5G_DATASET: HDstrcpy(type_name, "dataset"); break;
+	case H5G_TYPE: HDstrcpy(type_name, "datatype"); break;
+	case H5G_UNKNOWN:
+	default:
+   	{
+	   throwException("getObjTypeByIdx", "H5Gget_objtype_by_idx failed");
+	}
+   }
+   return (obj_type);
+}
+//--------------------------------------------------------------------------
+// Function:	CommonFG::getObjTypeByIdx
+///\brief	This is an overloaded member function, provided for convenience.
+///		It differs from the above function because it also provides
+///		the returned object type in text (H5std_string&)
 ///\param	idx       - IN: Transient index of the object
 ///\param	type_name - IN: Object type in text
 ///\return	Object type
