@@ -1,4 +1,13 @@
-! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
+!****h* root/fortran/test/tf.f90
+!
+! NAME
+!  tf.f90
+!
+! FUNCTION
+!  Contains subroutines which are needed in all the hdf5 fortran tests
+!
+! COPYRIGHT
+! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 !   Copyright by The HDF Group.                                               *
 !   Copyright by the Board of Trustees of the University of Illinois.         *
 !   All rights reserved.                                                      *
@@ -11,18 +20,35 @@
 !   is linked from the top-level documents page.  It can also be found at     *
 !   http://hdfgroup.org/HDF5/doc/Copyright.html.  If you do not have          *
 !   access to either file, you may request a copy from help@hdfgroup.org.     *
-! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
+! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 !
+! CONTAINS SUBROUTINES
+!  write_test_status, check, verify, verifyLogical, verifyString, h5_fixname_f,
+!  h5_cleanup_f, h5_exit_f, h5_env_nocleanup_f
 !
-! 
-!    This file contains subroutines which are used in
-!    all the hdf5 fortran tests
-!
-
+!*****
 
 !This definition is needed for Windows DLLs
 !DEC$if defined(BUILD_HDF5_DLL)
-!DEC$attributes dllexport :: write_test_status 
+!DEC$attributes dllexport :: verify_real
+!DEC$endif
+SUBROUTINE verify_real_kind_7(string,value,correct_value,total_error)
+  USE HDF5
+	  
+  INTEGER, PARAMETER :: real_kind_7 = SELECTED_REAL_KIND(Fortran_REAL_4) !should map to REAL*4 on most modern processors
+  CHARACTER(LEN=*) :: string
+  REAL(real_kind_7) :: value, correct_value
+  INTEGER :: total_error
+  IF (value .NE. correct_value) THEN
+     total_error=total_error+1
+     WRITE(*,*) "ERROR: INCORRECT REAL VALIDATION ", string
+  ENDIF
+  RETURN
+END SUBROUTINE verify_real_kind_7
+
+!This definition is needed for Windows DLLs
+!DEC$if defined(BUILD_HDF5_DLL)
+!DEC$attributes dllexport :: write_test_status
 !DEC$endif
   SUBROUTINE write_test_status( test_result, test_title, total_error)
 
@@ -31,7 +57,7 @@
     IMPLICIT NONE
 
     INTEGER, INTENT(IN) :: test_result  ! negative,  --skip --
-                                        ! 0       ,   passed 
+                                        ! 0       ,   passed
                                         ! positive,   failed
 
     CHARACTER(LEN=*), INTENT(IN) :: test_title ! Short description of test
@@ -51,9 +77,9 @@
     ELSE IF (test_result == -1) THEN
        error_string = skip
     ENDIF
-       
+
     WRITE(*, fmt = '(A, T72, A)') test_title, error_string
-    
+
     IF(test_result.GT.0) total_error = total_error + test_result
 
   END SUBROUTINE write_test_status
@@ -77,7 +103,7 @@ END SUBROUTINE check
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$attributes dllexport :: verify
 !DEC$endif
-SUBROUTINE VERIFY(string,value,correct_value,total_error)
+SUBROUTINE verify(string,value,correct_value,total_error)
   CHARACTER(LEN=*) :: string
   INTEGER :: value, correct_value, total_error
   IF (value .NE. correct_value) THEN
@@ -87,6 +113,25 @@ SUBROUTINE VERIFY(string,value,correct_value,total_error)
   RETURN
 END SUBROUTINE verify
 
+!This definition is needed for Windows DLLs
+!DEC$if defined(BUILD_HDF5_DLL)
+!DEC$attributes dllexport :: verify
+!DEC$endif
+SUBROUTINE verify_Fortran_INTEGER_4(string,value,correct_value,total_error)
+  USE HDF5
+  INTEGER, PARAMETER :: int_kind_8 = SELECTED_INT_KIND(Fortran_INTEGER_4)  ! should map to INTEGER*4 on most modern processors	
+  CHARACTER(LEN=*) :: string
+  INTEGER(int_kind_8) :: value, correct_value, total_error
+  IF (value .NE. correct_value) THEN
+     total_error=total_error+1
+     WRITE(*,*) "ERROR: INCORRECT VALIDATION ", string
+  ENDIF
+  RETURN
+END SUBROUTINE verify_Fortran_INTEGER_4
+
+
+
+
 !This definition is needed for Windows DLLs
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$attributes dllexport :: verifyLogical
@@ -106,32 +151,32 @@ END SUBROUTINE verifyLogical
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$attributes dllexport :: verifyString
 !DEC$endif
-SUBROUTINE verifyString(string, value,correct_value,total_error)
-  CHARACTER(LEN=*) :: string
-  CHARACTER(LEN=*) :: value, correct_value
+SUBROUTINE verifystring(string, value,correct_value,total_error)
+  CHARACTER*(*) :: string
+  CHARACTER*(*) :: value, correct_value
   INTEGER :: total_error
   IF (TRIM(value) .NE. TRIM(correct_value)) THEN
      total_error = total_error + 1
      WRITE(*,*) "ERROR: INCORRECT VALIDATION ", string
   ENDIF
   RETURN
-END SUBROUTINE verifyString
+END SUBROUTINE verifystring
 
 
 !----------------------------------------------------------------------
-! Name:		h5_fixname_f 
+! Name:		h5_fixname_f
 !
 ! Purpose:	Create a file name from the a file base name.
 !               It is a fortran counterpart for the h5_fixname in ../../test/h5test.c
 !
-! Inputs:  
-!		base_name	- base name of the file 
-!               fapl		- file access property list 
-! Outputs:  
+! Inputs:
+!		base_name	- base name of the file
+!               fapl		- file access property list
+! Outputs:
 !		full_name	- full file name
-!		hdferr:		- error code		
+!		hdferr:		- error code
 !				 	Success:  0
-!				 	Failure: -1   
+!				 	Failure: -1
 !
 ! Programmer:	Elena Pourmal
 !		September 13, 2002
@@ -144,13 +189,13 @@ SUBROUTINE h5_fixname_f(base_name, full_name, fapl, hdferr)
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$attributes dllexport :: h5_fixname_f
 !DEC$endif
-  USE H5GLOBAL           
+  USE H5GLOBAL
   IMPLICIT NONE
-  CHARACTER(LEN=*), INTENT(IN) :: base_name   ! base name 
-  CHARACTER(LEN=*), INTENT(IN) :: full_name   ! full name 
-  INTEGER, INTENT(OUT) :: hdferr         ! Error code 
+  CHARACTER(LEN=*), INTENT(IN) :: base_name   ! base name
+  CHARACTER(LEN=*), INTENT(IN) :: full_name   ! full name
+  INTEGER, INTENT(OUT) :: hdferr         ! Error code
   INTEGER(HID_T), INTENT(IN) :: fapl ! file access property list
-  
+
   INTEGER(SIZE_T) :: base_namelen ! Length of the base name character string
   INTEGER(SIZE_T) :: full_namelen ! Length of the full name character string
 !            INTEGER(HID_T) :: fapl_default
@@ -162,8 +207,8 @@ SUBROUTINE h5_fixname_f(base_name, full_name, fapl, hdferr)
        !DEC$ IF DEFINED(HDF5F90_WINDOWS)
        !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5_FIXNAME_C':: h5_fixname_c
        !DEC$ ENDIF
-       !DEC$ATTRIBUTES reference :: base_name 
-       !DEC$ATTRIBUTES reference :: full_name 
+       !DEC$ATTRIBUTES reference :: base_name
+       !DEC$ATTRIBUTES reference :: full_name
        CHARACTER(LEN=*), INTENT(IN) :: base_name
        INTEGER(SIZE_T) :: base_namelen
        INTEGER(HID_T), INTENT(IN) :: fapl
@@ -171,27 +216,27 @@ SUBROUTINE h5_fixname_f(base_name, full_name, fapl, hdferr)
        INTEGER(SIZE_T) :: full_namelen
      END FUNCTION h5_fixname_c
   END INTERFACE
-  
+
   base_namelen = LEN(base_name)
   full_namelen = LEN(full_name)
   hdferr = h5_fixname_c(base_name, base_namelen, fapl, &
-       full_name, full_namelen) 
-  
+       full_name, full_namelen)
+
 END SUBROUTINE h5_fixname_f
-          
+
 !----------------------------------------------------------------------
-! Name:		h5_cleanup_f 
+! Name:		h5_cleanup_f
 !
 ! Purpose:	Cleanups tests files
 !               It is a fortran counterpart for the h5_cleanup in ../../test/h5test.c
 !
-! Inputs:  
-!		base_name	- base name of the file 
-!               fapl		- file access property list 
-! Outputs:  
-!		hdferr:		- error code		
+! Inputs:
+!		base_name	- base name of the file
+!               fapl		- file access property list
+! Outputs:
+!		hdferr:		- error code
 !				 	Success:  0
-!				 	Failure: -1   
+!				 	Failure: -1
 !
 ! Programmer:	Elena Pourmal
 !		September 19, 2002
@@ -204,34 +249,34 @@ SUBROUTINE h5_cleanup_f(base_name, fapl, hdferr)
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$attributes dllexport :: h5_cleanup_f
 !DEC$endif
-  USE H5GLOBAL           
+  USE H5GLOBAL
   IMPLICIT NONE
-  CHARACTER(LEN=*), INTENT(IN) :: base_name   ! base name 
-  INTEGER, INTENT(OUT) :: hdferr         ! Error code 
+  CHARACTER(LEN=*), INTENT(IN) :: base_name   ! base name
+  INTEGER, INTENT(OUT) :: hdferr         ! Error code
   INTEGER(HID_T), INTENT(IN) :: fapl ! file access property list
-  
+
   INTEGER(SIZE_T) :: base_namelen ! Length of the base name character string
-  
+
   INTERFACE
      INTEGER FUNCTION h5_cleanup_c(base_name, base_namelen, fapl)
        USE H5GLOBAL
        !DEC$ IF DEFINED(HDF5F90_WINDOWS)
        !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5_CLEANUP_C':: h5_cleanup_c
        !DEC$ ENDIF
-       !DEC$ATTRIBUTES reference :: base_name 
+       !DEC$ATTRIBUTES reference :: base_name
        CHARACTER(LEN=*), INTENT(IN) :: base_name
        INTEGER(SIZE_T) :: base_namelen
        INTEGER(HID_T), INTENT(IN) :: fapl
      END FUNCTION h5_cleanup_c
   END INTERFACE
-  
+
   base_namelen = LEN(base_name)
   hdferr = h5_cleanup_c(base_name, base_namelen, fapl)
-  
+
 END SUBROUTINE h5_cleanup_f
 
 !----------------------------------------------------------------------
-! Name:		h5_exit_f 
+! Name:		h5_exit_f
 !
 ! Purpose:	Exit application
 !               It is a fortran counterpart for the standard C 'exit()' routine
@@ -239,10 +284,10 @@ END SUBROUTINE h5_cleanup_f
 ! 		UNIX supports a very small range such as 1 byte.
 !		Therefore, exit(256) may end up as exit(0).
 !
-! Inputs:  
+! Inputs:
 !		status	- Status to return from application
 !
-! Outputs:  
+! Outputs:
 !		none
 !
 ! Programmer:	Quincey Koziol
@@ -258,7 +303,7 @@ SUBROUTINE h5_exit_f(status)
 !DEC$endif
   IMPLICIT NONE
   INTEGER, INTENT(IN) :: status         ! Return code
-  
+
   INTERFACE
      SUBROUTINE h5_exit_c(status)
        !DEC$ IF DEFINED(HDF5F90_WINDOWS)
@@ -267,18 +312,18 @@ SUBROUTINE h5_exit_f(status)
        INTEGER, INTENT(IN) :: status
      END SUBROUTINE h5_exit_c
   END INTERFACE
-  
+
   CALL h5_exit_c(status)
-  
-END SUBROUTINE h5_exit_f 
+
+END SUBROUTINE h5_exit_f
 
 !----------------------------------------------------------------------
-! Name:		h5_env_nocleanup_f 
+! Name:		h5_env_nocleanup_f
 !
-! Purpose:	Uses the HDF5_NOCLEANUP environment variable in Fortran 
+! Purpose:	Uses the HDF5_NOCLEANUP environment variable in Fortran
 !               tests to determine if the output files should be removed
 !
-! Inputs:  
+! Inputs:
 !
 ! Outputs:      HDF5_NOCLEANUP:  .true. - don't remove test files
 !		                .false. - remove test files
@@ -305,13 +350,13 @@ SUBROUTINE h5_env_nocleanup_f(HDF5_NOCLEANUP)
        INTEGER :: status
      END SUBROUTINE h5_env_nocleanup_c
   END INTERFACE
-  
+
   CALL h5_env_nocleanup_c(status)
 
   HDF5_NOCLEANUP = .FALSE.
   IF(status.EQ.1)THEN
      HDF5_NOCLEANUP = .TRUE.
   ENDIF
-  
+
 END SUBROUTINE h5_env_nocleanup_f
 
