@@ -27,7 +27,7 @@
 #endif
 
 #ifdef H5_HAVE_WINSOCK_H
-#	include <winsock2.h>
+#  include <winsock2.h>
 #endif /* H5_HAVE_WINSOCK_H */
 
 /* The different types of timers we can have */
@@ -52,8 +52,8 @@ typedef enum timer_type_ {
 
 /* Miscellaneous identifiers */
 enum {
-    START,          /* Start a specified timer              */
-    STOP            /* Stop a specified timer               */
+    TSTART,          /* Start a specified timer              */
+    TSTOP            /* Stop a specified timer               */
 };
 
 /* The performance time structure */
