@@ -22,13 +22,15 @@
 /* Headers */
 /***********/
 #include "H5private.h"		/* Generic Functions			*/
+#include "H5ACprivate.h"	/* Metadata cache			*/
 #include "H5Dprivate.h"		/* Datasets				*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5FLprivate.h"	/* Free lists                           */
 #include "H5Lprivate.h"		/* Links		  		*/
+#include "H5MMprivate.h"        /* Memory management                    */
 #include "H5Pprivate.h"		/* Property lists			*/
 #include "H5Tprivate.h"		/* Datatypes				*/
-#include "H5SLprivate.h"    /* Skip lists                           */
+#include "H5SLprivate.h"        /* Skip lists                           */
 
 
 /****************/
@@ -102,7 +104,7 @@ H5_init_library(void)
 {
     herr_t ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI(H5_init_library, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
     /*
      * Make sure the package information is updated.
      */
@@ -183,7 +185,7 @@ H5_init_library(void)
     H5_debug_mask(HDgetenv("HDF5_DEBUG"));
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value)
+    FUNC_LEAVE_NOAPI(ret_value);
 } /* end H5_init_library() */
 
 
@@ -308,6 +310,16 @@ H5_term_library(void)
     } /* end if */
 #endif
 
+    /* Free open debugging streams */
+    while(H5_debug_g.open_stream) {
+        H5_debug_open_stream_t  *tmp_open_stream;
+
+        tmp_open_stream = H5_debug_g.open_stream;
+        (void)HDfclose(H5_debug_g.open_stream->stream);
+        H5_debug_g.open_stream = H5_debug_g.open_stream->next;
+        (void)H5MM_free(tmp_open_stream);
+    } /* end while */
+
     /* Mark library as closed */
     H5_INIT_GLOBAL = FALSE;
 done:
@@ -348,7 +360,7 @@ H5dont_atexit(void)
 {
     herr_t ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_API_NOINIT_NOFS(H5dont_atexit)
+    FUNC_ENTER_API_NOINIT_NOERR_NOFS
     H5TRACE0("e","");
 
     if(H5_dont_atexit_g)
@@ -387,7 +399,7 @@ H5garbage_collect(void)
 {
     herr_t                  ret_value = SUCCEED;
 
-    FUNC_ENTER_API(H5garbage_collect, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE0("e","");
 
     /* Call the garbage collection routines in the library */
@@ -439,7 +451,7 @@ H5set_free_list_limits(int reg_global_lim, int reg_list_lim, int arr_global_lim,
 {
     herr_t                  ret_value = SUCCEED;
 
-    FUNC_ENTER_API(H5set_free_list_limits, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE6("e", "IsIsIsIsIsIs", reg_global_lim, reg_list_lim, arr_global_lim,
              arr_list_lim, blk_global_lim, blk_list_lim);
 
@@ -531,9 +543,21 @@ H5_debug_mask(const char *s)
 	    }
 
 	} else if (HDisdigit(*s)) {
-	    int fd = (int)HDstrtol (s, &rest, 0);
-	    if ((stream=HDfdopen(fd, "w"))!=NULL)
-	        (void)HDsetvbuf (stream, NULL, _IOLBF, (size_t)0);
+	    int fd = (int)HDstrtol(s, &rest, 0);
+	    H5_debug_open_stream_t *open_stream;
+
+	    if((stream = HDfdopen(fd, "w")) != NULL) {
+	        (void)HDsetvbuf(stream, NULL, _IOLBF, (size_t)0);
+
+	        if(NULL == (open_stream = (H5_debug_open_stream_t *)H5MM_malloc(sizeof(H5_debug_open_stream_t)))) {
+                    (void)HDfclose(stream);
+                    return;
+                } /* end if */
+
+                open_stream->stream = stream;
+                open_stream->next = H5_debug_g.open_stream;
+                H5_debug_g.open_stream = open_stream;
+            } /* end if */
 	    s = rest;
 	} else {
 	    s++;
@@ -570,7 +594,7 @@ H5get_libversion(unsigned *majnum, unsigned *minnum, unsigned *relnum)
 {
     herr_t                  ret_value = SUCCEED;
 
-    FUNC_ENTER_API(H5get_libversion, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "*Iu*Iu*Iu", majnum, minnum, relnum);
 
     /* Set the version information */
@@ -621,10 +645,11 @@ H5check_version(unsigned majnum, unsigned minnum, unsigned relnum)
     char	lib_str[256];
     char	substr[] = H5_VERS_SUBRELEASE;
     static int	checked = 0;            /* If we've already checked the version info */
-    static int	disable_version_check = 0;      /* Set if the version check should be disabled */
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    static unsigned int	disable_version_check = 0;      /* Set if the version check should be disabled */
+    static const char *version_mismatch_warning = VERSION_MISMATCH_WARNING;
+    herr_t      ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API_NOINIT_NOFS(H5check_version)
+    FUNC_ENTER_API_NOINIT_NOERR_NOFS
     H5TRACE3("e", "IuIuIu", majnum, minnum, relnum);
 
     /* Don't check again, if we already have */
@@ -637,39 +662,44 @@ H5check_version(unsigned majnum, unsigned minnum, unsigned relnum)
         s = HDgetenv ("HDF5_DISABLE_VERSION_CHECK");
 
         if (s && HDisdigit(*s))
-            disable_version_check = (int)HDstrtol (s, NULL, 0);
+            disable_version_check = (unsigned int)HDstrtol (s, NULL, 0);
     }
 
     if (H5_VERS_MAJOR!=majnum || H5_VERS_MINOR!=minnum ||
             H5_VERS_RELEASE!=relnum) {
         switch (disable_version_check) {
 	case 0:
-            HDfputs (VERSION_MISMATCH_WARNING
+	    HDfprintf(stderr, "%s%s", version_mismatch_warning,
 		     "You can, at your own risk, disable this warning by setting the environment\n"
 		     "variable 'HDF5_DISABLE_VERSION_CHECK' to a value of '1'.\n"
-		     "Setting it to 2 will suppress the warning messages totally.\n",
-		     stderr);
+		     "Setting it to 2 or higher will suppress the warning messages totally.\n");
 	    /* Mention the versions we are referring to */
 	    HDfprintf (stderr, "Headers are %u.%u.%u, library is %u.%u.%u\n",
 		     majnum, minnum, relnum,
 		     (unsigned)H5_VERS_MAJOR, (unsigned)H5_VERS_MINOR, (unsigned)H5_VERS_RELEASE);
+	    /* Show library settings if available */
+	    HDfprintf (stderr, "%s", H5libhdf5_settings);
 
 	    /* Bail out now. */
 	    HDfputs ("Bye...\n", stderr);
 	    HDabort ();
-	case 2:
-	    /* continue silently */
-	    break;
-	default:
+	case 1:
 	    /* continue with a warning */
-            HDfprintf (stderr, VERSION_MISMATCH_WARNING
-                     "'HDF5_DISABLE_VERSION_CHECK' "
+	    /* Note that the warning message is embedded in the format string.*/
+            HDfprintf (stderr,
+                     "%s'HDF5_DISABLE_VERSION_CHECK' "
                      "environment variable is set to %d, application will\n"
-                     "continue at your own risk.\n", disable_version_check);
+                     "continue at your own risk.\n",
+		     version_mismatch_warning, disable_version_check);
 	    /* Mention the versions we are referring to */
 	    HDfprintf (stderr, "Headers are %u.%u.%u, library is %u.%u.%u\n",
 		     majnum, minnum, relnum,
 		     (unsigned)H5_VERS_MAJOR, (unsigned)H5_VERS_MINOR, (unsigned)H5_VERS_RELEASE);
+	    /* Show library settings if available */
+	    HDfprintf (stderr, "%s", H5libhdf5_settings);
+	    break;
+	default:
+	    /* 2 or higer: continue silently */
 	    break;
         } /* end switch */
 
@@ -686,7 +716,7 @@ H5check_version(unsigned majnum, unsigned minnum, unsigned relnum)
 	 */
 	sprintf(lib_str, "HDF5 library version: %d.%d.%d",
 	    H5_VERS_MAJOR, H5_VERS_MINOR, H5_VERS_RELEASE);
-	if (*substr){
+	if(*substr) {
 	    HDstrcat(lib_str, "-");
 	    HDstrncat(lib_str, substr, (sizeof(lib_str) - HDstrlen(lib_str)) - 1);
 	} /* end if */
@@ -733,7 +763,7 @@ H5open(void)
 {
     herr_t ret_value=SUCCEED;   /* Return value */
 
-    FUNC_ENTER_API_NOCLEAR(H5open, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     H5TRACE0("e","");
     /* all work is done by FUNC_ENTER() */
 done:
@@ -763,7 +793,7 @@ H5close(void)
      * whole library just to release it all right away.  It is safe to call
      * this function for an uninitialized library.
      */
-    FUNC_ENTER_API_NOINIT_NOFS(H5close)
+    FUNC_ENTER_API_NOINIT_NOERR_NOFS
     H5TRACE0("e","");
 
     H5_term_library();
