@@ -113,7 +113,7 @@ H5A_init(void)
 {
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5A_init, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
     /* FUNC_ENTER() does all the work */
 
 done:
@@ -138,7 +138,7 @@ H5A_init_interface(void)
 {
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5A_init_interface)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /*
      * Create attribute ID type.
@@ -172,7 +172,7 @@ H5A_term_interface(void)
 {
     int	n = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_term_interface)
+    FUNC_ENTER_NOAPI_NOINIT
 
     if(H5_interface_initialize_g) {
 	if((n = H5I_nmembers(H5I_ATTR))>0) {
@@ -225,7 +225,7 @@ H5Acreate2(hid_t loc_id, const char *attr_name, hid_t type_id, hid_t space_id,
     H5S_t		*space;                 /* Dataspace to use for attribute */
     hid_t		ret_value;              /* Return value */
 
-    FUNC_ENTER_API(H5Acreate2, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE6("i", "i*siiii", loc_id, attr_name, type_id, space_id, acpl_id, aapl_id);
 
     /* check arguments */
@@ -296,7 +296,7 @@ H5Acreate_by_name(hid_t loc_id, const char *obj_name, const char *attr_name,
     H5S_t		*space;                 /* Dataspace to use for attribute */
     hid_t		ret_value;              /* Return value */
 
-    FUNC_ENTER_API(H5Acreate_by_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE8("i", "i*s*siiiii", loc_id, obj_name, attr_name, type_id, space_id,
              acpl_id, aapl_id, lapl_id);
 
@@ -369,7 +369,7 @@ H5A_create(const H5G_loc_t *loc, const char *name, const H5T_t *type,
     htri_t      tri_ret;        /* htri_t return value */
     hid_t	ret_value;      /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5A_create)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(loc);
@@ -534,7 +534,7 @@ H5Aopen(hid_t loc_id, const char *attr_name, hid_t UNUSED aapl_id)
     H5A_t               *attr = NULL;   /* Attribute opened */
     hid_t		ret_value;
 
-    FUNC_ENTER_API(H5Aopen, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("i", "i*si", loc_id, attr_name, aapl_id);
 
     /* check arguments */
@@ -596,7 +596,7 @@ H5Aopen_by_name(hid_t loc_id, const char *obj_name, const char *attr_name,
     H5A_t               *attr = NULL;   /* Attribute opened */
     hid_t		ret_value;
 
-    FUNC_ENTER_API(H5Aopen_by_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE5("i", "i*s*sii", loc_id, obj_name, attr_name, aapl_id, lapl_id);
 
     /* check arguments */
@@ -664,7 +664,7 @@ H5Aopen_by_idx(hid_t loc_id, const char *obj_name, H5_index_t idx_type,
     H5G_loc_t	loc;	        /* Object location */
     hid_t	ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Aopen_by_idx, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE7("i", "i*sIiIohii", loc_id, obj_name, idx_type, order, n, aapl_id,
              lapl_id);
 
@@ -726,7 +726,7 @@ H5A_open_common(const H5G_loc_t *loc, H5A_t *attr)
 {
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5A_open_common)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(loc);
@@ -783,7 +783,7 @@ H5A_open_by_idx(const H5G_loc_t *loc, const char *obj_name, H5_index_t idx_type,
     H5A_t       *attr = NULL;           /* Attribute from object header */
     H5A_t       *ret_value;             /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5A_open_by_idx)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(loc);
@@ -847,7 +847,7 @@ H5A_open_by_name(const H5G_loc_t *loc, const char *obj_name, const char *attr_na
     H5A_t       *attr = NULL;           /* Attribute from object header */
     H5A_t       *ret_value;             /* Return value */
 
-    FUNC_ENTER_NOAPI(H5A_open_by_name, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     /* check args */
     HDassert(loc);
@@ -912,7 +912,7 @@ H5Awrite(hid_t attr_id, hid_t dtype_id, const void *buf)
     H5T_t *mem_type;            /* Memory datatype */
     herr_t ret_value;           /* Return value */
 
-    FUNC_ENTER_API(H5Awrite, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ii*x", attr_id, dtype_id, buf);
 
     /* check arguments */
@@ -963,7 +963,7 @@ H5A_write(H5A_t *attr, const H5T_t *mem_type, const void *buf, hid_t dxpl_id)
     size_t		buf_size;		/* desired buffer size	*/
     herr_t		ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5A_write)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(attr);
     HDassert(mem_type);
@@ -1069,7 +1069,7 @@ H5Aread(hid_t attr_id, hid_t dtype_id, void *buf)
     H5T_t *mem_type;            /* Memory datatype */
     herr_t ret_value;           /* Return value */
 
-    FUNC_ENTER_API(H5Aread, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ii*x", attr_id, dtype_id, buf);
 
     /* check arguments */
@@ -1119,7 +1119,7 @@ H5A_read(const H5A_t *attr, const H5T_t *mem_type, void *buf, hid_t dxpl_id)
     size_t		buf_size;		/* desired buffer size	*/
     herr_t		ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5A_read)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(attr);
     HDassert(mem_type);
@@ -1215,7 +1215,7 @@ H5Aget_space(hid_t attr_id)
     H5S_t	*ds = NULL;             /* Copy of dataspace for attribute */
     hid_t	ret_value;
 
-    FUNC_ENTER_API(H5Aget_space, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", attr_id);
 
     /* check arguments */
@@ -1261,7 +1261,7 @@ H5Aget_type(hid_t attr_id)
     H5T_t	*dt = NULL;     /* Copy of attribute's datatype */
     hid_t	 ret_value;     /* Return value */
 
-    FUNC_ENTER_API(H5Aget_type, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", attr_id);
 
     /* check arguments */
@@ -1330,7 +1330,7 @@ H5Aget_create_plist(hid_t attr_id)
     H5P_genplist_t      *new_plist;          /* ACPL to return */
     hid_t		ret_value;
 
-    FUNC_ENTER_API(H5Aget_create_plist, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", attr_id);
 
     HDassert(H5P_LST_ATTRIBUTE_CREATE_g != -1);
@@ -1385,7 +1385,7 @@ H5Aget_name(hid_t attr_id, size_t buf_size, char *buf)
     H5A_t		*my_attr;               /* Attribute object for ID */
     ssize_t		ret_value;
 
-    FUNC_ENTER_API(H5Aget_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("Zs", "iz*s", attr_id, buf_size, buf);
 
     /* check arguments */
@@ -1425,7 +1425,7 @@ H5A_get_name(H5A_t *attr, size_t buf_size, char *buf)
     size_t              copy_len, nbytes;
     ssize_t		ret_value;
 
-    FUNC_ENTER_NOAPI(H5A_get_name, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* get the real attribute length */
     nbytes = HDstrlen(attr->shared->name);
@@ -1476,7 +1476,7 @@ H5Aget_name_by_idx(hid_t loc_id, const char *obj_name, H5_index_t idx_type,
     H5A_t	*attr = NULL;   /* Attribute object for name */
     ssize_t	ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Aget_name_by_idx, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE8("Zs", "i*sIiIohxzi", loc_id, obj_name, idx_type, order, n, name, size,
              lapl_id);
 
@@ -1543,7 +1543,7 @@ H5Aget_storage_size(hid_t attr_id)
     H5A_t	*attr;               /* Attribute object for ID */
     hsize_t	ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Aget_storage_size, 0)
+    FUNC_ENTER_API(0)
     H5TRACE1("h", "i", attr_id);
 
     /* Check args */
@@ -1577,7 +1577,7 @@ H5Aget_info(hid_t attr_id, H5A_info_t *ainfo)
     H5A_t	*attr;                  /* Attribute object for name */
     herr_t	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API(H5Aget_info, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*x", attr_id, ainfo);
 
     /* Check args */
@@ -1614,7 +1614,7 @@ H5Aget_info_by_name(hid_t loc_id, const char *obj_name, const char *attr_name,
     H5A_t	*attr = NULL;           /* Attribute object for name */
     herr_t	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API(H5Aget_info_by_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE5("e", "i*s*s*xi", loc_id, obj_name, attr_name, ainfo, lapl_id);
 
     /* Check args */
@@ -1673,7 +1673,7 @@ H5Aget_info_by_idx(hid_t loc_id, const char *obj_name, H5_index_t idx_type,
     H5A_t	*attr = NULL;           /* Attribute object for name */
     herr_t	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API(H5Aget_info_by_idx, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE7("e", "i*sIiIoh*xi", loc_id, obj_name, idx_type, order, n, ainfo,
              lapl_id);
 
@@ -1731,7 +1731,7 @@ H5A_get_info(const H5A_t *attr, H5A_info_t *ainfo)
 {
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI(H5A_get_info, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Check args */
     HDassert(attr);
@@ -1773,7 +1773,7 @@ H5Arename(hid_t loc_id, const char *old_name, const char *new_name)
     H5G_loc_t	loc;	                /* Object location */
     herr_t	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API(H5Arename, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "i*s*s", loc_id, old_name, new_name);
 
     /* check arguments */
@@ -1819,7 +1819,7 @@ H5Arename_by_name(hid_t loc_id, const char *obj_name, const char *old_attr_name,
     hbool_t     loc_found = FALSE;      /* Entry at 'obj_name' found */
     herr_t	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API(H5Arename_by_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE5("e", "i*s*s*si", loc_id, obj_name, old_attr_name, new_attr_name,
              lapl_id);
 
@@ -1916,7 +1916,7 @@ H5Aiterate2(hid_t loc_id, H5_index_t idx_type, H5_iter_order_t order,
     hsize_t	last_attr;      /* Index of last attribute examined */
     herr_t	ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Aiterate2, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE6("e", "iIiIo*hx*x", loc_id, idx_type, order, idx, op, op_data);
 
     /* check arguments */
@@ -2004,7 +2004,7 @@ H5Aiterate_by_name(hid_t loc_id, const char *obj_name, H5_index_t idx_type,
     hsize_t	last_attr;      /* Index of last attribute examined */
     herr_t	ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Aiterate_by_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE8("e", "i*sIiIo*hx*xi", loc_id, obj_name, idx_type, order, idx, op,
              op_data, lapl_id);
 
@@ -2085,7 +2085,7 @@ H5Adelete(hid_t loc_id, const char *name)
     H5G_loc_t	loc;		        /* Object location */
     herr_t	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API(H5Adelete, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*s", loc_id, name);
 
     /* check arguments */
@@ -2132,7 +2132,7 @@ H5Adelete_by_name(hid_t loc_id, const char *obj_name, const char *attr_name,
     hbool_t     loc_found = FALSE;      /* Entry at 'obj_name' found */
     herr_t	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API(H5Adelete_by_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "i*s*si", loc_id, obj_name, attr_name, lapl_id);
 
     /* check arguments */
@@ -2208,7 +2208,7 @@ H5Adelete_by_idx(hid_t loc_id, const char *obj_name, H5_index_t idx_type,
     hbool_t     loc_found = FALSE;      /* Entry at 'obj_name' found */
     herr_t	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_API(H5Adelete_by_idx, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE6("e", "i*sIiIohi", loc_id, obj_name, idx_type, order, n, lapl_id);
 
     /* check arguments */
@@ -2271,7 +2271,7 @@ H5Aclose(hid_t attr_id)
 {
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_API(H5Aclose, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", attr_id);
 
     /* check arguments */
@@ -2312,7 +2312,7 @@ H5A_copy(H5A_t *_new_attr, const H5A_t *old_attr)
     hbool_t     allocated_attr = FALSE;   /* Whether the attribute was allocated */
     H5A_t	*ret_value = NULL;        /* Return value */
 
-    FUNC_ENTER_NOAPI(H5A_copy, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     /* check args */
     HDassert(old_attr);
@@ -2374,7 +2374,7 @@ H5A_free(H5A_t *attr)
 {
     herr_t ret_value = SUCCEED;           /* Return value */
 
-    FUNC_ENTER_NOAPI(H5A_free, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     HDassert(attr);
 
@@ -2422,7 +2422,7 @@ H5A_close(H5A_t *attr)
 {
     herr_t ret_value = SUCCEED;           /* Return value */
 
-    FUNC_ENTER_NOAPI(H5A_close, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     HDassert(attr);
     HDassert(attr->shared);
@@ -2478,7 +2478,7 @@ H5A_oloc(H5A_t *attr)
 {
     H5O_loc_t *ret_value;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5A_oloc, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     HDassert(attr);
 
@@ -2510,7 +2510,7 @@ H5A_nameof(H5A_t *attr)
 {
     H5G_name_t *ret_value;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5A_nameof, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     HDassert(attr);
 
@@ -2521,6 +2521,36 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5A_nameof() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:    H5A_type
+ *
+ * Purpose:     Return the datatype for an attribute.
+ *
+ * Return:      Success:        Ptr to entry
+ *              Failure:        NULL
+ *
+ * Programmer:  Neil Fortner
+ *              Friday, November  11, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+H5T_t *
+H5A_type(const H5A_t *attr)
+{
+    H5T_t *ret_value;   /* Return value */
+
+    FUNC_ENTER_NOAPI(NULL)
+
+    HDassert(attr);
+
+    /* Set return value */
+    ret_value = attr->shared->dt;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5A_type() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5Aexists
@@ -2542,7 +2572,7 @@ H5Aexists(hid_t obj_id, const char *attr_name)
     H5G_loc_t   loc;                    /* Object location */
     htri_t	ret_value;              /* Return value */
 
-    FUNC_ENTER_API(H5Aexists, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("t", "i*s", obj_id, attr_name);
 
     /* check arguments */
@@ -2586,7 +2616,7 @@ H5Aexists_by_name(hid_t loc_id, const char *obj_name, const char *attr_name,
     hbool_t     loc_found = FALSE;      /* Entry at 'obj_name' found */
     htri_t	ret_value;              /* Return value */
 
-    FUNC_ENTER_API(H5Aexists_by_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("t", "i*s*si", loc_id, obj_name, attr_name, lapl_id);
 
     /* check arguments */
