@@ -81,21 +81,20 @@ typedef struct H5A_fh_ud_cmp_t {
 /* v2 B-tree driver callbacks for 'creation order' index */
 static herr_t H5A_dense_btree2_corder_store(void *native, const void *udata);
 static herr_t H5A_dense_btree2_corder_compare(const void *rec1, const void *rec2);
-static herr_t H5A_dense_btree2_corder_encode(const H5F_t *f, uint8_t *raw,
-    const void *native);
-static herr_t H5A_dense_btree2_corder_decode(const H5F_t *f, const uint8_t *raw,
-    void *native);
+static herr_t H5A_dense_btree2_corder_encode(uint8_t *raw, const void *native,
+    void *ctx);
+static herr_t H5A_dense_btree2_corder_decode(const uint8_t *raw, void *native,
+    void *ctx);
 static herr_t H5A_dense_btree2_corder_debug(FILE *stream, const H5F_t *f, hid_t dxpl_id,
     int indent, int fwidth, const void *record, const void *_udata);
 
 /* v2 B-tree driver callbacks for 'name' index */
 static herr_t H5A_dense_btree2_name_store(void *native, const void *udata);
-static herr_t H5A_dense_btree2_name_retrieve(void *udata, const void *native);
 static herr_t H5A_dense_btree2_name_compare(const void *rec1, const void *rec2);
-static herr_t H5A_dense_btree2_name_encode(const H5F_t *f, uint8_t *raw,
-    const void *native);
-static herr_t H5A_dense_btree2_name_decode(const H5F_t *f, const uint8_t *raw,
-    void *native);
+static herr_t H5A_dense_btree2_name_encode(uint8_t *raw, const void *native,
+    void *ctx);
+static herr_t H5A_dense_btree2_name_decode(const uint8_t *raw, void *native,
+    void *ctx);
 static herr_t H5A_dense_btree2_name_debug(FILE *stream, const H5F_t *f, hid_t dxpl_id,
     int indent, int fwidth, const void *record, const void *_udata);
 
@@ -108,24 +107,34 @@ static herr_t H5A_dense_fh_name_cmp(const void *obj, size_t obj_len, void *op_da
 /*********************/
 /* v2 B-tree class for indexing 'name' field of attributes */
 const H5B2_class_t H5A_BT2_NAME[1]={{  /* B-tree class information */
-    H5B2_ATTR_DENSE_NAME_ID,                /* Type of B-tree */
+    H5B2_ATTR_DENSE_NAME_ID,           /* Type of B-tree */
+    "H5B2_ATTR_DENSE_NAME_ID",         /* Name of B-tree class */
     sizeof(H5A_dense_bt2_name_rec_t),  /* Size of native record */
+    NULL,                              /* Create client callback context */
+    NULL,                              /* Destroy client callback context */
     H5A_dense_btree2_name_store,       /* Record storage callback */
     H5A_dense_btree2_name_compare,     /* Record comparison callback */
     H5A_dense_btree2_name_encode,      /* Record encoding callback */
     H5A_dense_btree2_name_decode,      /* Record decoding callback */
-    H5A_dense_btree2_name_debug        /* Record debugging callback */
+    H5A_dense_btree2_name_debug,       /* Record debugging callback */
+    NULL,                              /* Create debugging context */
+    NULL                               /* Destroy debugging context */
 }};
 
 /* v2 B-tree class for indexing 'creation order' field of attributes */
-const H5B2_class_t H5A_BT2_CORDER[1]={{  /* B-tree class information */
-    H5B2_ATTR_DENSE_CORDER_ID,                /* Type of B-tree */
+const H5B2_class_t H5A_BT2_CORDER[1]={{ /* B-tree class information */
+    H5B2_ATTR_DENSE_CORDER_ID,         /* Type of B-tree */
+    "H5B2_ATTR_DENSE_CORDER_ID",       /* Name of B-tree class */
     sizeof(H5A_dense_bt2_corder_rec_t),/* Size of native record */
+    NULL,                              /* Create client callback context */
+    NULL,                              /* Destroy client callback context */
     H5A_dense_btree2_corder_store,     /* Record storage callback */
     H5A_dense_btree2_corder_compare,   /* Record comparison callback */
     H5A_dense_btree2_corder_encode,    /* Record encoding callback */
     H5A_dense_btree2_corder_decode,    /* Record decoding callback */
-    H5A_dense_btree2_corder_debug      /* Record debugging callback */
+    H5A_dense_btree2_corder_debug,     /* Record debugging callback */
+    NULL,                              /* Create debugging context */
+    NULL                               /* Destroy debugging context */
 }};
 
 
@@ -162,7 +171,7 @@ H5A_dense_fh_name_cmp(const void *obj, size_t UNUSED obj_len, void *_udata)
     hbool_t took_ownership = FALSE;     /* Whether the "found" operator took ownership of the attribute */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5A_dense_fh_name_cmp)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Decode attribute information */
     if(NULL == (attr = (H5A_t *)H5O_msg_decode(udata->f, udata->dxpl_id, NULL, H5O_ATTR_ID, (const unsigned char *)obj)))
@@ -213,7 +222,7 @@ H5A_dense_btree2_name_store(void *_nrecord, const void *_udata)
     const H5A_bt2_ud_ins_t *udata = (const H5A_bt2_ud_ins_t *)_udata;
     H5A_dense_bt2_name_rec_t *nrecord = (H5A_dense_bt2_name_rec_t *)_nrecord;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_name_store)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Copy user information info native record */
     nrecord->id = udata->id;
@@ -246,7 +255,7 @@ H5A_dense_btree2_name_compare(const void *_bt2_udata, const void *_bt2_rec)
     const H5A_dense_bt2_name_rec_t *bt2_rec = (const H5A_dense_bt2_name_rec_t *)_bt2_rec;
     herr_t ret_value;           /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_name_compare)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Sanity check */
     HDassert(bt2_udata);
@@ -310,14 +319,15 @@ H5A_dense_btree2_name_compare(const void *_bt2_udata, const void *_bt2_rec)
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5A_dense_btree2_name_encode(const H5F_t UNUSED *f, uint8_t *raw, const void *_nrecord)
+H5A_dense_btree2_name_encode(uint8_t *raw, const void *_nrecord, void UNUSED *ctx)
 {
     const H5A_dense_bt2_name_rec_t *nrecord = (const H5A_dense_bt2_name_rec_t *)_nrecord;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_name_encode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Encode the record's fields */
-    UINT64ENCODE(raw, nrecord->id);
+    HDmemcpy(raw, nrecord->id.id, (size_t)H5O_FHEAP_ID_LEN);
+    raw += H5O_FHEAP_ID_LEN;
     *raw++ = nrecord->flags;
     UINT32ENCODE(raw, nrecord->corder)
     UINT32ENCODE(raw, nrecord->hash)
@@ -340,14 +350,15 @@ H5A_dense_btree2_name_encode(const H5F_t UNUSED *f, uint8_t *raw, const void *_n
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5A_dense_btree2_name_decode(const H5F_t UNUSED *f, const uint8_t *raw, void *_nrecord)
+H5A_dense_btree2_name_decode(const uint8_t *raw, void *_nrecord, void UNUSED *ctx)
 {
     H5A_dense_bt2_name_rec_t *nrecord = (H5A_dense_bt2_name_rec_t *)_nrecord;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_name_decode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Decode the record's fields */
-    UINT64DECODE(raw, nrecord->id);
+    HDmemcpy(nrecord->id.id, raw, (size_t)H5O_FHEAP_ID_LEN);
+    raw += H5O_FHEAP_ID_LEN;
     nrecord->flags = *raw++;
     UINT32DECODE(raw, nrecord->corder)
     UINT32DECODE(raw, nrecord->hash)
@@ -375,11 +386,11 @@ H5A_dense_btree2_name_debug(FILE *stream, const H5F_t UNUSED *f, hid_t UNUSED dx
 {
     const H5A_dense_bt2_name_rec_t *nrecord = (const H5A_dense_bt2_name_rec_t *)_nrecord;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_name_debug)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDfprintf(stream, "%*s%-*s {%016Hx, %02x, %u, %08lx}\n", indent, "", fwidth,
         "Record:",
-        (hsize_t)nrecord->id, (unsigned)nrecord->flags, (unsigned)nrecord->corder, (unsigned long)nrecord->hash);
+        (hsize_t)nrecord->id.val, (unsigned)nrecord->flags, (unsigned)nrecord->corder, (unsigned long)nrecord->hash);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* H5A_dense_btree2_name_debug() */
@@ -404,7 +415,7 @@ H5A_dense_btree2_corder_store(void *_nrecord, const void *_udata)
     const H5A_bt2_ud_ins_t *udata = (const H5A_bt2_ud_ins_t *)_udata;
     H5A_dense_bt2_corder_rec_t *nrecord = (H5A_dense_bt2_corder_rec_t *)_nrecord;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_corder_store)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Copy user information info native record */
     nrecord->id = udata->id;
@@ -436,7 +447,7 @@ H5A_dense_btree2_corder_compare(const void *_bt2_udata, const void *_bt2_rec)
     const H5A_dense_bt2_corder_rec_t *bt2_rec = (const H5A_dense_bt2_corder_rec_t *)_bt2_rec;
     herr_t ret_value;           /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_corder_compare)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Sanity check */
     HDassert(bt2_udata);
@@ -468,14 +479,15 @@ H5A_dense_btree2_corder_compare(const void *_bt2_udata, const void *_bt2_rec)
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5A_dense_btree2_corder_encode(const H5F_t UNUSED *f, uint8_t *raw, const void *_nrecord)
+H5A_dense_btree2_corder_encode(uint8_t *raw, const void *_nrecord, void UNUSED *ctx)
 {
     const H5A_dense_bt2_corder_rec_t *nrecord = (const H5A_dense_bt2_corder_rec_t *)_nrecord;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_corder_encode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Encode the record's fields */
-    UINT64ENCODE(raw, nrecord->id);
+    HDmemcpy(raw, nrecord->id.id, (size_t)H5O_FHEAP_ID_LEN);
+    raw += H5O_FHEAP_ID_LEN;
     *raw++ = nrecord->flags;
     UINT32ENCODE(raw, nrecord->corder)
 
@@ -497,14 +509,15 @@ H5A_dense_btree2_corder_encode(const H5F_t UNUSED *f, uint8_t *raw, const void *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5A_dense_btree2_corder_decode(const H5F_t UNUSED *f, const uint8_t *raw, void *_nrecord)
+H5A_dense_btree2_corder_decode(const uint8_t *raw, void *_nrecord, void UNUSED *ctx)
 {
     H5A_dense_bt2_corder_rec_t *nrecord = (H5A_dense_bt2_corder_rec_t *)_nrecord;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_corder_decode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Decode the record's fields */
-    UINT64DECODE(raw, nrecord->id);
+    HDmemcpy(nrecord->id.id, raw, (size_t)H5O_FHEAP_ID_LEN);
+    raw += H5O_FHEAP_ID_LEN;
     nrecord->flags = *raw++;
     UINT32DECODE(raw, nrecord->corder)
 
@@ -531,11 +544,11 @@ H5A_dense_btree2_corder_debug(FILE *stream, const H5F_t UNUSED *f, hid_t UNUSED
 {
     const H5A_dense_bt2_corder_rec_t *nrecord = (const H5A_dense_bt2_corder_rec_t *)_nrecord;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_dense_btree2_corder_debug)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDfprintf(stream, "%*s%-*s {%016Hx, %02x, %u}\n", indent, "", fwidth,
         "Record:",
-        (hsize_t)nrecord->id, (unsigned)nrecord->flags, (unsigned)nrecord->corder);
+        (hsize_t)nrecord->id.val, (unsigned)nrecord->flags, (unsigned)nrecord->corder);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* H5A_dense_btree2_corder_debug() */
