@@ -100,7 +100,7 @@ DESCRIPTION
 static herr_t
 H5A_init_deprec_interface(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5A_init_deprec_interface)
+    FUNC_ENTER_NOAPI_NOINIT
 
     FUNC_LEAVE_NOAPI(H5A_init())
 } /* H5A_init_deprec_interface() */
@@ -143,7 +143,7 @@ H5Acreate1(hid_t loc_id, const char *name, hid_t type_id, hid_t space_id,
     H5S_t		*space;                 /* Dataspace to use for attribute */
     hid_t		ret_value;              /* Return value */
 
-    FUNC_ENTER_API(H5Acreate1, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE5("i", "i*siii", loc_id, name, type_id, space_id, plist_id);
 
     /* check arguments */
@@ -198,7 +198,7 @@ H5Aopen_name(hid_t loc_id, const char *name)
     H5A_t               *attr = NULL;   /* Attribute opened */
     hid_t		ret_value;
 
-    FUNC_ENTER_API(H5Aopen_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("i", "i*s", loc_id, name);
 
     /* check arguments */
@@ -256,7 +256,7 @@ H5Aopen_idx(hid_t loc_id, unsigned idx)
     H5A_t       *attr = NULL;   /* Attribute opened */
     hid_t	ret_value;
 
-    FUNC_ENTER_API(H5Aopen_idx, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("i", "iIu", loc_id, idx);
 
     /* check arguments */
@@ -308,7 +308,7 @@ H5Aget_num_attrs(hid_t loc_id)
     void           	*obj;
     int			ret_value;
 
-    FUNC_ENTER_API(H5Aget_num_attrs, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Is", "i", loc_id);
 
     /* check arguments */
@@ -405,7 +405,7 @@ H5Aiterate1(hid_t loc_id, unsigned *attr_num, H5A_operator1_t op, void *op_data)
     hsize_t		last_attr;      /* Index of last attribute examined */
     herr_t	        ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Aiterate1, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "i*Iux*x", loc_id, attr_num, op, op_data);
 
     /* check arguments */
