@@ -53,6 +53,10 @@ static hid_t H5O_dset_open(const H5G_loc_t *obj_loc, hid_t lapl_id,
 static void *H5O_dset_create(H5F_t *f, void *_crt_info, H5G_loc_t *obj_loc,
     hid_t dxpl_id);
 static H5O_loc_t *H5O_dset_get_oloc(hid_t obj_id);
+static herr_t H5O_dset_bh_info(H5F_t *f, hid_t dxpl_id, H5O_t *oh,
+    H5_ih_info_t *bh_info);
+
+static herr_t H5O_dset_flush(H5G_loc_t *obj_loc, hid_t dxpl_id);
 
 
 /*********************/
@@ -78,7 +82,9 @@ const H5O_obj_class_t H5O_OBJ_DATASET[1] = {{
     H5O_dset_isa, 		/* "isa" message		*/
     H5O_dset_open, 		/* open an object of this class */
     H5O_dset_create, 		/* create an object of this class */
-    H5O_dset_get_oloc 		/* get an object header location for an object */
+    H5O_dset_get_oloc, 		/* get an object header location for an object */
+    H5O_dset_bh_info, 		/* get the index & heap info for an object */
+    H5O_dset_flush 		/* flush an opened object of this class */
 }};
 
 /* Declare a free list to manage the H5D_copy_file_ud_t struct */
@@ -105,7 +111,7 @@ H5O_dset_get_copy_file_udata(void)
 {
     void *ret_value;       /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_dset_get_copy_file_udata)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Allocate space for the 'copy file' user data for copying datasets */
     if(NULL == (ret_value = H5FL_CALLOC(H5D_copy_file_ud_t)))
@@ -138,7 +144,7 @@ H5O_dset_free_copy_file_udata(void *_udata)
 {
     H5D_copy_file_ud_t *udata = (H5D_copy_file_ud_t *)_udata;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_dset_free_copy_file_udata)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Sanity check */
     HDassert(udata);
@@ -152,11 +158,11 @@ H5O_dset_free_copy_file_udata(void *_udata)
         H5T_close(udata->src_dtype);
 
     /* Release copy of dataset's filter pipeline, if it was set */
-    if(udata->src_pline)
-        H5O_msg_free(H5O_PLINE_ID, udata->src_pline);
+    if(udata->common.src_pline)
+        H5O_msg_free(H5O_PLINE_ID, udata->common.src_pline);
 
     /* Release space for 'copy file' user data */
-    (void)H5FL_FREE(H5D_copy_file_ud_t, udata);
+    udata = H5FL_FREE(H5D_copy_file_ud_t, udata);
 
     FUNC_LEAVE_NOAPI_VOID
 } /* end H5O_dset_free_copy_file_udata() */
@@ -179,13 +185,13 @@ H5O_dset_free_copy_file_udata(void *_udata)
  *
  *-------------------------------------------------------------------------
  */
-htri_t
+static htri_t
 H5O_dset_isa(H5O_t *oh)
 {
     htri_t	exists;                 /* Flag if header message of interest exists */
     htri_t	ret_value = TRUE;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5O_dset_isa, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(oh);
 
@@ -227,7 +233,7 @@ H5O_dset_open(const H5G_loc_t *obj_loc, hid_t lapl_id, hid_t dxpl_id, hbool_t ap
     hid_t   dapl_id;                /* dapl to use to open this dataset */
     hid_t	ret_value;              /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_dset_open)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(obj_loc);
 
@@ -280,16 +286,15 @@ H5O_dset_create(H5F_t *f, void *_crt_info, H5G_loc_t *obj_loc, hid_t dxpl_id)
     H5D_t *dset = NULL;         /* New dataset created */
     void *ret_value;            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_dset_create)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Sanity checks */
     HDassert(f);
     HDassert(crt_info);
     HDassert(obj_loc);
 
-    /* Create the the dqtaset */
-    if(NULL == (dset = H5D_create(f, crt_info->type_id, crt_info->space,
-        crt_info->dcpl_id, crt_info->dapl_id, dxpl_id)))
+    /* Create the the dataset */
+    if(NULL == (dset = H5D_create(f, crt_info->type_id, crt_info->space, crt_info->dcpl_id, crt_info->dapl_id, dxpl_id)))
         HGOTO_ERROR(H5E_DATASET, H5E_CANTINIT, NULL, "unable to create dataset")
 
     /* Set up the new dataset's location */
@@ -329,7 +334,7 @@ H5O_dset_get_oloc(hid_t obj_id)
     H5D_t       *dset;                  /* Dataset opened */
     H5O_loc_t	*ret_value;             /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_dset_get_oloc)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Get the dataset */
     if(NULL == (dset = (H5D_t *)H5I_object(obj_id)))
@@ -356,15 +361,25 @@ done:
  * Programmer:  Vailin Choi
  *              July 11, 2007
  *
+ * Modification:Raymond Lu
+ *              5 February, 2010
+ *              I added the call to H5O_msg_reset after H5D_chunk_bh_info 
+ *              to free the PLINE. 
  *-------------------------------------------------------------------------
  */
-herr_t
+static herr_t
 H5O_dset_bh_info(H5F_t *f, hid_t dxpl_id, H5O_t *oh, H5_ih_info_t *bh_info)
 {
     H5O_layout_t        layout;         	/* Data storage layout message */
-    herr_t      	ret_value = SUCCEED;      /* Return value */
+    H5O_pline_t         pline;                  /* I/O pipeline message */
+    H5O_efl_t           efl;			/* External File List message */
+    hbool_t             layout_read = FALSE;    /* Whether the layout message was read */
+    hbool_t             pline_read = FALSE;     /* Whether the I/O pipeline message was read */
+    hbool_t             efl_read = FALSE;       /* Whether the external file list message was read */
+    htri_t		exists;                 /* Flag if header message of interest exists */
+    herr_t      	ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_NOAPI(H5O_dset_bh_info, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Sanity check */
     HDassert(f);
@@ -373,14 +388,98 @@ H5O_dset_bh_info(H5F_t *f, hid_t dxpl_id, H5O_t *oh, H5_ih_info_t *bh_info)
 
     /* Get the layout message from the object header */
     if(NULL == H5O_msg_read_oh(f, dxpl_id, oh, H5O_LAYOUT_ID, &layout))
-	HGOTO_ERROR(H5E_OHDR, H5E_CANTGET, FAIL, "can't find LAYOUT message")
+	HGOTO_ERROR(H5E_OHDR, H5E_CANTGET, FAIL, "can't find layout message")
+    layout_read = TRUE;
 
     /* Check for chunked dataset storage */
-    if(layout.type == H5D_CHUNKED && H5D_chunk_is_space_alloc(&layout))
-        if(H5D_chunk_bh_info(f, dxpl_id, &layout, &(bh_info->index_size)) < 0)
+    if(layout.type == H5D_CHUNKED && H5D_chunk_is_space_alloc(&layout.storage)) {
+        /* Check for I/O pipeline message */
+        if((exists = H5O_msg_exists_oh(oh, H5O_PLINE_ID)) < 0)
+            HGOTO_ERROR(H5E_DATASET, H5E_CANTINIT, FAIL, "unable to read object header")
+        else if(exists) {
+            if(NULL == H5O_msg_read_oh(f, dxpl_id, oh, H5O_PLINE_ID, &pline))
+                HGOTO_ERROR(H5E_OHDR, H5E_CANTGET, FAIL, "can't find I/O pipeline message")
+            pline_read = TRUE;
+        } /* end else if */
+        else
+            HDmemset(&pline, 0, sizeof(pline));
+
+        if(H5D_chunk_bh_info(f, dxpl_id, &layout, &pline, &(bh_info->index_size)) < 0)
             HGOTO_ERROR(H5E_OHDR, H5E_CANTGET, FAIL, "can't determine chunked dataset btree info")
+    } /* end if */
+
+    /* Check for External File List message in the object header */
+    if((exists = H5O_msg_exists_oh(oh, H5O_EFL_ID)) < 0)
+	HGOTO_ERROR(H5E_OHDR, H5E_NOTFOUND, FAIL, "unable to check for EFL message")
+
+    if(exists && H5D_efl_is_space_alloc(&layout.storage)) {
+        /* Start with clean EFL info */
+        HDmemset(&efl, 0, sizeof(efl));
+
+	/* Get External File List message from the object header */
+	if(NULL == H5O_msg_read_oh(f, dxpl_id, oh, H5O_EFL_ID, &efl))
+	    HGOTO_ERROR(H5E_OHDR, H5E_CANTGET, FAIL, "can't find EFL message")
+        efl_read = TRUE;
+
+	/* Get size of local heap for EFL message's file list */
+	if(H5D_efl_bh_info(f, dxpl_id, &efl, &(bh_info->heap_size)) < 0)
+            HGOTO_ERROR(H5E_OHDR, H5E_CANTGET, FAIL, "can't determine EFL heap info")
+    } /* end if */
 
 done:
+    /* Free messages, if they've been read in */
+    if(layout_read && H5O_msg_reset(H5O_LAYOUT_ID, &layout) < 0)
+        HDONE_ERROR(H5E_DATASET, H5E_CANTRESET, FAIL, "unable to reset data storage layout message")
+    if(pline_read && H5O_msg_reset(H5O_PLINE_ID, &pline) < 0)
+        HDONE_ERROR(H5E_DATASET, H5E_CANTRESET, FAIL, "unable to reset I/O pipeline message")
+    if(efl_read && H5O_msg_reset(H5O_EFL_ID, &efl) < 0)
+        HDONE_ERROR(H5E_DATASET, H5E_CANTRESET, FAIL, "unable to reset external file list message")
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_dset_bh_info() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:    H5O_dset_flush
+ *
+ * Purpose:     To flush any dataset information cached in memory
+ *
+ * Return:      Success:        non-negative
+ *              Failure:        negative
+ *
+ * Programmer:  Vailin Choi
+ *              February 2012
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5O_dset_flush(H5G_loc_t *obj_loc, hid_t dxpl_id)
+{
+    H5D_t       *dset;                  /* Dataset opened */
+    H5O_type_t 	obj_type;              /* Type of object at location */
+    herr_t      ret_value = SUCCEED;    /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT
+
+    HDassert(obj_loc);
+    HDassert(obj_loc->oloc);
+
+    /* Check that the object found is the correct type */
+    if(H5O_obj_type(obj_loc->oloc, &obj_type, dxpl_id) < 0)
+        HGOTO_ERROR(H5E_DATASET, H5E_CANTGET, FAIL, "can't get object type")
+
+    if(obj_type != H5O_TYPE_DATASET)
+        HGOTO_ERROR(H5E_DATASET, H5E_BADTYPE, FAIL, "not a dataset")
+
+    /* Open the dataset */
+    if(NULL == (dset = H5D_open(obj_loc, H5P_DATASET_ACCESS_DEFAULT, dxpl_id)))
+        HGOTO_ERROR(H5E_DATASET, H5E_CANTOPENOBJ, FAIL, "unable to open dataset")
+    
+    if(H5D_flush_real(dset, dxpl_id) < 0)
+	HDONE_ERROR(H5E_DATASET, H5E_WRITEERROR, FAIL, "unable to flush cached dataset info")
+
+done:
+    if(dset && H5D_close(dset) < 0)
+	HDONE_ERROR(H5E_DATASET, H5E_CLOSEERROR, NULL, "unable to release dataset")
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_dset_flush() */
