@@ -107,7 +107,7 @@ static H5E_cls_t *H5E_register_class(const char *cls_name, const char *lib_name,
                                 const char *version);
 static herr_t  H5E_unregister_class(H5E_cls_t *cls);
 static ssize_t H5E_get_class_name(const H5E_cls_t *cls, char *name, size_t size);
-static int H5E_close_msg_cb(void *obj_ptr, hid_t obj_id, void *key);
+static int H5E_close_msg_cb(void *obj_ptr, hid_t obj_id, void *udata);
 static herr_t  H5E_close_msg(H5E_msg_t *err);
 static H5E_msg_t *H5E_create_msg(H5E_cls_t *cls, H5E_type_t msg_type, const char *msg);
 static H5E_t  *H5E_get_current_stack(void);
@@ -144,7 +144,7 @@ H5E_init(void)
 {
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5E_init, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
     /* FUNC_ENTER() does all the work */
 
 done:
@@ -168,7 +168,7 @@ done:
 static herr_t
 H5E_set_default_auto(H5E_t *stk)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_set_default_auto)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
 #ifndef H5_NO_DEPRECATED_SYMBOLS
 #ifdef H5_USE_16_API_DEFAULT
@@ -210,7 +210,7 @@ H5E_init_interface(void)
     char lib_vers[128];         /* Buffer to constructu library version within */
     herr_t      ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5E_init_interface)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Initialize the atom group for the error class IDs */
     if(H5I_register_type(H5I_ERROR_CLASS, (size_t)H5I_ERRCLS_HASHSIZE, H5E_RESERVED_ATOMS,
@@ -268,7 +268,7 @@ H5E_term_interface(void)
 {
     int	n = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_term_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     if(H5_interface_initialize_g) {
         int ncls, nmsg, nstk;
@@ -341,7 +341,7 @@ H5E_get_stack(void)
 {
     H5E_t *estack;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_get_stack)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     estack = (H5E_t *)H5TS_get_thread_local_value(H5TS_errstk_key_g);
 
@@ -382,7 +382,7 @@ H5E_get_stack(void)
 static herr_t
 H5E_free_class(H5E_cls_t *cls)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_free_class)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check arguments */
     HDassert(cls);
@@ -415,7 +415,7 @@ H5Eregister_class(const char *cls_name, const char *lib_name, const char *versio
     H5E_cls_t   *cls;        /* Pointer to error class */
     hid_t       ret_value;   /* Return value */
 
-    FUNC_ENTER_API(H5Eregister_class, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("i", "*s*s*s", cls_name, lib_name, version);
 
     /* Check arguments */
@@ -453,7 +453,7 @@ H5E_register_class(const char *cls_name, const char *lib_name, const char *versi
     H5E_cls_t   *cls = NULL; /* Pointer to error class */
     H5E_cls_t   *ret_value;  /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5E_register_class)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments */
     HDassert(cls_name);
@@ -501,7 +501,7 @@ H5Eunregister_class(hid_t class_id)
 {
     herr_t       ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_API(H5Eunregister_class, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", class_id);
 
     /* Check arguments */
@@ -537,14 +537,14 @@ H5E_unregister_class(H5E_cls_t *cls)
 {
     herr_t      ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5E_unregister_class)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments */
     HDassert(cls);
 
     /* Iterate over all the messages and delete those in this error class */
-    /* (Ignore return value, since callback isn't designed to return a particular object) */
-    (void)H5I_search(H5I_ERROR_MSG, H5E_close_msg_cb, cls, FALSE);
+    if(H5I_iterate(H5I_ERROR_MSG, H5E_close_msg_cb, cls, FALSE) < 0)
+        HGOTO_ERROR(H5E_ERROR, H5E_BADITER, FAIL, "unable to free all messages in this error class")
 
     /* Free error class structure */
     if(H5E_free_class(cls) < 0)
@@ -574,7 +574,7 @@ H5Eget_class_name(hid_t class_id, char *name, size_t size)
     H5E_cls_t   *cls;        /* Pointer to error class */
     ssize_t     ret_value;   /* Return value */
 
-    FUNC_ENTER_API(H5Eget_class_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("Zs", "i*sz", class_id, name, size);
 
     /* Get the error class */
@@ -608,7 +608,7 @@ H5E_get_class_name(const H5E_cls_t *cls, char *name, size_t size)
 {
     ssize_t       len;          /* Length of error class's name */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_get_class_name)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check arguments */
     HDassert(cls);
@@ -631,7 +631,7 @@ H5E_get_class_name(const H5E_cls_t *cls, char *name, size_t size)
 /*-------------------------------------------------------------------------
  * Function:    H5E_close_msg_cb
  *
- * Purpose:     H5I_search callback function to close error messages in the
+ * Purpose:     H5I_iterate callback function to close error messages in the
  *              error class.
  *
  * Return:	Non-negative value on success/Negative on failure
@@ -642,13 +642,13 @@ H5E_get_class_name(const H5E_cls_t *cls, char *name, size_t size)
  *-------------------------------------------------------------------------
  */
 static int
-H5E_close_msg_cb(void *obj_ptr, hid_t obj_id, void *key)
+H5E_close_msg_cb(void *obj_ptr, hid_t obj_id, void *udata)
 {
     H5E_msg_t   *err_msg = (H5E_msg_t*)obj_ptr;
-    H5E_cls_t   *cls = (H5E_cls_t*)key;
+    H5E_cls_t   *cls = (H5E_cls_t*)udata;
     herr_t      ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5E_close_msg_cb)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments */
     HDassert(err_msg);
@@ -683,7 +683,7 @@ H5Eclose_msg(hid_t err_id)
 {
     herr_t       ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_API(H5Eclose_msg, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", err_id);
 
     /* Check arguments */
@@ -714,7 +714,7 @@ done:
 static herr_t
 H5E_close_msg(H5E_msg_t *err)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_close_msg)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check arguments */
     HDassert(err);
@@ -747,7 +747,7 @@ H5Ecreate_msg(hid_t class_id, H5E_type_t msg_type, const char *msg_str)
     H5E_msg_t   *msg;           /* Pointer to new error message */
     hid_t       ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Ecreate_msg, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("i", "iEt*s", class_id, msg_type, msg_str);
 
     /* Check arguments */
@@ -790,7 +790,7 @@ H5E_create_msg(H5E_cls_t *cls, H5E_type_t msg_type, const char *msg_str)
     H5E_msg_t   *msg = NULL;    /* Pointer to new error message */
     H5E_msg_t   *ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5E_create_msg)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments */
     HDassert(cls);
@@ -838,7 +838,7 @@ H5Eget_msg(hid_t msg_id, H5E_type_t *type, char *msg_str, size_t size)
     H5E_msg_t   *msg;           /* Pointer to error message */
     ssize_t      ret_value;     /* Return value */
 
-    FUNC_ENTER_API(H5Eget_msg, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("Zs", "i*Et*sz", msg_id, type, msg_str, size);
 
     /* Get the message object */
@@ -872,7 +872,7 @@ H5Ecreate_stack(void)
     H5E_t	*stk;           /* Error stack */
     hid_t       ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Ecreate_stack, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE0("i","");
 
     /* Allocate a new error stack */
@@ -911,7 +911,7 @@ H5Eget_current_stack(void)
     hid_t       ret_value;   /* Return value */
 
     /* Don't clear the error stack! :-) */
-    FUNC_ENTER_API_NOCLEAR(H5Eget_current_stack, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     H5TRACE0("i","");
 
     /* Get the current stack */
@@ -947,7 +947,7 @@ H5E_get_current_stack(void)
     unsigned    u;              /* Local index variable */
     H5E_t      *ret_value;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5E_get_current_stack)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Get a pointer to the current error stack */
     if(NULL == (current_stack = H5E_get_my_stack())) /*lint !e506 !e774 Make lint 'constant value Boolean' in non-threaded case */
@@ -1028,7 +1028,7 @@ H5Eset_current_stack(hid_t err_stack)
     H5E_t *estack;
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_API(H5Eset_current_stack, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", err_stack);
 
     if(err_stack != H5E_DEFAULT) {
@@ -1071,7 +1071,7 @@ H5E_set_current_stack(H5E_t *estack)
     unsigned     u;                     /* Local index variable */
     herr_t       ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5E_set_current_stack)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Sanity check */
     HDassert(estack);
@@ -1133,7 +1133,7 @@ H5Eclose_stack(hid_t stack_id)
 {
     herr_t       ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_API(H5Eclose_stack, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", stack_id);
 
     if(H5E_DEFAULT != stack_id) {
@@ -1169,7 +1169,7 @@ done:
 static herr_t
 H5E_close_stack(H5E_t *estack)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_close_stack)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Sanity check */
     HDassert(estack);
@@ -1203,7 +1203,7 @@ H5Eget_num(hid_t error_stack_id)
     ssize_t ret_value;      /* Return value */
 
     /* Don't clear the error stack! :-) */
-    FUNC_ENTER_API_NOCLEAR(H5Eget_num, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     H5TRACE1("Zs", "i", error_stack_id);
 
     /* Need to check for errors */
@@ -1244,7 +1244,7 @@ done:
 static ssize_t
 H5E_get_num(const H5E_t *estack)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_get_num)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     HDassert(estack);
 
@@ -1271,7 +1271,7 @@ H5Epop(hid_t err_stack, size_t count)
     herr_t ret_value = SUCCEED;   /* Return value */
 
     /* Don't clear the error stack! :-) */
-    FUNC_ENTER_API_NOCLEAR(H5Epop, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     H5TRACE2("e", "iz", err_stack, count);
 
     /* Need to check for errors */
@@ -1337,7 +1337,7 @@ H5Epush2(hid_t err_stack, const char *file, const char *func, unsigned line,
     herr_t	ret_value=SUCCEED;      /* Return value */
 
     /* Don't clear the error stack! :-) */
-    FUNC_ENTER_API_NOCLEAR(H5Epush2, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     H5TRACE8("e", "i*s*sIuiii*s", err_stack, file, func, line, cls_id, maj_id,
              min_id, fmt);
 
@@ -1429,7 +1429,7 @@ H5Eclear2(hid_t err_stack)
     herr_t ret_value = SUCCEED;   /* Return value */
 
     /* Don't clear the error stack! :-) */
-    FUNC_ENTER_API_NOCLEAR(H5Eclear2, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     H5TRACE1("e", "i", err_stack);
 
     /* Need to check for errors */
@@ -1474,7 +1474,7 @@ H5Eprint2(hid_t err_stack, FILE *stream)
     herr_t ret_value = SUCCEED;   /* Return value */
 
     /* Don't clear the error stack! :-) */
-    FUNC_ENTER_API_NOCLEAR(H5Eprint2, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     /*NO TRACE*/
 
     /* Need to check for errors */
@@ -1520,7 +1520,7 @@ H5Ewalk2(hid_t err_stack, H5E_direction_t direction, H5E_walk2_t stack_func, voi
     herr_t ret_value = SUCCEED;         /* Return value */
 
     /* Don't clear the error stack! :-) */
-    FUNC_ENTER_API_NOCLEAR(H5Ewalk2, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     /*NO TRACE*/
 
     /* Need to check for errors */
@@ -1574,7 +1574,7 @@ H5Eget_auto2(hid_t estack_id, H5E_auto2_t *func, void **client_data)
     H5E_auto_op_t op;           /* Error stack function */
     herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_API(H5Eget_auto2, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "i*x**x", estack_id, func, client_data);
 
     if(estack_id == H5E_DEFAULT) {
@@ -1636,7 +1636,7 @@ H5Eset_auto2(hid_t estack_id, H5E_auto2_t func, void *client_data)
     herr_t ret_value = SUCCEED; /* Return value */
 
     /* Don't clear the error stack! :-) */
-    FUNC_ENTER_API_NOCLEAR(H5Eset_auto2, FAIL)
+    FUNC_ENTER_API_NOCLEAR(FAIL)
     H5TRACE3("e", "ix*x", estack_id, func, client_data);
 
     if(estack_id == H5E_DEFAULT) {
@@ -1693,7 +1693,7 @@ H5Eauto_is_v2(hid_t estack_id, unsigned *is_stack)
     H5E_t   *estack;            /* Error stack to operate on */
     herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_API(H5Eauto_is_v2, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*Iu", estack_id, is_stack);
 
     if(estack_id == H5E_DEFAULT) {
