@@ -61,14 +61,19 @@ typedef struct H5FD_family_t {
     hid_t	memb_fapl_id;	/*file access property list for members	*/
     hsize_t	memb_size;	/*actual size of each member file	*/
     hsize_t	pmem_size;	/*member size passed in from property	*/
-    hsize_t	mem_newsize;	/*new member size passed in as private
-                                 *property. It's used only by h5repart  */
     unsigned	nmembs;		/*number of family members		*/
     unsigned	amembs;		/*number of member slots allocated	*/
     H5FD_t	**memb;		/*dynamic array of member pointers	*/
     haddr_t	eoa;		/*end of allocated addresses		*/
     char	*name;		/*name generator printf format		*/
     unsigned	flags;		/*flags for opening additional members	*/
+
+    /* Information from properties set by 'h5repart' tool */
+    hsize_t	mem_newsize;	/*new member size passed in as private
+                                 * property. It's used only by h5repart */
+    hbool_t     repart_members; /* Whether to mark the superblock dirty
+                                 * when it is loaded, so that the family
+                                 * member sizes can be re-encoded       */
 } H5FD_family_t;
 
 /* Driver-specific file access properties */
@@ -161,7 +166,7 @@ DESCRIPTION
 static herr_t
 H5FD_family_init_interface(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_family_init_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     FUNC_LEAVE_NOAPI(H5FD_family_init())
 } /* H5FD_family_init_interface() */
@@ -189,7 +194,7 @@ H5FD_family_init(void)
 {
     hid_t ret_value=H5FD_FAMILY_g;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_init, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     if (H5I_VFL!=H5Iget_type(H5FD_FAMILY_g))
         H5FD_FAMILY_g = H5FD_register(&H5FD_family_g,sizeof(H5FD_class_t),FALSE);
@@ -219,7 +224,7 @@ done:
 void
 H5FD_family_term(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_family_term)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Reset VFL ID */
     H5FD_FAMILY_g=0;
@@ -260,7 +265,7 @@ H5Pset_fapl_family(hid_t fapl_id, hsize_t msize, hid_t memb_fapl_id)
     H5FD_family_fapl_t	fa={0, -1};
     H5P_genplist_t *plist;      /* Property list pointer */
 
-    FUNC_ENTER_API(H5Pset_fapl_family, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ihi", fapl_id, msize, memb_fapl_id);
 
 
@@ -319,7 +324,7 @@ H5Pget_fapl_family(hid_t fapl_id, hsize_t *msize/*out*/,
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t      ret_value=SUCCEED;       /* Return value */
 
-    FUNC_ENTER_API(H5Pget_fapl_family, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ixx", fapl_id, msize, memb_fapl_id);
 
     if(NULL == (plist = H5P_object_verify(fapl_id,H5P_FILE_ACCESS)))
@@ -366,7 +371,7 @@ H5FD_family_fapl_get(H5FD_t *_file)
     H5P_genplist_t *plist;      /* Property list pointer */
     void *ret_value;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_fapl_get, NULL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     if(NULL == (fa = (H5FD_family_fapl_t *)H5MM_calloc(sizeof(H5FD_family_fapl_t))))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
@@ -412,7 +417,7 @@ H5FD_family_fapl_copy(const void *_old_fa)
     H5P_genplist_t *plist;      /* Property list pointer */
     void *ret_value;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_fapl_copy, NULL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     if(NULL == (new_fa = (H5FD_family_fapl_t *)H5MM_malloc(sizeof(H5FD_family_fapl_t))))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
@@ -463,11 +468,11 @@ static herr_t
 H5FD_family_fapl_free(void *_fa)
 {
     H5FD_family_fapl_t	*fa = (H5FD_family_fapl_t*)_fa;
-    herr_t ret_value=SUCCEED;   /* Return value */
+    herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_fapl_free, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
-    if(H5I_dec_ref(fa->memb_fapl_id, FALSE)<0)
+    if(H5I_dec_ref(fa->memb_fapl_id) < 0)
         HGOTO_ERROR(H5E_VFL, H5E_CANTDEC, FAIL, "can't close driver ID")
     H5MM_xfree(fa);
 
@@ -500,7 +505,7 @@ H5FD_family_dxpl_copy(const void *_old_dx)
     H5P_genplist_t *plist;      /* Property list pointer */
     void *ret_value;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_dxpl_copy, NULL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     if(NULL == (new_dx = (H5FD_family_dxpl_t *)H5MM_malloc(sizeof(H5FD_family_dxpl_t))))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
@@ -549,11 +554,11 @@ static herr_t
 H5FD_family_dxpl_free(void *_dx)
 {
     H5FD_family_dxpl_t	*dx = (H5FD_family_dxpl_t*)_dx;
-    herr_t ret_value=SUCCEED;   /* Return value */
+    herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_dxpl_free, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
-    if(H5I_dec_ref(dx->memb_dxpl_id, FALSE)<0)
+    if(H5I_dec_ref(dx->memb_dxpl_id) < 0)
         HGOTO_ERROR(H5E_VFL, H5E_CANTDEC, FAIL, "can't close driver ID")
     H5MM_xfree(dx);
 
@@ -582,16 +587,11 @@ done:
 static hsize_t
 H5FD_family_sb_size(H5FD_t UNUSED *_file)
 {
-    hsize_t		ret_value = 0; /*size of header*/
-
-    FUNC_ENTER_NOAPI(H5FD_family_sb_size, UFAIL)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* 8 bytes field for the size of member file size field should be
      * enough for now. */
-    ret_value += 8;
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
+    FUNC_LEAVE_NOAPI(8)
 }
 
 
@@ -620,16 +620,16 @@ H5FD_family_sb_encode(H5FD_t *_file, char *name/*out*/, unsigned char *buf/*out*
 {
     H5FD_family_t	*file = (H5FD_family_t*)_file;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_family_sb_encode)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Name and version number */
-    HDstrncpy(name, "NCSAfami", (size_t)8);
+    HDstrncpy(name, "NCSAfami", (size_t)9);
     name[8] = '\0';
 
     /* Store member file size.  Use the member file size from the property here.
-     * This is to guarantee backward compatibility.  If a file is created with 
+     * This is to guarantee backward compatibility.  If a file is created with
      * v1.6 library and the driver info isn't saved in the superblock.  We open
-     * it with v1.8, the FILE->MEMB_SIZE will be the actual size of the first 
+     * it with v1.8, the FILE->MEMB_SIZE will be the actual size of the first
      * member file (see H5FD_family_open).  So it isn't safe to use FILE->MEMB_SIZE.
      * If the file is created with v1.8, the correctness of FILE->PMEM_SIZE is
      * checked in H5FD_family_sb_decode. SLU - 2009/3/21
@@ -656,8 +656,6 @@ H5FD_family_sb_encode(H5FD_t *_file, char *name/*out*/, unsigned char *buf/*out*
  * Programmer:	Raymond Lu
  *              Tuesday, May 10, 2005
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
@@ -667,7 +665,7 @@ H5FD_family_sb_decode(H5FD_t *_file, const char UNUSED *name, const unsigned cha
     uint64_t            msize;
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_sb_decode, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Read member file size. Skip name template for now although it's saved. */
     UINT64DECODE(buf, msize);
@@ -687,9 +685,9 @@ H5FD_family_sb_decode(H5FD_t *_file, const char UNUSED *name, const unsigned cha
 
     /* Check if member size from file access property is correct */
     if(msize != file->pmem_size) {
-        char                err_msg[128];
+        char err_msg[128];
 
-        sprintf(err_msg, "Family member size should be %lu.  But the size from file access property is %lu", (unsigned long)msize, (unsigned long)file->pmem_size);
+        HDsnprintf(err_msg, sizeof(err_msg), "Family member size should be %lu.  But the size from file access property is %lu", (unsigned long)msize, (unsigned long)file->pmem_size);
         HGOTO_ERROR(H5E_FILE, H5E_BADVALUE, FAIL, err_msg)
     } /* end if */
 
@@ -743,11 +741,9 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
     H5FD_t     		*ret_value=NULL;
     char		memb_name[4096], temp[4096];
     hsize_t		eof=HADDR_UNDEF;
-    hsize_t             fam_newsize = 0;
     unsigned		t_flags = flags & ~H5F_ACC_CREAT;
-    H5P_genplist_t      *plist;      /* Property list pointer */
 
-    FUNC_ENTER_NOAPI(H5FD_family_open, NULL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments */
     if(!name || !*name)
@@ -767,6 +763,7 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
         file->mem_newsize = 0;            /*New member size used by h5repart only       */
     } /* end if */
     else {
+        H5P_genplist_t      *plist;      /* Property list pointer */
         H5FD_family_fapl_t *fa;
 
         if(NULL == (plist = (H5P_genplist_t *)H5I_object(fapl_id)))
@@ -774,11 +771,19 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
         fa = (H5FD_family_fapl_t *)H5P_get_driver_info(plist);
         HDassert(fa);
 
-        /* New family file size. It's used by h5repart only. */
-        if(H5P_exist_plist(plist, H5F_ACS_FAMILY_NEWSIZE_NAME) > 0)
+        /* Check for new family file size. It's used by h5repart only. */
+        if(H5P_exist_plist(plist, H5F_ACS_FAMILY_NEWSIZE_NAME) > 0) {
+            hsize_t fam_newsize = 0;        /* New member size, when repartitioning */
+
+            /* Get the new family file size */
             if(H5P_get(plist, H5F_ACS_FAMILY_NEWSIZE_NAME, &fam_newsize) < 0)
                 HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, NULL, "can't get new family member size")
 
+            /* Store information for later */
+            file->mem_newsize = fam_newsize; /* New member size passed in through property */
+            file->repart_members = TRUE;
+        } /* end if */
+
         if(fa->memb_fapl_id==H5P_FILE_ACCESS_DEFAULT) {
             if(H5I_inc_ref(fa->memb_fapl_id, FALSE)<0)
                 HGOTO_ERROR(H5E_VFL, H5E_CANTINC, NULL, "unable to increment ref count on VFL driver")
@@ -791,28 +796,27 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
         } /* end else */
         file->memb_size = fa->memb_size; /* Actual member size to be updated later */
         file->pmem_size = fa->memb_size; /* Member size passed in through property */
-        file->mem_newsize = fam_newsize; /* New member size passed in through property */
     } /* end else */
     file->name = H5MM_strdup(name);
     file->flags = flags;
 
     /* Check that names are unique */
-    sprintf(memb_name, name, 0);
-    sprintf(temp, name, 1);
-
-    if (!strcmp(memb_name, temp))
+    HDsnprintf(memb_name, sizeof(memb_name), name, 0);
+    HDsnprintf(temp, sizeof(temp), name, 1);
+    if(!HDstrcmp(memb_name, temp))
         HGOTO_ERROR(H5E_FILE, H5E_FILEEXISTS, NULL, "file names not unique")
 
     /* Open all the family members */
     while(1) {
-        sprintf(memb_name, name, file->nmembs);
+        HDsnprintf(memb_name, sizeof(memb_name), name, file->nmembs);
 
         /* Enlarge member array */
         if(file->nmembs >= file->amembs) {
             unsigned n = MAX(64, 2 * file->amembs);
-            H5FD_t **x = (H5FD_t **)H5MM_realloc(file->memb, n * sizeof(H5FD_t *));
+            H5FD_t **x;
 
-            if(!x)
+            HDassert(n > 0);
+            if(NULL == (x = (H5FD_t **)H5MM_realloc(file->memb, n * sizeof(H5FD_t *))))
                 HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "unable to reallocate members")
             file->amembs = n;
             file->memb = x;
@@ -846,29 +850,30 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
 
 done:
     /* Cleanup and fail */
-    if (ret_value==NULL && file!=NULL) {
-        unsigned nerrors=0;     /* Number of errors closing member files */
+    if(ret_value == NULL && file != NULL) {
+        unsigned nerrors = 0;   /* Number of errors closing member files */
         unsigned u;             /* Local index variable */
 
         /* Close as many members as possible. Use private function here to avoid clearing
          * the error stack. We need the error message to indicate wrong member file size. */
-        for (u=0; u<file->nmembs; u++)
-            if (file->memb[u])
-                if (H5FD_close(file->memb[u])<0)
+        for(u = 0; u < file->nmembs; u++)
+            if(file->memb[u])
+                if(H5FD_close(file->memb[u]) < 0)
                     nerrors++;
-        if (nerrors)
+        if(nerrors)
             HGOTO_ERROR(H5E_FILE, H5E_CANTCLOSEFILE, NULL, "unable to close member files")
 
-        if (file->memb)
+        if(file->memb)
             H5MM_xfree(file->memb);
-        if(H5I_dec_ref(file->memb_fapl_id, FALSE)<0)
+        if(H5I_dec_ref(file->memb_fapl_id) < 0)
             HDONE_ERROR(H5E_VFL, H5E_CANTDEC, NULL, "can't close driver ID")
-        if (file->name)
+        if(file->name)
             H5MM_xfree(file->name);
         H5MM_xfree(file);
-    }
+    } /* end if */
+
     FUNC_LEAVE_NOAPI(ret_value)
-}
+} /* end H5FD_family_open() */
 
 
 /*-------------------------------------------------------------------------
@@ -885,45 +890,42 @@ done:
  * Programmer:	Robb Matzke
  *              Wednesday, August  4, 1999
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
 H5FD_family_close(H5FD_t *_file)
 {
-    H5FD_family_t	*file = (H5FD_family_t*)_file;
-    unsigned		nerrors=0;      /* Number of errors while closing member files */
-    unsigned		u;              /* Local index variable */
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    H5FD_family_t *file = (H5FD_family_t*)_file;
+    unsigned	nerrors = 0;    /* Number of errors while closing member files */
+    unsigned	u;              /* Local index variable */
+    herr_t      ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_close, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Close as many members as possible. Use private function here to avoid clearing
      * the error stack. We need the error message to indicate wrong member file size. */
-    for (u=0; u<file->nmembs; u++) {
-        if (file->memb[u]) {
-            if (H5FD_close(file->memb[u])<0)
+    for(u = 0; u < file->nmembs; u++) {
+        if(file->memb[u]) {
+            if(H5FD_close(file->memb[u]) < 0)
                 nerrors++;
             else
                 file->memb[u] = NULL;
-        }
-    }
-    if (nerrors)
-        HGOTO_ERROR(H5E_FILE, H5E_CANTCLOSEFILE, FAIL, "unable to close member files")
+        } /* end if */
+    } /* end for */
+    if(nerrors)
+        /* Push error, but keep going*/
+        HDONE_ERROR(H5E_FILE, H5E_CANTCLOSEFILE, FAIL, "unable to close member files")
 
     /* Clean up other stuff */
-    if(H5I_dec_ref(file->memb_fapl_id, FALSE)<0)
-        HGOTO_ERROR(H5E_VFL, H5E_CANTDEC, FAIL, "can't close driver ID")
-    if (file->memb)
-        H5MM_xfree(file->memb);
-    if (file->name)
-        H5MM_xfree(file->name);
+    if(H5I_dec_ref(file->memb_fapl_id) < 0)
+        /* Push error, but keep going*/
+        HDONE_ERROR(H5E_VFL, H5E_CANTDEC, FAIL, "can't close driver ID")
+    H5MM_xfree(file->memb);
+    H5MM_xfree(file->name);
     H5MM_xfree(file);
 
-done:
     FUNC_LEAVE_NOAPI(ret_value)
-}
+} /* end H5FD_family_close() */
 
 
 /*-------------------------------------------------------------------------
@@ -951,7 +953,7 @@ H5FD_family_cmp(const H5FD_t *_f1, const H5FD_t *_f2)
     const H5FD_family_t	*f2 = (const H5FD_family_t*)_f2;
     int ret_value=(H5FD_VFD_DEFAULT);
 
-    FUNC_ENTER_NOAPI(H5FD_family_cmp, H5FD_VFD_DEFAULT)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     assert(f1->nmembs>=1 && f1->memb[0]);
     assert(f2->nmembs>=1 && f2->memb[0]);
@@ -970,23 +972,20 @@ done:
  *              (listed in H5FDpublic.h)
  *
  * Return:	Success:	non-negative
- *
  *		Failure:	negative
  *
  * Programmer:	Quincey Koziol
  *              Friday, August 25, 2000
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
 static herr_t
-H5FD_family_query(const H5FD_t UNUSED * _f, unsigned long *flags /* out */)
+H5FD_family_query(const H5FD_t * _file, unsigned long *flags /* out */)
 {
-    herr_t ret_value = SUCCEED;
+    const H5FD_family_t	*file = (const H5FD_family_t*)_file;    /* Family VFD info */
 
-    FUNC_ENTER_NOAPI(H5FD_family_query, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Set the VFL feature flags that this driver supports */
     if(flags) {
@@ -995,10 +994,13 @@ H5FD_family_query(const H5FD_t UNUSED * _f, unsigned long *flags /* out */)
         *flags |= H5FD_FEAT_ACCUMULATE_METADATA; /* OK to accumulate metadata for faster writes. */
         *flags |= H5FD_FEAT_DATA_SIEVE;       /* OK to perform data sieving for faster raw data reads & writes */
         *flags |= H5FD_FEAT_AGGREGATE_SMALLDATA; /* OK to aggregate "small" raw data allocations */
+
+        /* Check for flags that are set by h5repart */
+        if(file && file->repart_members)
+            *flags |= H5FD_FEAT_DIRTY_SBLK_LOAD; /* Mark the superblock dirty when it is loaded (so the family member sizes are rewritten) */
     } /* end if */
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
+    FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5FD_family_query() */
 
 
@@ -1027,15 +1029,10 @@ static haddr_t
 H5FD_family_get_eoa(const H5FD_t *_file, H5FD_mem_t UNUSED type)
 {
     const H5FD_family_t	*file = (const H5FD_family_t*)_file;
-    haddr_t ret_value;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_get_eoa, HADDR_UNDEF)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
-    /* Set return value */
-    ret_value=file->eoa;
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
+    FUNC_LEAVE_NOAPI(file->eoa)
 }
 
 
@@ -1067,7 +1064,7 @@ H5FD_family_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t abs_eoa)
     unsigned		u;                      /* Local index variable */
     herr_t              ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_set_eoa, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     for(u = 0; addr || u < file->nmembs; u++) {
 
@@ -1086,7 +1083,7 @@ H5FD_family_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t abs_eoa)
         /* Create another file if necessary */
         if(u >= file->nmembs || !file->memb[u]) {
             file->nmembs = MAX(file->nmembs, u+1);
-            sprintf(memb_name, file->name, u);
+            HDsnprintf(memb_name, sizeof(memb_name), file->name, u);
             H5E_BEGIN_TRY {
                 H5_CHECK_OVERFLOW(file->memb_size, hsize_t, haddr_t);
                 file->memb[u] = H5FDopen(memb_name, file->flags | H5F_ACC_CREAT,
@@ -1145,7 +1142,7 @@ H5FD_family_get_eof(const H5FD_t *_file)
     int			i;      /* Local index variable */
     haddr_t ret_value;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_get_eof, HADDR_UNDEF)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /*
      * Find the last member that has a non-zero EOF and break out of the loop
@@ -1172,7 +1169,6 @@ H5FD_family_get_eof(const H5FD_t *_file)
     /* Set return value */
     ret_value = MAX(eof, file->eoa);
 
-done:
     FUNC_LEAVE_NOAPI(ret_value)
 }
 
@@ -1200,7 +1196,7 @@ H5FD_family_get_handle(H5FD_t *_file, hid_t fapl, void** file_handle)
     int                 memb;
     herr_t              ret_value;
 
-    FUNC_ENTER_NOAPI(H5FD_family_get_handle, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Get the plist structure and family offset */
     if(NULL == (plist = H5P_object_verify(fapl, H5P_FILE_ACCESS)))
@@ -1252,7 +1248,7 @@ H5FD_family_read(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr, si
     H5P_genplist_t      *plist;      /* Property list pointer */
     herr_t              ret_value=SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_read, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /*
      * Get the member data transfer property list. If the transfer property
@@ -1329,7 +1325,7 @@ H5FD_family_write(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr, s
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t      ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_write, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /*
      * Get the member data transfer property list. If the transfer property
@@ -1394,7 +1390,7 @@ H5FD_family_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing)
     unsigned		u, nerrors = 0;
     herr_t      ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_flush, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     for(u = 0; u < file->nmembs; u++)
         if(file->memb[u] && H5FD_flush(file->memb[u], dxpl_id, closing) < 0)
@@ -1429,7 +1425,7 @@ H5FD_family_truncate(H5FD_t *_file, hid_t dxpl_id, unsigned closing)
     unsigned		u, nerrors = 0;
     herr_t      	ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_family_truncate, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     for(u = 0; u < file->nmembs; u++)
         if(file->memb[u] && H5FD_truncate(file->memb[u], dxpl_id, closing) < 0)
