@@ -254,7 +254,7 @@ DESCRIPTION
 static herr_t
 H5FD_log_init_interface(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_init_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     FUNC_LEAVE_NOAPI(H5FD_log_init())
 } /* H5FD_log_init_interface() */
@@ -279,7 +279,7 @@ H5FD_log_init(void)
 {
     hid_t ret_value;            /* Return value */
 
-    FUNC_ENTER_NOAPI(H5FD_log_init, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     if(H5I_VFL != H5I_get_type(H5FD_LOG_g))
         H5FD_LOG_g = H5FD_register(&H5FD_log_g, sizeof(H5FD_class_t), FALSE);
@@ -307,7 +307,7 @@ done:
 void
 H5FD_log_term(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_term)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Reset VFL ID */
     H5FD_LOG_g = 0;
@@ -336,7 +336,7 @@ H5Pset_fapl_log(hid_t fapl_id, const char *logfile, unsigned long long flags, si
     H5P_genplist_t      *plist;     /* Property list pointer */
     herr_t              ret_value;  /* Return value */
 
-    FUNC_ENTER_API(H5Pset_fapl_log, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "i*sULz", fapl_id, logfile, flags, buf_size);
 
     /* Check arguments */
@@ -379,7 +379,7 @@ H5FD_log_fapl_get(H5FD_t *_file)
     H5FD_log_t	*file = (H5FD_log_t *)_file;
     void *ret_value;    /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_fapl_get)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Set return value */
     ret_value = H5FD_log_fapl_copy(&(file->fa));
@@ -408,7 +408,7 @@ H5FD_log_fapl_copy(const void *_old_fa)
     H5FD_log_fapl_t *new_fa = NULL;    /* New FAPL info */
     void *ret_value;    /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5FD_log_fapl_copy)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(old_fa);
 
@@ -457,7 +457,7 @@ H5FD_log_fapl_free(void *_fa)
 {
     H5FD_log_fapl_t	*fa = (H5FD_log_fapl_t*)_fa;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_fapl_free)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Free the fapl information */
     if(fa->logfile)
@@ -502,7 +502,7 @@ H5FD_log_open(const char *name, unsigned flags, hid_t fapl_id, haddr_t maxaddr)
     h5_stat_t	sb;
     H5FD_t	*ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5FD_log_open)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Sanity check on file offsets */
     HDcompile_assert(sizeof(HDoff_t) >= sizeof(size_t));
@@ -698,7 +698,7 @@ H5FD_log_close(H5FD_t *_file)
 #endif /* H5_HAVE_GETTIMEOFDAY */
     herr_t ret_value = SUCCEED;                 /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5FD_log_close)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Sanity check */
     HDassert(file);
@@ -846,7 +846,7 @@ H5FD_log_cmp(const H5FD_t *_f1, const H5FD_t *_f2)
     const H5FD_log_t	*f2 = (const H5FD_log_t *)_f2;
     int ret_value = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_cmp)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
 #ifdef H5_HAVE_WIN32_API
     if(f1->dwVolumeSerialNumber < f2->dwVolumeSerialNumber) HGOTO_DONE(-1)
@@ -904,7 +904,7 @@ H5FD_log_query(const H5FD_t *_file, unsigned long *flags /* out */)
 {
     const H5FD_log_t	*file = (const H5FD_log_t *)_file;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_query)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Set the VFL feature flags that this driver supports */
     if(flags) {
@@ -916,7 +916,7 @@ H5FD_log_query(const H5FD_t *_file, unsigned long *flags /* out */)
         *flags |= H5FD_FEAT_POSIX_COMPAT_HANDLE; /* VFD handle is POSIX I/O call compatible */
 
         /* Check for flags that are set by h5repart */
-        if(file->fam_to_sec2)
+        if(file && file->fam_to_sec2)
             *flags |= H5FD_FEAT_IGNORE_DRVRINFO; /* Ignore the driver info when file is opened (which eliminates it) */
     } /* end if */
 
@@ -945,7 +945,7 @@ H5FD_log_alloc(H5FD_t *_file, H5FD_mem_t type, hid_t UNUSED dxpl_id, hsize_t siz
     haddr_t addr;
     haddr_t ret_value;          /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_alloc)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Compute the address for the block to allocate */
     addr = file->eoa;
@@ -998,7 +998,7 @@ H5FD_log_get_eoa(const H5FD_t *_file, H5FD_mem_t UNUSED type)
 {
     const H5FD_log_t	*file = (const H5FD_log_t *)_file;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_get_eoa)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     FUNC_LEAVE_NOAPI(file->eoa)
 } /* end H5FD_log_get_eoa() */
@@ -1024,7 +1024,7 @@ H5FD_log_set_eoa(H5FD_t *_file, H5FD_mem_t UNUSED type, haddr_t addr)
 {
     H5FD_log_t	*file = (H5FD_log_t *)_file;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_set_eoa)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     file->eoa = addr;
 
@@ -1054,7 +1054,7 @@ H5FD_log_get_eof(const H5FD_t *_file)
 {
     const H5FD_log_t	*file = (const H5FD_log_t *)_file;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_log_get_eof)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     FUNC_LEAVE_NOAPI(MAX(file->eof, file->eoa))
 } /* end H5FD_log_get_eof() */
@@ -1079,7 +1079,7 @@ H5FD_log_get_handle(H5FD_t *_file, hid_t UNUSED fapl, void **file_handle)
     H5FD_log_t          *file = (H5FD_log_t *)_file;
     herr_t              ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5FD_log_get_handle)
+    FUNC_ENTER_NOAPI_NOINIT
 
     if(!file_handle)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "file handle not valid")
@@ -1120,7 +1120,7 @@ H5FD_log_read(H5FD_t *_file, H5FD_mem_t type, hid_t UNUSED dxpl_id, haddr_t addr
 #endif /* H5_HAVE_GETTIMEOFDAY */
     herr_t              ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5FD_log_read)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(file && file->pub.cls);
     HDassert(buf);
@@ -1321,7 +1321,7 @@ H5FD_log_write(H5FD_t *_file, H5FD_mem_t type, hid_t UNUSED dxpl_id, haddr_t add
 #endif /* H5_HAVE_GETTIMEOFDAY */
     herr_t              ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5FD_log_write)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(file && file->pub.cls);
     HDassert(size > 0);
@@ -1519,7 +1519,7 @@ H5FD_log_truncate(H5FD_t *_file, hid_t UNUSED dxpl_id, hbool_t UNUSED closing)
     H5FD_log_t	*file = (H5FD_log_t *)_file;
     herr_t ret_value = SUCCEED;                 /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5FD_log_truncate)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(file);
 
