@@ -115,6 +115,11 @@ typedef struct {
     H5I_id_info_t **id_list;	/*pointer to an array of ptrs to IDs	    */
 } H5I_id_type_t;
 
+typedef struct {
+    H5I_search_func_t app_cb;   /* Application's callback routine */
+    void *app_key;              /* Application's "key" (user data) */
+    void *ret_obj;              /* Object to return */
+} H5I_search_ud_t;
 
 /*-------------------- Locally scoped variables -----------------------------*/
 
@@ -134,6 +139,7 @@ H5FL_DEFINE_STATIC(H5I_id_info_t);
 
 /*--------------------- Local function prototypes ---------------------------*/
 static H5I_id_info_t *H5I_find_id(hid_t id);
+static int H5I_search_cb(void *obj, hid_t id, void *udata);
 #ifdef H5I_DEBUG_OUTPUT
 static herr_t H5I_debug(H5I_type_t type);
 #endif /* H5I_DEBUG_OUTPUT */
@@ -154,7 +160,7 @@ DESCRIPTION
 static herr_t
 H5I_init_interface(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5I_init_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5I_init_interface() */
@@ -183,7 +189,7 @@ H5I_term_interface(void)
     H5I_type_t		type;
     int		n = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5I_term_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     if(H5_interface_initialize_g) {
         /* How many types are still being used? */
@@ -235,7 +241,7 @@ H5Iregister_type(size_t hash_size, unsigned reserved, H5I_free_t free_func)
 {
     H5I_type_t ret_value;       /* Return value */
 
-    FUNC_ENTER_API(H5Iregister_type, H5I_BADID)
+    FUNC_ENTER_API(H5I_BADID)
     H5TRACE3("It", "zIux", hash_size, reserved, free_func);
 
     /* Call H5I_register_type with a value of 0 to get a new type */
@@ -277,7 +283,7 @@ H5I_register_type(H5I_type_t type_id, size_t hash_size, unsigned reserved,
     H5I_id_type_t	*type_ptr = NULL;		/*ptr to the atomic type*/
     H5I_type_t ret_value = H5I_BADID;                   /* type ID to return */
 
-    FUNC_ENTER_NOAPI(H5I_register_type, H5I_BADID)
+    FUNC_ENTER_NOAPI(H5I_BADID)
 
     /* Check that type_id is either a library type or zero */
     if(type_id < 0 || type_id >= H5I_NTYPES)
@@ -389,7 +395,7 @@ H5Itype_exists(H5I_type_t type)
 {
     htri_t ret_value = TRUE;                      /* Return value */
 
-    FUNC_ENTER_API(H5Itype_exists, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("t", "It", type);
 
     if(type <= H5I_BADID || type >= H5I_next_type)
@@ -425,7 +431,7 @@ H5Inmembers(H5I_type_t type, hsize_t *num_members)
 {
     int ret_value = SUCCEED;                      /* Return value */
 
-    FUNC_ENTER_API(H5Inmembers, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "It*h", type, num_members);
 
     if(H5I_IS_LIB_TYPE(type))
@@ -475,7 +481,7 @@ H5I_nmembers(H5I_type_t type)
     H5I_id_type_t	*type_ptr = NULL;
     int		ret_value;
 
-    FUNC_ENTER_NOAPI(H5I_nmembers, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     if(type <= H5I_BADID || type >= H5I_next_type)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, FAIL, "invalid type number")
@@ -511,7 +517,7 @@ H5Iclear_type(H5I_type_t type, hbool_t force)
 {
     herr_t ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Iclear_type, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "Itb", type, force);
 
     if(H5I_IS_LIB_TYPE(type))
@@ -564,7 +570,7 @@ H5I_clear_type(H5I_type_t type, hbool_t force, hbool_t app_ref)
     unsigned	i;                      /* Local index variable */
     int		ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_clear_type, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     if(type <= H5I_BADID || type >= H5I_next_type)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, FAIL, "invalid type number")
@@ -701,7 +707,7 @@ H5Idestroy_type(H5I_type_t type)
 {
     herr_t ret_value;           /* Return value */
 
-    FUNC_ENTER_API(H5Idestroy_type, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "It", type);
 
     if(H5I_IS_LIB_TYPE(type))
@@ -735,7 +741,7 @@ H5I_destroy_type(H5I_type_t type)
     H5I_id_type_t *type_ptr;	/* ptr to the atomic type */
     herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_destroy_type, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     if(type <= H5I_BADID || type >= H5I_next_type)
         HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, FAIL, "invalid type number")
@@ -776,7 +782,7 @@ H5Iregister(H5I_type_t type, const void *object)
 {
     hid_t ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Iregister, H5I_INVALID_HID)
+    FUNC_ENTER_API(H5I_INVALID_HID)
     H5TRACE2("i", "It*x", type, object);
 
     if(H5I_IS_LIB_TYPE(type))
@@ -825,7 +831,7 @@ H5I_register(H5I_type_t type, const void *object, hbool_t app_ref)
     unsigned		i;		/*counter			*/
     hid_t		ret_value = SUCCEED; /*return value		*/
 
-    FUNC_ENTER_NOAPI(H5I_register, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Check arguments */
     if(type <= H5I_BADID || type >= H5I_next_type)
@@ -950,7 +956,7 @@ H5I_subst(hid_t id, const void *new_object)
     H5I_id_info_t	*id_ptr;	/* Ptr to the atom	*/
     void		*ret_value;	/* Return value		*/
 
-    FUNC_ENTER_NOAPI(H5I_subst, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     /* General lookup of the ID */
     if(NULL == (id_ptr = H5I_find_id(id)))
@@ -987,7 +993,7 @@ H5I_object(hid_t id)
     H5I_id_info_t	*id_ptr;		/*ptr to the new atom	*/
     void		*ret_value = NULL;	/*return value		*/
 
-    FUNC_ENTER_NOAPI(H5I_object, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     /* General lookup of the ID */
     if(NULL != (id_ptr = H5I_find_id(id))) {
@@ -1023,7 +1029,7 @@ H5Iobject_verify(hid_t id, H5I_type_t id_type)
 {
     void * ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Iobject_verify, NULL)
+    FUNC_ENTER_API(NULL)
 
     if(H5I_IS_LIB_TYPE(id_type))
         HGOTO_ERROR(H5E_ATOM, H5E_BADGROUP, NULL, "cannot call public function on library type")
@@ -1059,7 +1065,7 @@ H5I_object_verify(hid_t id, H5I_type_t id_type)
     H5I_id_info_t	*id_ptr = NULL;		/*ptr to the new atom	*/
     void		*ret_value = NULL;	/*return value		*/
 
-    FUNC_ENTER_NOAPI(H5I_object_verify, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     HDassert(id_type >= 1 && id_type < H5I_next_type);
 
@@ -1096,7 +1102,7 @@ H5I_get_type(hid_t id)
 {
     H5I_type_t		ret_value = H5I_BADID;
 
-    FUNC_ENTER_NOAPI(H5I_get_type, H5I_BADID)
+    FUNC_ENTER_NOAPI(H5I_BADID)
 
     if(id > 0)
         ret_value = H5I_TYPE(id);
@@ -1128,7 +1134,7 @@ H5Iget_type(hid_t id)
 {
     H5I_type_t		ret_value = H5I_BADID;          /* Return value */
 
-    FUNC_ENTER_API(H5Iget_type, H5I_BADID)
+    FUNC_ENTER_API(H5I_BADID)
     H5TRACE1("It", "i", id);
 
     ret_value = H5I_get_type(id);
@@ -1163,7 +1169,7 @@ H5Iremove_verify(hid_t id, H5I_type_t id_type)
 {
     void * ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Iremove_verify, NULL)
+    FUNC_ENTER_API(NULL)
 
     if(H5I_IS_LIB_TYPE(id_type))
         HGOTO_ERROR(H5E_ATOM, H5E_BADGROUP, NULL, "cannot call public function on library type")
@@ -1197,7 +1203,7 @@ H5I_remove_verify(hid_t id, H5I_type_t id_type)
 {
     void * ret_value = NULL;	/*return value			*/
 
-    FUNC_ENTER_NOAPI(H5I_remove_verify, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     /* Argument checking will be performed by H5I_remove() */
 
@@ -1236,7 +1242,7 @@ H5I_remove(hid_t id)
     unsigned		hash_loc;	/*atom's hash table location	*/
     void *	      ret_value = NULL;	/*return value			*/
 
-    FUNC_ENTER_NOAPI(H5I_remove, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     /* Check arguments */
     type = H5I_TYPE(id);
@@ -1328,7 +1334,7 @@ H5Idec_ref(hid_t id)
 {
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Idec_ref, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Is", "i", id);
 
     /* Check arguments */
@@ -1369,7 +1375,7 @@ H5I_dec_ref(hid_t id)
     H5I_id_info_t	*id_ptr;	/*ptr to the new ID	*/
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_dec_ref, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
     HDassert(id >= 0);
@@ -1440,7 +1446,7 @@ H5I_dec_app_ref(hid_t id)
     H5I_id_info_t	*id_ptr;	/*ptr to the new ID	*/
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_dec_app_ref, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
     HDassert(id >= 0);
@@ -1487,7 +1493,7 @@ H5I_dec_app_ref_always_close(hid_t id)
 {
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_dec_app_ref_always_close, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
     HDassert(id >= 0);
@@ -1531,7 +1537,7 @@ H5Iinc_ref(hid_t id)
 {
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Iinc_ref, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Is", "i", id);
 
     /* Check arguments */
@@ -1575,7 +1581,7 @@ H5I_inc_ref(hid_t id, hbool_t app_ref)
     H5I_id_info_t	*id_ptr;	/*ptr to the ID		*/
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_inc_ref, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
     HDassert(id >= 0);
@@ -1623,7 +1629,7 @@ H5Iget_ref(hid_t id)
 {
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Iget_ref, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Is", "i", id);
 
     /* Check arguments */
@@ -1667,7 +1673,7 @@ H5I_get_ref(hid_t id, hbool_t app_ref)
     H5I_id_info_t	*id_ptr;	/*ptr to the ID		*/
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_get_ref, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
     HDassert(id >= 0);
@@ -1711,7 +1717,7 @@ H5Iinc_type_ref(H5I_type_t type)
 {
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Iinc_type_ref, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Is", "It", type);
 
     /* Check arguments */
@@ -1750,7 +1756,7 @@ H5I_inc_type_ref(H5I_type_t type)
     H5I_id_type_t	*type_ptr;	/* ptr to the type	*/
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_inc_type_ref, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
     HDassert(type > 0 && type < H5I_next_type);
@@ -1795,7 +1801,7 @@ H5Idec_type_ref(H5I_type_t type)
 {
     herr_t ret_value;           /* Return value */
 
-    FUNC_ENTER_API(H5Idec_type_ref, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "It", type);
 
     if(H5I_IS_LIB_TYPE(type))
@@ -1834,7 +1840,7 @@ H5I_dec_type_ref(H5I_type_t type)
     H5I_id_type_t	*type_ptr;      /* Pointer to the ID type */
     herr_t		ret_value;      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_dec_type_ref, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     if(type <= H5I_BADID || type >= H5I_next_type)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, FAIL, "invalid type number")
@@ -1882,7 +1888,7 @@ H5Iget_type_ref(H5I_type_t type)
 {
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_API(H5Iget_type_ref, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Is", "It", type);
 
     /* Check arguments */
@@ -1922,7 +1928,7 @@ H5I_get_type_ref(H5I_type_t type)
     H5I_id_type_t	*type_ptr;	/*ptr to the type	*/
     int ret_value;                      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_get_type_ref, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Sanity check */
     HDassert(type >= 0);
@@ -1961,7 +1967,7 @@ H5Iis_valid(hid_t id)
     H5I_id_info_t   *id_ptr;            /* ptr to the ID */
     htri_t          ret_value = TRUE;   /* Return value */
 
-    FUNC_ENTER_API(H5Iis_valid, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("t", "i", id);
 
     /* Find the ID */
@@ -1976,6 +1982,40 @@ done:
     FUNC_LEAVE_API(ret_value)
 } /* end H5Iis_valid() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5I_search_cb
+ *
+ * Purpose:	Callback routine for H5Isearch, when it calls H5I_iterate.
+ *		Calls "user" callback search function, and then sets return
+ *		value, based on the result of that callback.
+ *
+ * Return:	Success:	The first object in the type for which FUNC
+ *				returns non-zero. NULL if FUNC returned zero
+ *				for every object in the type.
+ *		Failure:	NULL
+ *
+ * Programmer:	Quincey Koziol
+ *		Friday, March 30, 2012
+ *
+ *-------------------------------------------------------------------------
+ */
+static int
+H5I_search_cb(void *obj, hid_t id, void *_udata)
+{
+    H5I_search_ud_t *udata = (H5I_search_ud_t *)_udata; /* User data for callback */
+    int ret_value;     /* Callback return value */
+
+    FUNC_ENTER_NOAPI_NOINIT
+
+    ret_value = (*udata->app_cb)(obj, id, udata->app_key);
+    if(ret_value > 0)
+        udata->ret_obj = obj;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5I_search_cb() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5Isearch
@@ -2004,14 +2044,27 @@ done:
 void *
 H5Isearch(H5I_type_t type, H5I_search_func_t func, void *key)
 {
-    void * ret_value;                      /* Return value */
+    H5I_search_ud_t udata;      /* Context for iteration */
+    void *ret_value;            /* Return value */
 
-    FUNC_ENTER_API(H5Isearch, NULL)
+    FUNC_ENTER_API(NULL)
 
+    /* Check arguments */
     if(H5I_IS_LIB_TYPE(type))
         HGOTO_ERROR(H5E_ATOM, H5E_BADGROUP, NULL, "cannot call public function on library type")
 
-    ret_value = H5I_search(type, func, key, TRUE);
+    /* Set up udata struct */
+    udata.app_cb = func;
+    udata.app_key = key;
+    udata.ret_obj = NULL;
+
+    /* Note that H5I_iterate returns an error code.  We ignore it 
+     * here, as we can't do anything with it without revising the API.
+     */
+    H5I_iterate(type, H5I_search_cb, &udata, TRUE);
+
+    /* Set return value */
+    ret_value = udata.ret_obj;
 
 done:
     FUNC_LEAVE_API(ret_value)
@@ -2019,62 +2072,71 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5I_search
+ * Function:	H5I_iterate
  *
- * Purpose:	Apply function FUNC to each member of type TYPE and return a
- *		pointer to the first object for which FUNC returns non-zero.
- *		The FUNC should take a pointer to the object and the KEY as
- *		arguments and return non-zero to terminate the search (zero
- *		to continue).
+ * Purpose:	Apply function FUNC to each member of type TYPE (with 
+ *		non-zero application reference count if app_ref is TRUE).  
+ *		Stop if FUNC returns a non zero value (i.e. anything 
+ *		other than H5_ITER_CONT).  
  *
- * Limitation:	Currently there is no way to start searching from where a
- *		previous search left off.
+ *		If FUNC returns a positive value (i.e. H5_ITER_STOP), 
+ *		return SUCCEED.
  *
- * Return:	Success:	The first object in the type for which FUNC
- *				returns non-zero. NULL if FUNC returned zero
- *				for every object in the type.
- *		Failure:	NULL
+ *		If FUNC returns a negative value (i.e. H5_ITER_ERROR), 
+ *		return FAIL.
+ *		
+ *		The FUNC should take a pointer to the object and the 
+ *		udata as arguments and return non-zero to terminate 
+ *		siteration, and zero to continue.
  *
- * Programmer:	Robb Matzke
- *		Friday, February 19, 1999
+ * Limitation:	Currently there is no way to start the iteration from 
+ *		where a previous iteration left off.
  *
- * Modifications:  Neil Fortner
- *      Wednesday, October 1, 2008
- *      Added app_ref parameter.  When set to TRUE, the function will only
- *      operate on ids that have a nonzero application reference count.
+ * Return:	Success:	SUCCEED
+ *		Failure:	FAIL
+ *
+ * Programmer:	John Mainzer
+ *		Monday, December 6, 2011
  *
  *-------------------------------------------------------------------------
  */
-void *
-H5I_search(H5I_type_t type, H5I_search_func_t func, void *key, hbool_t app_ref)
+herr_t
+H5I_iterate(H5I_type_t type, H5I_search_func_t func, void *udata, hbool_t app_ref)
 {
-    H5I_id_type_t	*type_ptr;	/*ptr to the type	*/
-    void		*ret_value = NULL;	/*return value		*/
+    H5I_id_type_t *type_ptr;		/*ptr to the type	*/
+    herr_t	   ret_value = SUCCEED;	/*return value		*/
 
-    FUNC_ENTER_NOAPI(H5I_search, NULL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Check arguments */
     if(type <= H5I_BADID || type >= H5I_next_type)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, NULL, "invalid type number")
+	HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, FAIL, "invalid type number")
     type_ptr = H5I_id_type_list_g[type];
-    if(type_ptr == NULL || type_ptr->count <= 0)
-	HGOTO_ERROR(H5E_ATOM, H5E_BADGROUP, NULL, "invalid type")
 
-    /* Only iterate through hash table if there are IDs in group */
-    if(type_ptr->ids > 0) {
-        H5I_id_info_t	*id_ptr;	/*ptr to the new ID	*/
-        H5I_id_info_t	*next_id;	/*ptr to the next ID	*/
-        unsigned i;			/*counter		*/
+    /* Only iterate through hash table if it is initialized and there are IDs in group */
+    if(type_ptr && type_ptr->count > 0 && type_ptr->ids > 0) {
+        unsigned u;			/* Counter		*/
 
         /* Start at the beginning of the array */
-        for(i = 0; i < type_ptr->hash_size; i++) {
-            id_ptr = type_ptr->id_list[i];
+        for(u = 0; u < type_ptr->hash_size; u++) {
+            H5I_id_info_t	*id_ptr;	/* Ptr to the new ID	*/
+
+            id_ptr = type_ptr->id_list[u];
             while(id_ptr) {
-                next_id = id_ptr->next;      /* Protect against ID being deleted in callback */
-                /* (Casting away const OK -QAK) */
-                if((!app_ref || id_ptr->app_count) && (*func)((void *)id_ptr->obj_ptr, id_ptr->id, key))
-                    /* (Casting away const OK -QAK) */
-                    HGOTO_DONE((void *)id_ptr->obj_ptr);	/*found the item*/
+                H5I_id_info_t	*next_id;	/* Ptr to the next ID	*/
+
+                /* Protect against ID being deleted in callback */
+                next_id = id_ptr->next;
+                if((!app_ref) || (id_ptr->app_count > 0)) {
+                    herr_t cb_ret_val;  /* Callback return value */
+
+                    /* (Casting away const OK) */
+		    cb_ret_val = (*func)((void *)id_ptr->obj_ptr, id_ptr->id, udata);
+		    if(cb_ret_val > 0)
+                        HGOTO_DONE(SUCCEED)	/* terminate iteration early */
+		    else if(cb_ret_val < 0)
+			HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, FAIL, "func failed")
+		} /* end if */
                 id_ptr = next_id;
             } /* end while */
         } /* end for */
@@ -2082,7 +2144,7 @@ H5I_search(H5I_type_t type, H5I_search_func_t func, void *key, hbool_t app_ref)
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5I_search() */
+} /* end H5I_iterate() */
 
 
 /*-------------------------------------------------------------------------
@@ -2109,7 +2171,7 @@ H5I_find_id(hid_t id)
     unsigned		hash_loc;		/*bucket pointer	*/
     H5I_id_info_t	*ret_value;		/*return value		*/
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5I_find_id)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check arguments */
     type = H5I_TYPE(id);
@@ -2179,7 +2241,7 @@ H5Iget_name(hid_t id, char *name/*out*/, size_t size)
     H5G_loc_t     loc;          /* Object location */
     ssize_t       ret_value;    /* Return value */
 
-    FUNC_ENTER_API(H5Iget_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("Zs", "ixz", id, name, size);
 
     /* Get object location */
@@ -2215,7 +2277,7 @@ H5Iget_file_id(hid_t obj_id)
 {
     hid_t ret_value;            /* Return value */
 
-    FUNC_ENTER_API(H5Iget_file_id, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", obj_id);
 
     if((ret_value = H5I_get_file_id(obj_id, TRUE)) < 0)
@@ -2246,7 +2308,7 @@ H5I_get_file_id(hid_t obj_id, hbool_t app_ref)
     H5I_type_t type;            /* ID type */
     hid_t ret_value;            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5I_get_file_id)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Get object type */
     type = H5I_TYPE(obj_id);
@@ -2301,7 +2363,7 @@ H5I_debug(H5I_type_t type)
     unsigned int iu;
     herr_t ret_value;  /* Return value */
 
-    FUNC_ENTER_NOAPI(H5I_debug, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     fprintf(stderr, "Dumping ID type %d\n", (int)type);
     type_ptr = H5I_id_type_list_g[type];
