@@ -134,7 +134,7 @@ H5MF_sect_simple_new(haddr_t sect_off, hsize_t sect_size)
     H5MF_free_section_t *sect = NULL;   /* 'Simple' free space section to add */
     H5MF_free_section_t *ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5MF_sect_simple_new)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments.  */
     HDassert(sect_size);
@@ -180,7 +180,7 @@ H5MF_sect_simple_deserialize(const H5FS_section_class_t UNUSED *cls,
     H5MF_free_section_t *sect;          /* New section */
     H5FS_section_info_t *ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5MF_sect_simple_deserialize)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments. */
     HDassert(H5F_addr_defined(sect_addr));
@@ -221,7 +221,7 @@ H5MF_sect_simple_can_merge(const H5FS_section_info_t *_sect1,
     const H5MF_free_section_t *sect2 = (const H5MF_free_section_t *)_sect2;   /* File free section */
     htri_t ret_value;                   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5MF_sect_simple_can_merge)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check arguments. */
     HDassert(sect1);
@@ -259,7 +259,7 @@ H5MF_sect_simple_merge(H5FS_section_info_t *_sect1, H5FS_section_info_t *_sect2,
     H5MF_free_section_t *sect2 = (H5MF_free_section_t *)_sect2;   /* File free section */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5MF_sect_simple_merge)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments. */
     HDassert(sect1);
@@ -302,7 +302,7 @@ H5MF_sect_simple_can_shrink(const H5FS_section_info_t *_sect, void *_udata)
     haddr_t end;                /* End of section to extend */
     htri_t ret_value;           /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5MF_sect_simple_can_shrink)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments. */
     HDassert(sect);
@@ -395,7 +395,7 @@ H5MF_sect_simple_shrink(H5FS_section_info_t **_sect, void *_udata)
     H5MF_sect_ud_t *udata = (H5MF_sect_ud_t *)_udata;   /* User data for callback */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5MF_sect_simple_shrink)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments. */
     HDassert(sect);
@@ -404,8 +404,11 @@ H5MF_sect_simple_shrink(H5FS_section_info_t **_sect, void *_udata)
 
     /* Check for shrinking file */
     if(H5MF_SHRINK_EOA == udata->shrink) {
+        /* Sanity check */
+        HDassert(H5F_INTENT(udata->f) & H5F_ACC_RDWR);
+
         /* Release section's space at EOA with file driver */
-        if(H5FD_free(udata->f->shared->lf, udata->dxpl_id, udata->alloc_type, (*sect)->sect_info.addr, (*sect)->sect_info.size) < 0)
+        if(H5FD_free(udata->f->shared->lf, udata->dxpl_id, udata->alloc_type, udata->f, (*sect)->sect_info.addr, (*sect)->sect_info.size) < 0)
             HGOTO_ERROR(H5E_RESOURCE, H5E_CANTFREE, FAIL, "driver free request failed")
     } /* end if */
     else {
@@ -450,13 +453,13 @@ H5MF_sect_simple_free(H5FS_section_info_t *_sect)
 {
     H5MF_free_section_t *sect = (H5MF_free_section_t *)_sect;   /* File free section */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5MF_sect_simple_free)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check arguments. */
     HDassert(sect);
 
     /* Release the section */
-    (void)H5FL_FREE(H5MF_free_section_t, sect);
+    sect = H5FL_FREE(H5MF_free_section_t, sect);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 }   /* H5MF_sect_simple_free() */
@@ -477,7 +480,7 @@ H5MF_sect_simple_free(H5FS_section_info_t *_sect)
  */
 static herr_t
 H5MF_sect_simple_valid(const H5FS_section_class_t UNUSED *cls,
-    const H5FS_section_info_t 
+    const H5FS_section_info_t
 #ifdef NDEBUG
     UNUSED
 #endif /* NDEBUG */
@@ -487,7 +490,7 @@ H5MF_sect_simple_valid(const H5FS_section_class_t UNUSED *cls,
     const H5MF_free_section_t *sect = (const H5MF_free_section_t *)_sect;   /* File free section */
 #endif /* NDEBUG */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5MF_sect_simple_valid)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check arguments. */
     HDassert(sect);
@@ -514,7 +517,7 @@ H5MF_sect_simple_split(H5FS_section_info_t *sect, hsize_t frag_size)
 {
     H5MF_free_section_t *ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5MF_sect_simple_split)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Allocate space for new section */
     if(NULL == (ret_value = H5MF_sect_simple_new(sect->addr, frag_size)))
@@ -528,4 +531,3 @@ done:
     FUNC_LEAVE_NOAPI((H5FS_section_info_t *)ret_value)
 } /* end H5MF_sect_simple_split() */
 
-
