@@ -61,6 +61,7 @@ static herr_t H5O_attr_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg,
 #define H5O_SHARED_COPY_FILE_REAL	H5O_attr_copy_file
 #define H5O_SHARED_POST_COPY_FILE	H5O_attr_shared_post_copy_file
 #define H5O_SHARED_POST_COPY_FILE_REAL	H5O_attr_post_copy_file
+#undef  H5O_SHARED_POST_COPY_FILE_UPD
 #define H5O_SHARED_DEBUG		H5O_attr_shared_debug
 #define H5O_SHARED_DEBUG_REAL		H5O_attr_debug
 #include "H5Oshared.h"			/* Shared Object Header Message Callbacks */
@@ -130,7 +131,7 @@ H5O_attr_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh, unsigned UNUSED mesg_fl
     unsigned            flags = 0;      /* Attribute flags */
     H5A_t		*ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_decode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(f);
@@ -171,7 +172,7 @@ H5O_attr_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh, unsigned UNUSED mesg_fl
      * as well as some reserved bytes.
      */
     if(attr->shared->version >= H5O_ATTR_VERSION_3)
-        attr->shared->encoding = *p++;
+        attr->shared->encoding = (H5T_cset_t)*p++;
 
     /* Decode and store the name */
     if(NULL == (attr->shared->name = H5MM_strdup((const char *)p)))
@@ -205,7 +206,7 @@ H5O_attr_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh, unsigned UNUSED mesg_fl
     HDmemcpy(&(attr->shared->ds->extent), extent, sizeof(H5S_extent_t));
 
     /* Release temporary extent information */
-    (void)H5FL_FREE(H5S_extent_t, extent);
+    extent = H5FL_FREE(H5S_extent_t, extent);
 
     /* Default to entire dataspace being selected */
     if(H5S_select_all(attr->shared->ds, FALSE) < 0)
@@ -226,9 +227,6 @@ H5O_attr_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh, unsigned UNUSED mesg_fl
         HDmemcpy(attr->shared->data, p, attr->shared->data_size);
     } /* end if */
 
-    /* Indicate that the fill values aren't to be written out */
-    attr->shared->initialized = 1;
-
     /* Increment the reference count for this object header message in cache(compact
        storage) or for the object from dense storage. */
     attr->shared->nrefs++;
@@ -237,6 +235,20 @@ H5O_attr_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh, unsigned UNUSED mesg_fl
     ret_value = attr;
 
 done:
+    if(NULL == ret_value)
+        if(attr) {
+            if(attr->shared) {
+                /* Free any dynamicly allocated items */
+                if(H5A_free(attr) < 0)
+                    HDONE_ERROR(H5E_ATTR, H5E_CANTRELEASE, NULL, "can't release attribute info")
+
+                /* Destroy shared attribute struct */
+                attr->shared = H5FL_FREE(H5A_shared_t, attr->shared);
+            } /* end if */
+
+            attr = H5FL_FREE(H5A_t, attr);
+        } /* end if */
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_attr_decode() */
 
@@ -267,7 +279,7 @@ H5O_attr_encode(H5F_t *f, uint8_t *p, const void *mesg)
     unsigned    flags = 0;      /* Attribute flags */
     herr_t      ret_value = SUCCEED;      /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_encode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(f);
@@ -370,7 +382,7 @@ H5O_attr_copy(const void *_src, void *_dst)
 {
     void *ret_value;            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_copy)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(_src);
@@ -407,7 +419,7 @@ H5O_attr_size(const H5F_t UNUSED *f, const void *_mesg)
     size_t		name_len;
     size_t		ret_value = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_attr_size)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     HDassert(attr);
 
@@ -468,7 +480,7 @@ H5O_attr_size(const H5F_t UNUSED *f, const void *_mesg)
 herr_t
 H5O_attr_reset(void UNUSED *_mesg)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_attr_reset)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5O_attr_reset() */
@@ -496,7 +508,7 @@ H5O_attr_free(void *mesg)
     H5A_t *attr = (H5A_t *)mesg;
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_free)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(mesg);
 
@@ -526,7 +538,7 @@ H5O_attr_delete(H5F_t *f, hid_t dxpl_id, H5O_t *oh, void *_mesg)
     H5A_t *attr = (H5A_t *) _mesg;
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_delete)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(f);
@@ -564,7 +576,7 @@ H5O_attr_link(H5F_t *f, hid_t dxpl_id, H5O_t *oh, void *_mesg)
     H5A_t *attr = (H5A_t *) _mesg;
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_link)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(f);
@@ -605,7 +617,7 @@ static herr_t
 H5O_attr_pre_copy_file(H5F_t UNUSED *file_src, const void UNUSED *native_src,
     hbool_t *deleted, const H5O_copy_t *cpy_info, void UNUSED *udata)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_attr_pre_copy_file)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check args */
     HDassert(deleted);
@@ -636,13 +648,13 @@ H5O_attr_pre_copy_file(H5F_t UNUSED *file_src, const void UNUSED *native_src,
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_attr_copy_file(H5F_t UNUSED *file_src, const H5O_msg_class_t UNUSED *mesg_type,
+H5O_attr_copy_file(H5F_t *file_src, const H5O_msg_class_t UNUSED *mesg_type,
     void *native_src, H5F_t *file_dst, hbool_t *recompute_size,
     H5O_copy_t *cpy_info, void UNUSED *udata, hid_t dxpl_id)
 {
     void        *ret_value;             /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_copy_file)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(native_src);
@@ -650,7 +662,13 @@ H5O_attr_copy_file(H5F_t UNUSED *file_src, const H5O_msg_class_t UNUSED *mesg_ty
     HDassert(cpy_info);
     HDassert(!cpy_info->copy_without_attr);
 
-    if ( NULL == (ret_value=H5A_attr_copy_file((H5A_t *)native_src, file_dst, recompute_size, cpy_info,  dxpl_id)))
+    /* Mark datatype as being on disk now.  This step used to be done in a lower level
+     * by H5O_dtype_decode.  But it has been moved up.  Not an ideal place, but no better
+     * place than here. */
+    if(H5T_set_loc(((H5A_t *)native_src)->shared->dt, file_src, H5T_LOC_DISK) < 0)
+        HGOTO_ERROR(H5E_ATTR, H5E_CANTINIT, NULL, "invalid datatype location")
+
+    if(NULL == (ret_value = H5A_attr_copy_file((H5A_t *)native_src, file_dst, recompute_size, cpy_info,  dxpl_id)))
         HGOTO_ERROR(H5E_ATTR, H5E_CANTCOPY, NULL, "can't copy attribute")
 
 done:
@@ -681,7 +699,7 @@ H5O_attr_post_copy_file(const H5O_loc_t *src_oloc, const void *mesg_src,
     herr_t ret_value = SUCCEED;   /* Return value */
 
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_post_copy_file)
+    FUNC_ENTER_NOAPI_NOINIT
 
     if ( H5A_attr_post_copy_file(src_oloc, (const H5A_t *)mesg_src,
          dst_oloc, (H5A_t *)mesg_dst, dxpl_id, cpy_info) < 0)
@@ -710,7 +728,7 @@ H5O_attr_get_crt_index(const void *_mesg, H5O_msg_crt_idx_t *crt_idx /*out*/)
 {
     const H5A_t  *attr = (const H5A_t *)_mesg;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_attr_get_crt_index)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     HDassert(attr);
     HDassert(crt_idx);
@@ -740,7 +758,7 @@ H5O_attr_set_crt_index(void *_mesg, H5O_msg_crt_idx_t crt_idx)
 {
     H5A_t  *attr = (H5A_t *)_mesg;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_attr_set_crt_index)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     HDassert(attr);
 
@@ -778,7 +796,7 @@ H5O_attr_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg, FILE * stream, int in
     char		buf[256];       /* Temporary string buffer */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_attr_debug)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(f);
@@ -793,20 +811,38 @@ H5O_attr_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg, FILE * stream, int in
         case H5T_CSET_ASCII:
             s = "ASCII";
             break;
+
         case H5T_CSET_UTF8:
             s = "UTF-8";
             break;
-        default:
+
+        case H5T_CSET_RESERVED_2:
+        case H5T_CSET_RESERVED_3:
+        case H5T_CSET_RESERVED_4:
+        case H5T_CSET_RESERVED_5:
+        case H5T_CSET_RESERVED_6:
+        case H5T_CSET_RESERVED_7:
+        case H5T_CSET_RESERVED_8:
+        case H5T_CSET_RESERVED_9:
+        case H5T_CSET_RESERVED_10:
+        case H5T_CSET_RESERVED_11:
+        case H5T_CSET_RESERVED_12:
+        case H5T_CSET_RESERVED_13:
+        case H5T_CSET_RESERVED_14:
+        case H5T_CSET_RESERVED_15:
             sprintf(buf, "H5T_CSET_RESERVED_%d", (int)(mesg->shared->encoding));
             s = buf;
             break;
+
+        case H5T_CSET_ERROR:
+        default:
+            sprintf(buf, "Unknown character set: %d", (int)(mesg->shared->encoding));
+            s = buf;
+            break;
     } /* end switch */
     fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
             "Character Set of Name:",
             s);
-    HDfprintf(stream, "%*s%-*s %t\n", indent, "", fwidth,
-	    "Initialized:",
-	    mesg->shared->initialized);
     HDfprintf(stream, "%*s%-*s %t\n", indent, "", fwidth,
 	    "Object opened:",
 	    mesg->obj_opened);
