@@ -29,14 +29,15 @@
 #include "H5Opkg.h"             /* Object headers			*/
 
 /* PRIVATE PROTOTYPES */
-static void *H5O_efl_decode(H5F_t *f, hid_t dxpl_id, unsigned mesg_flags, const uint8_t *p);
+static void *H5O_efl_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh,
+    unsigned mesg_flags, unsigned *ioflags, const uint8_t *p);
 static herr_t H5O_efl_encode(H5F_t *f, hbool_t disable_shared, uint8_t *p, const void *_mesg);
 static void *H5O_efl_copy(const void *_mesg, void *_dest);
 static size_t H5O_efl_size(const H5F_t *f, hbool_t disable_shared, const void *_mesg);
 static herr_t H5O_efl_reset(void *_mesg);
 static void *H5O_efl_copy_file(H5F_t *file_src, void *mesg_src,
-    H5F_t *file_dst, hbool_t *recompute_size, H5O_copy_t *cpy_info,
-    void *udata, hid_t dxpl_id);
+    H5F_t *file_dst, hbool_t *recompute_size, unsigned *mesg_flags,
+    H5O_copy_t *cpy_info, void *udata, hid_t dxpl_id);
 static herr_t H5O_efl_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg, FILE * stream,
 			    int indent, int fwidth);
 
@@ -80,11 +81,16 @@ const H5O_msg_class_t H5O_MSG_EFL[1] = {{
  * Programmer:	Robb Matzke
  *		Tuesday, November 25, 1997
  *
+ * Modification:
+ *              Raymond Lu
+ *              11 April 2011
+ *              We allow zero dimension size starting from the 1.8.7 release.
+ *              The dataset size of external storage can be zero.
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_efl_decode(H5F_t *f, hid_t dxpl_id, unsigned UNUSED mesg_flags,
-    const uint8_t *p)
+H5O_efl_decode(H5F_t *f, hid_t dxpl_id, H5O_t UNUSED *open_oh,
+    unsigned UNUSED mesg_flags, unsigned UNUSED *ioflags, const uint8_t *p)
 {
     H5O_efl_t		*mesg = NULL;
     int			version;
@@ -93,13 +99,13 @@ H5O_efl_decode(H5F_t *f, hid_t dxpl_id, unsigned UNUSED mesg_flags,
     size_t		u;      /* Local index variable */
     void *ret_value;            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_decode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     HDassert(f);
     HDassert(p);
 
-    if(NULL == (mesg = H5MM_calloc(sizeof(H5O_efl_t))))
+    if(NULL == (mesg = (H5O_efl_t *)H5MM_calloc(sizeof(H5O_efl_t))))
 	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
     /* Version */
@@ -125,17 +131,17 @@ H5O_efl_decode(H5F_t *f, hid_t dxpl_id, unsigned UNUSED mesg_flags,
     if(NULL == (heap = H5HL_protect(f, dxpl_id, mesg->heap_addr, H5AC_READ)))
         HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read protect link value")
 
-    s = H5HL_offset_into(f, heap, 0);
+    s = (const char *)H5HL_offset_into(heap, 0);
 
     HDassert(s && !*s);
 
-    if(H5HL_unprotect(f, dxpl_id, heap, mesg->heap_addr) < 0)
+    if(H5HL_unprotect(heap) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read unprotect link value")
     heap = NULL;
 #endif
 
     /* Decode the file list */
-    mesg->slot = H5MM_calloc(mesg->nalloc * sizeof(H5O_efl_entry_t));
+    mesg->slot = (H5O_efl_entry_t *)H5MM_calloc(mesg->nalloc * sizeof(H5O_efl_entry_t));
     if(NULL == mesg->slot)
 	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
@@ -145,7 +151,7 @@ H5O_efl_decode(H5F_t *f, hid_t dxpl_id, unsigned UNUSED mesg_flags,
 	/* Name */
 	H5F_DECODE_LENGTH (f, p, mesg->slot[u].name_offset);
 
-        s = H5HL_offset_into(f, heap, mesg->slot[u].name_offset);
+        s = (const char *)H5HL_offset_into(heap, mesg->slot[u].name_offset);
 	HDassert(s && *s);
 	mesg->slot[u].name = H5MM_xstrdup (s);
         HDassert(mesg->slot[u].name);
@@ -155,10 +161,9 @@ H5O_efl_decode(H5F_t *f, hid_t dxpl_id, unsigned UNUSED mesg_flags,
 
 	/* Size */
 	H5F_DECODE_LENGTH (f, p, mesg->slot[u].size);
-	HDassert(mesg->slot[u].size > 0);
     } /* end for */
 
-    if(H5HL_unprotect(f, dxpl_id, heap, mesg->heap_addr) < 0)
+    if(H5HL_unprotect(heap) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read unprotect link value")
     heap = NULL;
 
@@ -192,7 +197,7 @@ H5O_efl_encode(H5F_t *f, hbool_t UNUSED disable_shared, uint8_t *p, const void *
     const H5O_efl_t	*mesg = (const H5O_efl_t *)_mesg;
     size_t		u;      /* Local index variable */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_encode)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check args */
     HDassert(f);
@@ -256,32 +261,63 @@ H5O_efl_copy(const void *_mesg, void *_dest)
     size_t		u;              /* Local index variable */
     void                *ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_copy)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(mesg);
     if(!dest) {
-	if(NULL == (dest = H5MM_calloc(sizeof(H5O_efl_t))) ||
-                NULL == (dest->slot = H5MM_malloc(mesg->nalloc * sizeof(H5O_efl_entry_t))))
-	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
-    } else if(dest->nalloc < mesg->nalloc) {
-	H5MM_xfree(dest->slot);
-	if(NULL == (dest->slot = H5MM_malloc(mesg->nalloc * sizeof(H5O_efl_entry_t))))
-	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
-    }
+	if(NULL == (dest = (H5O_efl_t *)H5MM_calloc(sizeof(H5O_efl_t))))
+	    HGOTO_ERROR(H5E_OHDR, H5E_CANTALLOC, NULL, "can't allocate efl message")
+        if(NULL == (dest->slot = (H5O_efl_entry_t *)H5MM_calloc(mesg->nalloc * sizeof(H5O_efl_entry_t))))
+	    HGOTO_ERROR(H5E_OHDR, H5E_CANTALLOC, NULL, "can't allocate efl message slots")
+    } /* end if */
+    else if(dest->nalloc < mesg->nalloc) {
+        H5O_efl_entry_t *temp_slot;     /* Temporary pointer to new slot information */
+
+        /* Allocate new 'slot' information */
+        if(NULL == (temp_slot = (H5O_efl_entry_t *)H5MM_calloc(mesg->nalloc * sizeof(H5O_efl_entry_t))))
+            HGOTO_ERROR(H5E_OHDR, H5E_CANTALLOC, NULL, "can't allocate efl message slots")
+
+        /* Release old 'slot' information */
+        for(u = 0; u < dest->nused; u++)
+            dest->slot[u].name = (char *)H5MM_xfree(dest->slot[u].name);
+        dest->slot = (H5O_efl_entry_t *)H5MM_xfree(dest->slot);
+
+        /* Point to new 'slot' information */
+        dest->slot = temp_slot;
+    } /* end if */
+    else {
+        /* Release old 'slot' information */
+        for(u = 0; u < dest->nused; u++)
+            dest->slot[u].name = (char *)H5MM_xfree(dest->slot[u].name);
+    } /* end else */
     dest->heap_addr = mesg->heap_addr;
     dest->nalloc = mesg->nalloc;
     dest->nused = mesg->nused;
 
     for(u = 0; u < mesg->nused; u++) {
 	dest->slot[u] = mesg->slot[u];
-	dest->slot[u].name = H5MM_xstrdup(mesg->slot[u].name);
+	if(NULL == (dest->slot[u].name = H5MM_xstrdup(mesg->slot[u].name)))
+            HGOTO_ERROR(H5E_OHDR, H5E_CANTALLOC, NULL, "can't allocate efl message slot name")
     } /* end for */
 
     /* Set return value */
     ret_value = dest;
 
 done:
+    if(NULL == ret_value) {
+        if(dest && NULL == _dest) {
+            if(dest->slot) {
+                for(u = 0; u < mesg->nused; u++) {
+                    if(dest->slot[u].name != NULL && dest->slot[u].name != mesg->slot[u].name)
+                        dest->slot[u].name = (char *)H5MM_xfree(dest->slot[u].name);
+                } /* end for */
+                dest->slot = (H5O_efl_entry_t *)H5MM_xfree(dest->slot);
+            } /* end if */
+            dest = (H5O_efl_t *)H5MM_xfree(dest);
+        } /* end if */
+    } /* end if */
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_efl_copy() */
 
@@ -309,7 +345,7 @@ H5O_efl_size(const H5F_t *f, hbool_t UNUSED disable_shared, const void *_mesg)
     const H5O_efl_t	*mesg = (const H5O_efl_t *) _mesg;
     size_t		ret_value = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_size)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check args */
     HDassert(f);
@@ -346,18 +382,19 @@ H5O_efl_reset(void *_mesg)
     H5O_efl_t	*mesg = (H5O_efl_t *) _mesg;
     size_t	u;              /* Local index variable */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_reset)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check args */
     HDassert(mesg);
 
     /* reset */
-    for(u = 0; u < mesg->nused; u++)
-	mesg->slot[u].name = H5MM_xfree(mesg->slot[u].name);
+    if(mesg->slot) {
+        for(u = 0; u < mesg->nused; u++)
+            mesg->slot[u].name = (char *)H5MM_xfree(mesg->slot[u].name);
+        mesg->slot = (H5O_efl_entry_t *)H5MM_xfree(mesg->slot);
+    } /* end if */
     mesg->heap_addr = HADDR_UNDEF;
     mesg->nused = mesg->nalloc = 0;
-    if(mesg->slot)
-        mesg->slot = H5MM_xfree(mesg->slot);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5O_efl_reset() */
@@ -383,7 +420,7 @@ H5O_efl_total_size (H5O_efl_t *efl)
 {
     hsize_t	ret_value = 0, tmp;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_total_size)
+    FUNC_ENTER_NOAPI_NOINIT
 
     if(efl->nused > 0 && H5O_EFL_UNLIMITED == efl->slot[efl->nused - 1].size)
 	ret_value = H5O_EFL_UNLIMITED;
@@ -418,8 +455,8 @@ done:
  */
 static void *
 H5O_efl_copy_file(H5F_t UNUSED *file_src, void *mesg_src, H5F_t *file_dst,
-    hbool_t UNUSED *recompute_size, H5O_copy_t UNUSED *cpy_info,
-    void UNUSED *_udata, hid_t dxpl_id)
+    hbool_t UNUSED *recompute_size, unsigned UNUSED *mesg_flags,
+    H5O_copy_t UNUSED *cpy_info, void UNUSED *_udata, hid_t dxpl_id)
 {
     H5O_efl_t   *efl_src = (H5O_efl_t *) mesg_src;
     H5O_efl_t   *efl_dst = NULL;
@@ -427,14 +464,14 @@ H5O_efl_copy_file(H5F_t UNUSED *file_src, void *mesg_src, H5F_t *file_dst,
     size_t      idx, size, name_offset, heap_size;
     void        *ret_value;          /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_copy_file)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check args */
     HDassert(efl_src);
     HDassert(file_dst);
 
     /* Allocate space for the destination efl */
-    if(NULL == (efl_dst = H5MM_calloc(sizeof(H5O_efl_t))))
+    if(NULL == (efl_dst = (H5O_efl_t *)H5MM_calloc(sizeof(H5O_efl_t))))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
     /* Copy the "top level" information */
@@ -461,7 +498,7 @@ H5O_efl_copy_file(H5F_t UNUSED *file_src, void *mesg_src, H5F_t *file_dst,
     /* allocate array of external file entries */
     if(efl_src->nalloc > 0) {
         size = efl_src->nalloc * sizeof(H5O_efl_entry_t);
-        if((efl_dst->slot = H5MM_calloc(size)) == NULL)
+        if((efl_dst->slot = (H5O_efl_entry_t *)H5MM_calloc(size)) == NULL)
             HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
         /* copy content from the source. Need to update later */
@@ -481,7 +518,7 @@ H5O_efl_copy_file(H5F_t UNUSED *file_src, void *mesg_src, H5F_t *file_dst,
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(file_dst, dxpl_id, heap, efl_dst->heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_EFL, H5E_PROTECT, NULL, "unable to unprotect EFL file name heap")
     if(!ret_value)
         if(efl_dst)
@@ -511,7 +548,7 @@ H5O_efl_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FILE * s
     char		    buf[64];
     size_t		    u;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_debug)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check args */
     HDassert(f);
