@@ -81,7 +81,7 @@ DESCRIPTION
 static herr_t
 H5P_init_pub_interface(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5P_init_pub_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     FUNC_LEAVE_NOAPI(H5P_init())
 } /* H5P_init_pub_interface() */
@@ -114,7 +114,7 @@ H5Pcopy(hid_t id)
     void *obj;                 /* Property object to copy */
     hid_t ret_value=FALSE;      /* return value */
 
-    FUNC_ENTER_API(H5Pcopy, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", id);
 
     if(H5P_DEFAULT==id)
@@ -128,7 +128,7 @@ H5Pcopy(hid_t id)
 
     /* Compare property lists */
     if(H5I_GENPROP_LST == H5I_get_type(id)) {
-        if((ret_value = H5P_copy_plist(obj)) < 0)
+        if((ret_value = H5P_copy_plist((H5P_genplist_t *)obj, TRUE)) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy property list");
     } /* end if */
     /* Must be property classes */
@@ -136,18 +136,18 @@ H5Pcopy(hid_t id)
         H5P_genclass_t *copy_class;      /* Copy of class */
 
         /* Copy the class */
-        if((copy_class = H5P_copy_pclass(obj)) == NULL)
+        if((copy_class = H5P_copy_pclass((H5P_genclass_t *)obj)) == NULL)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy property class");
 
         /* Get an atom for the copied class */
-        if((ret_value = H5I_register(H5I_GENPROP_CLS, copy_class)) < 0) {
+        if((ret_value = H5I_register(H5I_GENPROP_CLS, copy_class, TRUE)) < 0) {
             H5P_close_class(copy_class);
             HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to atomize property list class");
         } /* end if */
     } /* end else */
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pcopy() */
 
 
@@ -196,39 +196,39 @@ H5Pcreate_class(hid_t parent, const char *name,
     H5P_genclass_t	*pclass = NULL;     /* Property list class created */
     hid_t	ret_value;                  /* Return value		   */
 
-    FUNC_ENTER_API(H5Pcreate_class, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE8("i", "i*sx*xx*xx*x", parent, name, cls_create, create_data, cls_copy,
              copy_data, cls_close, close_data);
 
     /* Check arguments. */
     if(H5P_DEFAULT!=parent && (H5I_GENPROP_CLS!=H5I_get_type(parent)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list class");
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list class")
     if(!name || !*name)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid class name");
-    if((create_data!=NULL && cls_create==NULL)
-            || (copy_data!=NULL && cls_copy==NULL)
-            || (close_data!=NULL && cls_close==NULL))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "data specified, but no callback provided");
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid class name")
+    if((create_data != NULL && cls_create == NULL)
+            || (copy_data != NULL && cls_copy == NULL)
+            || (close_data != NULL && cls_close == NULL))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "data specified, but no callback provided")
 
     /* Get the pointer to the parent class */
-    if(parent==H5P_DEFAULT)
-        par_class=NULL;
-    else if(NULL == (par_class = H5I_object(parent)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "can't retrieve parent class");
+    if(parent == H5P_DEFAULT)
+        par_class = NULL;
+    else if(NULL == (par_class = (H5P_genclass_t *)H5I_object(parent)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "can't retrieve parent class")
 
     /* Create the new property list class */
-    if(NULL==(pclass=H5P_create_class(par_class, name, 0, cls_create, create_data, cls_copy, copy_data, cls_close, close_data)))
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTCREATE, FAIL, "unable to create property list class");
+    if(NULL == (pclass = H5P_create_class(par_class, name, FALSE, cls_create, create_data, cls_copy, copy_data, cls_close, close_data)))
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTCREATE, FAIL, "unable to create property list class")
 
     /* Get an atom for the class */
-    if((ret_value = H5I_register(H5I_GENPROP_CLS, pclass)) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to atomize property list class");
+    if((ret_value = H5I_register(H5I_GENPROP_CLS, pclass, TRUE)) < 0)
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to atomize property list class")
 
 done:
-    if(ret_value<0 && pclass)
+    if(ret_value < 0 && pclass)
         H5P_close_class(pclass);
 
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pcreate_class() */
 
 
@@ -260,19 +260,19 @@ H5Pcreate(hid_t cls_id)
     H5P_genclass_t	*pclass;   /* Property list class to modify */
     hid_t ret_value;               /* return value */
 
-    FUNC_ENTER_API(H5Pcreate, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", cls_id);
 
     /* Check arguments. */
-    if(NULL == (pclass = H5I_object_verify(cls_id, H5I_GENPROP_CLS)))
+    if(NULL == (pclass = (H5P_genclass_t *)H5I_object_verify(cls_id, H5I_GENPROP_CLS)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list class");
 
     /* Create the new property list */
-    if((ret_value = H5P_create_id(pclass)) < 0)
+    if((ret_value = H5P_create_id(pclass, TRUE)) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTCREATE, FAIL, "unable to create property list");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pcreate() */
 
 
@@ -437,27 +437,43 @@ H5Pregister2(hid_t cls_id, const char *name, size_t size, void *def_value,
     H5P_prp_copy_func_t prp_copy, H5P_prp_compare_func_t prp_cmp,
     H5P_prp_close_func_t prp_close)
 {
-    H5P_genclass_t	*pclass;   /* Property list class to modify */
-    herr_t ret_value;     /* return value */
+    H5P_genclass_t *pclass;     /* Property list class to modify */
+    H5P_genclass_t *orig_pclass; /* Original property class */
+    herr_t ret_value;           /* Return value */
 
-    FUNC_ENTER_API(H5Pregister2, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE11("e", "i*sz*xxxxxxxx", cls_id, name, size, def_value, prp_create,
              prp_set, prp_get, prp_delete, prp_copy, prp_cmp, prp_close);
 
     /* Check arguments. */
-    if(NULL == (pclass = H5I_object_verify(cls_id, H5I_GENPROP_CLS)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list class");
+    if(NULL == (pclass = (H5P_genclass_t *)H5I_object_verify(cls_id, H5I_GENPROP_CLS)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list class")
     if(!name || !*name)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid class name");
-    if(size>0 && def_value==NULL)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "properties >0 size must have default");
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid class name")
+    if(size > 0 && def_value == NULL)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "properties >0 size must have default")
 
     /* Create the new property list class */
-    if((ret_value = H5P_register(pclass,name,size,def_value,prp_create,prp_set,prp_get,prp_delete,prp_copy,prp_cmp,prp_close)) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to register property in class");
+    orig_pclass = pclass;
+    if((ret_value = H5P_register(&pclass, name, size, def_value, prp_create, prp_set, prp_get, prp_delete, prp_copy, prp_cmp, prp_close)) < 0)
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to register property in class")
+
+    /* Check if the property class changed and needs to be substituted in the ID */
+    if(pclass != orig_pclass) {
+        H5P_genclass_t *old_pclass;     /* Old property class */
+
+        /* Substitute the new property class in the ID */
+        if(NULL == (old_pclass = (H5P_genclass_t *)H5I_subst(cls_id, pclass)))
+            HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "unable to substitute property class in ID")
+        HDassert(old_pclass == orig_pclass);
+
+        /* Close the previous class */
+        if(H5P_close_class(old_pclass) < 0)
+            HGOTO_ERROR(H5E_PLIST, H5E_CANTCLOSEOBJ, FAIL, "unable to close original property class after substitution")
+    } /* end if */
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pregister2() */
 
 
@@ -609,12 +625,12 @@ H5Pinsert2(hid_t plist_id, const char *name, size_t size, void *value,
     H5P_genplist_t	*plist;    /* Property list to modify */
     herr_t ret_value;           /* return value */
 
-    FUNC_ENTER_API(H5Pinsert2, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE10("e", "i*sz*xxxxxxx", plist_id, name, size, value, prp_set, prp_get,
              prp_delete, prp_copy, prp_cmp, prp_close);
 
     /* Check arguments. */
-    if(NULL == (plist = H5I_object_verify(plist_id, H5I_GENPROP_LST)))
+    if(NULL == (plist = (H5P_genplist_t *)H5I_object_verify(plist_id, H5I_GENPROP_LST)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list")
     if(!name || !*name)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid property name")
@@ -626,7 +642,7 @@ H5Pinsert2(hid_t plist_id, const char *name, size_t size, void *value,
         HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to register property in plist")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pinsert2() */
 
 
@@ -667,11 +683,11 @@ H5Pset(hid_t plist_id, const char *name, void *value)
     H5P_genplist_t *plist;      /* Property list to modify */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pset, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "i*s*x", plist_id, name, value);
 
     /* Check arguments. */
-    if(NULL == (plist = H5I_object_verify(plist_id, H5I_GENPROP_LST)))
+    if(NULL == (plist = (H5P_genplist_t *)H5I_object_verify(plist_id, H5I_GENPROP_LST)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
     if(!name || !*name)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid property name");
@@ -683,7 +699,7 @@ H5Pset(hid_t plist_id, const char *name, void *value)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to set value in plist");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pset() */
 
 
@@ -716,7 +732,7 @@ H5Pexist(hid_t id, const char *name)
     H5P_genclass_t	*pclass;   /* Property class to query */
     htri_t ret_value;           /* return value */
 
-    FUNC_ENTER_API(H5Pexist, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("t", "i*s", id, name);
 
     /* Check arguments. */
@@ -727,14 +743,14 @@ H5Pexist(hid_t id, const char *name)
 
     /* Check for the existance of the property in the list or class */
     if(H5I_GENPROP_LST == H5I_get_type(id)) {
-        if(NULL == (plist = H5I_object(id)))
+        if(NULL == (plist = (H5P_genplist_t *)H5I_object(id)))
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
         if((ret_value = H5P_exist_plist(plist, name)) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "property does not exist in list");
     } /* end if */
     else
         if(H5I_GENPROP_CLS == H5I_get_type(id)) {
-            if(NULL == (pclass = H5I_object(id)))
+            if(NULL == (pclass = (H5P_genclass_t *)H5I_object(id)))
                 HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property class");
             if((ret_value = H5P_exist_pclass(pclass, name)) < 0)
                 HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "property does not exist in class");
@@ -743,7 +759,7 @@ H5Pexist(hid_t id, const char *name)
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property object");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pexist() */
 
 
@@ -777,7 +793,7 @@ H5Pget_size(hid_t id, const char *name, size_t *size)
     H5P_genplist_t	*plist;    /* Property list to query */
     herr_t ret_value;           /* return value */
 
-    FUNC_ENTER_API(H5Pget_size, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "i*s*z", id, name, size);
 
     /* Check arguments. */
@@ -789,7 +805,7 @@ H5Pget_size(hid_t id, const char *name, size_t *size)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid property size");
 
     if(H5I_GENPROP_LST == H5I_get_type(id)) {
-        if(NULL == (plist = H5I_object(id)))
+        if(NULL == (plist = (H5P_genplist_t *)H5I_object(id)))
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
 
         /* Check the property size */
@@ -798,7 +814,7 @@ H5Pget_size(hid_t id, const char *name, size_t *size)
     } /* end if */
     else
         if(H5I_GENPROP_CLS == H5I_get_type(id)) {
-            if(NULL == (pclass = H5I_object(id)))
+            if(NULL == (pclass = (H5P_genclass_t *)H5I_object(id)))
                 HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
 
             /* Check the property size */
@@ -809,7 +825,7 @@ H5Pget_size(hid_t id, const char *name, size_t *size)
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property object");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pget_size() */
 
 
@@ -840,11 +856,11 @@ H5Pget_class(hid_t plist_id)
     H5P_genclass_t	*pclass=NULL;   /* Property list class */
     hid_t ret_value=FAIL;           /* return value */
 
-    FUNC_ENTER_API(H5Pget_class, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", plist_id);
 
     /* Check arguments. */
-    if(NULL == (plist = H5I_object_verify(plist_id, H5I_GENPROP_LST)))
+    if(NULL == (plist = (H5P_genplist_t *)H5I_object_verify(plist_id, H5I_GENPROP_LST)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
 
     /* Retrieve the property list class */
@@ -856,14 +872,14 @@ H5Pget_class(hid_t plist_id)
         HGOTO_ERROR (H5E_PLIST, H5E_CANTINIT, FAIL,"Can't increment class ID ref count");
 
     /* Get an atom for the class */
-    if((ret_value = H5I_register(H5I_GENPROP_CLS, pclass)) < 0)
+    if((ret_value = H5I_register(H5I_GENPROP_CLS, pclass, TRUE)) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to atomize property list class");
 
 done:
     if(ret_value<0 && pclass)
         H5P_close_class(pclass);
 
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pget_class() */
 
 
@@ -897,7 +913,7 @@ H5Pget_nprops(hid_t id, size_t *nprops)
     H5P_genclass_t	*pclass;   /* Property class to query */
     herr_t ret_value=SUCCEED;      /* return value */
 
-    FUNC_ENTER_API(H5Pget_nprops, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*z", id, nprops);
 
     /* Check arguments. */
@@ -907,14 +923,14 @@ H5Pget_nprops(hid_t id, size_t *nprops)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid property nprops pointer");
 
     if(H5I_GENPROP_LST == H5I_get_type(id)) {
-        if(NULL == (plist = H5I_object(id)))
+        if(NULL == (plist = (H5P_genplist_t *)H5I_object(id)))
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
         if(H5P_get_nprops_plist(plist, nprops) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to query # of properties in plist");
     } /* end if */
     else
         if(H5I_GENPROP_CLS == H5I_get_type(id)) {
-            if(NULL == (pclass = H5I_object(id)))
+            if(NULL == (pclass = (H5P_genclass_t *)H5I_object(id)))
                 HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property class");
             if(H5P_get_nprops_pclass(pclass, nprops, FALSE) < 0)
                 HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to query # of properties in pclass");
@@ -923,7 +939,7 @@ H5Pget_nprops(hid_t id, size_t *nprops)
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property object");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pget_nprops() */
 
 
@@ -953,7 +969,7 @@ H5Pequal(hid_t id1, hid_t id2)
     void *obj1, *obj2;          /* Property objects to compare */
     htri_t ret_value = FALSE;     /* return value */
 
-    FUNC_ENTER_API(H5Pequal, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("t", "ii", id1, id2);
 
     /* Check arguments. */
@@ -967,12 +983,12 @@ H5Pequal(hid_t id1, hid_t id2)
 
     /* Compare property lists */
     if(H5I_GENPROP_LST == H5I_get_type(id1)) {
-        if(H5P_cmp_plist(obj1, obj2) == 0)
+        if(H5P_cmp_plist((const H5P_genplist_t *)obj1, (const H5P_genplist_t *)obj2) == 0)
             ret_value = TRUE;
     } /* end if */
     /* Must be property classes */
     else {
-        if(H5P_cmp_class(obj1, obj2) == 0)
+        if(H5P_cmp_class((const H5P_genclass_t *)obj1, (const H5P_genclass_t *)obj2) == 0)
             ret_value = TRUE;
     } /* end else */
 
@@ -1009,7 +1025,7 @@ H5Pisa_class(hid_t plist_id, hid_t pclass_id)
 {
     htri_t ret_value;                   /* return value */
 
-    FUNC_ENTER_API(H5Pisa_class, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("t", "ii", plist_id, pclass_id);
 
     /* Check arguments. */
@@ -1023,7 +1039,7 @@ H5Pisa_class(hid_t plist_id, hid_t pclass_id)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to compare property list classes");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pisa_class() */
 
 
@@ -1086,7 +1102,7 @@ H5Piterate(hid_t id, int *idx, H5P_iterate_t iter_func, void *iter_data)
     int fake_idx = 0;       /* Index when user doesn't provide one */
     int ret_value;          /* return value */
 
-    FUNC_ENTER_API(H5Piterate, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("Is", "i*Isx*x", id, idx, iter_func, iter_data);
 
     /* Check arguments. */
@@ -1110,7 +1126,7 @@ H5Piterate(hid_t id, int *idx, H5P_iterate_t iter_func, void *iter_data)
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property object");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Piterate() */
 
 
@@ -1148,11 +1164,11 @@ H5Pget(hid_t plist_id, const char *name, void *value)
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "i*s*x", plist_id, name, value);
 
     /* Check arguments. */
-    if(NULL == (plist = H5I_object_verify(plist_id, H5I_GENPROP_LST)))
+    if(NULL == (plist = (H5P_genplist_t *)H5I_object_verify(plist_id, H5I_GENPROP_LST)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
     if(!name || !*name)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid property name");
@@ -1164,7 +1180,7 @@ H5Pget(hid_t plist_id, const char *name, void *value)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "unable to query property value");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pget() */
 
 
@@ -1200,11 +1216,11 @@ H5Premove(hid_t plist_id, const char *name)
     H5P_genplist_t	*plist;    /* Property list to modify */
     herr_t ret_value;           /* return value */
 
-    FUNC_ENTER_API(H5Premove, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*s", plist_id, name);
 
     /* Check arguments. */
-    if(NULL == (plist = H5I_object_verify(plist_id, H5I_GENPROP_LST)))
+    if(NULL == (plist = (H5P_genplist_t *)H5I_object_verify(plist_id, H5I_GENPROP_LST)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
     if(!name || !*name)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid property name");
@@ -1214,7 +1230,7 @@ H5Premove(hid_t plist_id, const char *name)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTDELETE, FAIL, "unable to remove property");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Premove() */
 
 
@@ -1259,36 +1275,38 @@ done:
 herr_t
 H5Pcopy_prop(hid_t dst_id, hid_t src_id, const char *name)
 {
-    void *src_obj, *dst_obj;    /* Property objects to copy between */
-    herr_t ret_value=SUCCEED;      /* return value */
+    H5I_type_t src_id_type, dst_id_type;        /* ID types */
+    herr_t ret_value = SUCCEED;     /* return value */
 
-    FUNC_ENTER_API(H5Pcopy_prop, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ii*s", dst_id, src_id, name);
 
     /* Check arguments. */
-    if((H5I_GENPROP_LST != H5I_get_type(src_id) && H5I_GENPROP_CLS != H5I_get_type(src_id))
-            || (H5I_GENPROP_LST != H5I_get_type(dst_id) && H5I_GENPROP_CLS != H5I_get_type(dst_id)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not property objects");
-    if(H5I_get_type(src_id) != H5I_get_type(dst_id))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not the same kind of property objects");
+    if((src_id_type = H5I_get_type(src_id)) < 0)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid source ID")
+    if((dst_id_type = H5I_get_type(dst_id)) < 0)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid destination ID")
+    if((H5I_GENPROP_LST != src_id_type && H5I_GENPROP_CLS != src_id_type)
+            || (H5I_GENPROP_LST != dst_id_type && H5I_GENPROP_CLS != dst_id_type))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not property objects")
+    if(src_id_type != dst_id_type)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not the same kind of property objects")
     if(!name || !*name)
-        HGOTO_ERROR (H5E_ARGS, H5E_BADVALUE, FAIL, "no name given");
-    if(NULL == (src_obj = H5I_object(src_id)) || NULL == (dst_obj = H5I_object(dst_id)))
-        HGOTO_ERROR(H5E_PLIST, H5E_NOTFOUND, FAIL, "property object doesn't exist");
+        HGOTO_ERROR (H5E_ARGS, H5E_BADVALUE, FAIL, "no name given")
 
     /* Compare property lists */
-    if(H5I_GENPROP_LST == H5I_get_type(src_id)) {
+    if(H5I_GENPROP_LST == src_id_type) {
         if(H5P_copy_prop_plist(dst_id, src_id, name) < 0)
-            HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy property between lists");
+            HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy property between lists")
     } /* end if */
     /* Must be property classes */
     else {
-        if(H5P_copy_prop_pclass(dst_obj, src_obj, name) < 0)
-            HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy property between classes");
+        if(H5P_copy_prop_pclass(dst_id, src_id, name) < 0)
+            HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy property between classes")
     } /* end else */
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pcopy_prop() */
 
 
@@ -1319,11 +1337,11 @@ H5Punregister(hid_t pclass_id, const char *name)
     H5P_genclass_t	*pclass;   /* Property list class to modify */
     herr_t ret_value;           /* return value */
 
-    FUNC_ENTER_API(H5Punregister, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*s", pclass_id, name);
 
     /* Check arguments. */
-    if(NULL == (pclass = H5I_object_verify(pclass_id, H5I_GENPROP_CLS)))
+    if(NULL == (pclass = (H5P_genclass_t *)H5I_object_verify(pclass_id, H5I_GENPROP_CLS)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list class");
     if(!name || !*name)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid property name");
@@ -1333,7 +1351,7 @@ H5Punregister(hid_t pclass_id, const char *name)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to remove property from class");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Punregister() */
 
 
@@ -1361,24 +1379,24 @@ done:
 herr_t
 H5Pclose(hid_t plist_id)
 {
-    herr_t ret_value=SUCCEED;      /* return value */
+    herr_t ret_value = SUCCEED;      /* return value */
 
-    FUNC_ENTER_API(H5Pclose, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", plist_id);
 
-    if(plist_id==H5P_DEFAULT)
-        HGOTO_DONE(SUCCEED);
+    /* Allow default property lists to pass through without throwing an error */
+    if(H5P_DEFAULT != plist_id) {
+        /* Check arguments. */
+        if(H5I_GENPROP_LST != H5I_get_type(plist_id))
+            HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list")
 
-    /* Check arguments. */
-    if(H5I_GENPROP_LST != H5I_get_type(plist_id))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list");
-
-    /* Close the property list */
-    if(H5I_dec_ref(plist_id) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTFREE, FAIL, "can't close");
+        /* Close the property list */
+        if(H5I_dec_app_ref(plist_id) < 0)
+            HGOTO_ERROR(H5E_PLIST, H5E_CANTFREE, FAIL, "can't close")
+    } /* end if */
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pclose() */
 
 
@@ -1408,10 +1426,10 @@ H5Pget_class_name(hid_t pclass_id)
     H5P_genclass_t	*pclass;    /* Property class to query */
     char *ret_value;       /* return value */
 
-    FUNC_ENTER_API(H5Pget_class_name, NULL);
+    FUNC_ENTER_API(NULL)
 
     /* Check arguments. */
-    if(NULL == (pclass = H5I_object_verify(pclass_id, H5I_GENPROP_CLS)))
+    if(NULL == (pclass = (H5P_genclass_t *)H5I_object_verify(pclass_id, H5I_GENPROP_CLS)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, NULL, "not a property class");
 
     /* Get the property list class name */
@@ -1419,7 +1437,7 @@ H5Pget_class_name(hid_t pclass_id)
         HGOTO_ERROR(H5E_PLIST, H5E_NOTFOUND, NULL, "unable to query name of class");
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pget_class_name() */
 
 
@@ -1446,33 +1464,33 @@ hid_t
 H5Pget_class_parent(hid_t pclass_id)
 {
     H5P_genclass_t	*pclass;    /* Property class to query */
-    H5P_genclass_t	*parent=NULL;   /* Parent's property class */
+    H5P_genclass_t	*parent = NULL;   /* Parent's property class */
     hid_t ret_value;       /* return value */
 
-    FUNC_ENTER_API(H5Pget_class_parent, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", pclass_id);
 
     /* Check arguments. */
-    if(NULL == (pclass = H5I_object_verify(pclass_id, H5I_GENPROP_CLS)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property class");
+    if(NULL == (pclass = (H5P_genclass_t *)H5I_object_verify(pclass_id, H5I_GENPROP_CLS)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property class")
 
     /* Retrieve the property class's parent */
-    if((parent = H5P_get_class_parent(pclass)) == NULL)
-        HGOTO_ERROR(H5E_PLIST, H5E_NOTFOUND, FAIL, "unable to query class of property list");
+    if(NULL == (parent = H5P_get_class_parent(pclass)))
+        HGOTO_ERROR(H5E_PLIST, H5E_NOTFOUND, FAIL, "unable to query class of property list")
 
     /* Increment the outstanding references to the class object */
     if(H5P_access_class(parent, H5P_MOD_INC_REF) < 0)
-        HGOTO_ERROR (H5E_PLIST, H5E_CANTINIT, FAIL,"Can't increment class ID ref count");
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTINIT, FAIL,"Can't increment class ID ref count")
 
     /* Get an atom for the class */
-    if((ret_value = H5I_register(H5I_GENPROP_CLS, parent)) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to atomize property list class");
+    if((ret_value = H5I_register(H5I_GENPROP_CLS, parent, TRUE)) < 0)
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTREGISTER, FAIL, "unable to atomize property list class")
 
 done:
-    if(ret_value<0 && parent)
+    if(ret_value < 0 && parent)
         H5P_close_class(parent);
 
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pget_class_parent() */
 
 
@@ -1499,18 +1517,18 @@ H5Pclose_class(hid_t cls_id)
 {
     hid_t	ret_value = SUCCEED;    /* Return value			*/
 
-    FUNC_ENTER_API(H5Pclose_class, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", cls_id);
 
     /* Check arguments */
     if(H5I_GENPROP_CLS != H5I_get_type(cls_id))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list class");
+        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a property list class")
 
     /* Close the property list class */
-    if(H5I_dec_ref(cls_id) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTFREE, FAIL, "can't close");
+    if(H5I_dec_app_ref(cls_id) < 0)
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTFREE, FAIL, "can't close")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* H5Pclose_class() */
 
