@@ -215,7 +215,7 @@ H5P_dxfr_reg_prop(H5P_genclass_t *pclass)
     void *def_xfer_xform = H5D_XFER_XFORM_DEF;          /* Default value for data transform */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5P_dxfr_reg_prop)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Register the max. temp buffer size property */
     if(H5P_register_real(pclass, H5D_XFER_MAX_TEMP_BUF_NAME, H5D_XFER_MAX_TEMP_BUF_SIZE, &def_max_temp_buf, NULL, NULL, NULL, NULL, NULL, NULL, NULL) < 0)
@@ -336,7 +336,7 @@ H5P_dxfr_create(hid_t dxpl_id, void UNUSED *create_data)
     H5P_genplist_t *plist;      /* Property list */
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5P_dxfr_create)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments */
     if(NULL == (plist = (H5P_genplist_t *)H5I_object(dxpl_id)))
@@ -386,7 +386,7 @@ H5P_dxfr_copy(hid_t dst_dxpl_id, hid_t src_dxpl_id, void UNUSED *copy_data)
     H5P_genplist_t *src_plist;              /* Source property list */
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5P_dxfr_copy)
+    FUNC_ENTER_NOAPI_NOINIT
 
     if(NULL == (dst_plist = (H5P_genplist_t *)H5I_object(dst_dxpl_id)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "can't get property list")
@@ -437,7 +437,7 @@ H5P_dxfr_close(hid_t dxpl_id, void UNUSED *close_data)
     H5P_genplist_t *plist;      /* Property list */
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5P_dxfr_close)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check arguments */
     if(NULL == (plist = (H5P_genplist_t *)H5I_object(dxpl_id)))
@@ -477,7 +477,7 @@ H5P_dxfr_xform_del(hid_t UNUSED prop_id, const char UNUSED *name, size_t UNUSED
 {
     herr_t ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5P_dxfr_xform_del)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(value);
 
@@ -509,7 +509,7 @@ H5P_dxfr_xform_copy(const char UNUSED *name, size_t UNUSED size, void *value)
 {
     herr_t ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5P_dxfr_xform_copy)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(value);
 
@@ -540,7 +540,7 @@ H5P_dxfr_xform_close(const char UNUSED *name, size_t UNUSED size, void *value)
 {
     herr_t ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5P_dxfr_xform_close)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(value);
 
@@ -574,7 +574,7 @@ H5Pset_data_transform(hid_t plist_id, const char *expression)
     H5Z_data_xform_t *data_xform_prop = NULL;    /* New data xform property */
     herr_t ret_value = SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pset_data_transform, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*s", plist_id, expression);
 
     /* Check arguments */
@@ -640,7 +640,7 @@ H5Pget_data_transform(hid_t plist_id, char *expression /*out*/, size_t size)
     char*	pexp;
     ssize_t 	ret_value;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_data_transform, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("Zs", "ixz", plist_id, expression, size);
 
     /* Get the plist structure */
@@ -708,7 +708,7 @@ H5Pset_buffer(hid_t plist_id, size_t size, void *tconv, void *bkg)
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pset_buffer, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "iz*x*x", plist_id, size, tconv, bkg);
 
     /* Check arguments */
@@ -755,7 +755,7 @@ H5Pget_buffer(hid_t plist_id, void **tconv/*out*/, void **bkg/*out*/)
     size_t size;                /* Type conversion buffer size */
     size_t ret_value;           /* Return value */
 
-    FUNC_ENTER_API(H5Pget_buffer, 0)
+    FUNC_ENTER_API(0)
     H5TRACE3("z", "ixx", plist_id, tconv, bkg);
 
     /* Get the plist structure */
@@ -807,7 +807,7 @@ H5Pset_preserve(hid_t plist_id, hbool_t status)
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pset_preserve, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "ib", plist_id, status);
 
     /* Get the plist structure */
@@ -847,7 +847,7 @@ H5Pget_preserve(hid_t plist_id)
     H5P_genplist_t *plist;      /* Property list pointer */
     int ret_value;              /* return value */
 
-    FUNC_ENTER_API(H5Pget_preserve, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Is", "i", plist_id);
 
     /* Get the plist structure */
@@ -889,7 +889,7 @@ H5Pset_edc_check(hid_t plist_id, H5Z_EDC_t check)
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pset_edc_check, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "iZe", plist_id, check);
 
     /* Check argument */
@@ -932,7 +932,7 @@ H5Pget_edc_check(hid_t plist_id)
     H5P_genplist_t *plist;      /* Property list pointer */
     H5Z_EDC_t      ret_value;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_edc_check, H5Z_ERROR_EDC)
+    FUNC_ENTER_API(H5Z_ERROR_EDC)
     H5TRACE1("Ze", "i", plist_id);
 
     /* Get the plist structure */
@@ -975,7 +975,7 @@ H5Pset_filter_callback(hid_t plist_id, H5Z_filter_func_t func, void *op_data)
     herr_t              ret_value=SUCCEED;   /* return value */
     H5Z_cb_t            cb_struct;
 
-    FUNC_ENTER_API(H5Pset_filter_callback, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ix*x", plist_id, func, op_data);
 
     /* Get the plist structure */
@@ -1017,7 +1017,7 @@ H5Pset_type_conv_cb(hid_t plist_id, H5T_conv_except_func_t op, void *operate_dat
     herr_t              ret_value=SUCCEED;   /* return value */
     H5T_conv_cb_t       cb_struct;
 
-    FUNC_ENTER_API(H5Pset_type_conv_cb, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ix*x", plist_id, op, operate_data);
 
     /* Get the plist structure */
@@ -1059,7 +1059,7 @@ H5Pget_type_conv_cb(hid_t plist_id, H5T_conv_except_func_t *op, void **operate_d
     H5T_conv_cb_t       cb_struct;
     herr_t              ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_type_conv_cb, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "i*x**x", plist_id, op, operate_data);
 
     /* Get the plist structure */
@@ -1104,7 +1104,7 @@ H5Pget_btree_ratios(hid_t plist_id, double *left/*out*/, double *middle/*out*/,
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_btree_ratios, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "ixxx", plist_id, left, middle, right);
 
     /* Get the plist structure */
@@ -1158,7 +1158,7 @@ H5Pset_btree_ratios(hid_t plist_id, double left, double middle,
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pset_btree_ratios, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "iddd", plist_id, left, middle, right);
 
     /* Check arguments */
@@ -1211,7 +1211,7 @@ H5P_set_vlen_mem_manager(H5P_genplist_t *plist, H5MM_allocate_t alloc_func,
 {
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_NOAPI(H5P_set_vlen_mem_manager, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     assert(plist);
 
@@ -1258,7 +1258,7 @@ H5Pset_vlen_mem_manager(hid_t plist_id, H5MM_allocate_t alloc_func,
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pset_vlen_mem_manager, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE5("e", "ix*xx*x", plist_id, alloc_func, alloc_info, free_func,
              free_info);
 
@@ -1298,7 +1298,7 @@ H5Pget_vlen_mem_manager(hid_t plist_id, H5MM_allocate_t *alloc_func/*out*/,
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_vlen_mem_manager, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE5("e", "ixxxx", plist_id, alloc_func, alloc_info, free_func, free_info);
 
     /* Get the plist structure */
@@ -1352,7 +1352,7 @@ H5Pset_hyper_vector_size(hid_t plist_id, size_t vector_size)
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pset_hyper_vector_size, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "iz", plist_id, vector_size);
 
     /* Check arguments */
@@ -1392,7 +1392,7 @@ H5Pget_hyper_vector_size(hid_t plist_id, size_t *vector_size/*out*/)
     H5P_genplist_t *plist;      /* Property list pointer */
     herr_t ret_value=SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_hyper_vector_size, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "ix", plist_id, vector_size);
 
     /* Get the plist structure */
@@ -1427,7 +1427,7 @@ H5Pget_mpio_actual_chunk_opt_mode(hid_t plist_id, H5D_mpio_actual_chunk_opt_mode
     H5P_genplist_t     *plist;
     herr_t ret_value = SUCCEED;   /* return value */
     
-    FUNC_ENTER_API(H5Pget_mpio_actual_chunk_opt_mode, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*Do", plist_id, actual_chunk_opt_mode);
 
     /* Get the plist structure */
@@ -1462,7 +1462,7 @@ H5Pget_mpio_actual_io_mode(hid_t plist_id, H5D_mpio_actual_io_mode_t *actual_io_
     H5P_genplist_t     *plist;
     herr_t ret_value = SUCCEED;   /* return value */
     
-    FUNC_ENTER_API(H5Pget_mpio_actual_io_mode, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*Di", plist_id, actual_io_mode);
 
     /* Get the plist structure */
