@@ -110,16 +110,14 @@ H5P_gcrt_reg_prop(H5P_genclass_t *pclass)
     H5O_linfo_t linfo = H5G_CRT_LINK_INFO_DEF;      /* Default link info settings */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5P_gcrt_reg_prop)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Register group info property */
-    if(H5P_register(pclass, H5G_CRT_GROUP_INFO_NAME, H5G_CRT_GROUP_INFO_SIZE,
-             &ginfo, NULL, NULL, NULL, NULL, NULL, NULL, NULL) < 0)
+    if(H5P_register_real(pclass, H5G_CRT_GROUP_INFO_NAME, H5G_CRT_GROUP_INFO_SIZE, &ginfo, NULL, NULL, NULL, NULL, NULL, NULL, NULL) < 0)
          HGOTO_ERROR(H5E_PLIST, H5E_CANTINSERT, FAIL, "can't insert property into class")
 
     /* Register link info property */
-    if(H5P_register(pclass, H5G_CRT_LINK_INFO_NAME, H5G_CRT_LINK_INFO_SIZE,
-             &linfo, NULL, NULL, NULL, NULL, NULL, NULL, NULL) < 0)
+    if(H5P_register_real(pclass, H5G_CRT_LINK_INFO_NAME, H5G_CRT_LINK_INFO_SIZE, &linfo, NULL, NULL, NULL, NULL, NULL, NULL, NULL) < 0)
          HGOTO_ERROR(H5E_PLIST, H5E_CANTINSERT, FAIL, "can't insert property into class")
 
 done:
@@ -145,7 +143,7 @@ H5Pset_local_heap_size_hint(hid_t plist_id, size_t size_hint)
     H5O_ginfo_t ginfo;          /* Group information structure */
     herr_t ret_value = SUCCEED;       /* Return value */
 
-    FUNC_ENTER_API(H5Pset_local_heap_size_hint, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "iz", plist_id, size_hint);
 
     /* Get the plist structure */
@@ -185,7 +183,7 @@ H5Pget_local_heap_size_hint(hid_t plist_id, size_t *size_hint /*out*/)
 {
     herr_t ret_value = SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_local_heap_size_hint, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "ix", plist_id, size_hint);
 
     if(size_hint) {
@@ -233,7 +231,7 @@ H5Pset_link_phase_change(hid_t plist_id, unsigned max_compact, unsigned min_dens
     H5O_ginfo_t ginfo;                  /* Group information structure */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_API(H5Pset_link_phase_change, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "iIuIu", plist_id, max_compact, min_dense);
 
     /* Range check values */
@@ -257,8 +255,8 @@ H5Pset_link_phase_change(hid_t plist_id, unsigned max_compact, unsigned min_dens
         ginfo.store_link_phase_change = TRUE;
     else
         ginfo.store_link_phase_change = FALSE;
-    ginfo.max_compact = max_compact;
-    ginfo.min_dense = min_dense;
+    ginfo.max_compact = (uint16_t)max_compact;
+    ginfo.min_dense = (uint16_t)min_dense;
 
     /* Set group info */
     if(H5P_set(plist, H5G_CRT_GROUP_INFO_NAME, &ginfo) < 0)
@@ -286,7 +284,7 @@ H5Pget_link_phase_change(hid_t plist_id, unsigned *max_compact /*out*/, unsigned
 {
     herr_t ret_value = SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_link_phase_change, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ixx", plist_id, max_compact, min_dense);
 
     /* Get values */
@@ -339,7 +337,7 @@ H5Pset_est_link_info(hid_t plist_id, unsigned est_num_entries, unsigned est_name
     H5O_ginfo_t ginfo;                  /* Group information structure */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_API(H5Pset_est_link_info, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "iIuIu", plist_id, est_num_entries, est_name_len);
 
     /* Range check values */
@@ -361,8 +359,8 @@ H5Pset_est_link_info(hid_t plist_id, unsigned est_num_entries, unsigned est_name
         ginfo.store_est_entry_info = TRUE;
     else
         ginfo.store_est_entry_info = FALSE;
-    ginfo.est_num_entries = est_num_entries;
-    ginfo.est_name_len = est_name_len;
+    ginfo.est_num_entries = (uint16_t)est_num_entries;
+    ginfo.est_name_len = (uint16_t)est_name_len;
 
     /* Set group info */
     if(H5P_set(plist, H5G_CRT_GROUP_INFO_NAME, &ginfo) < 0)
@@ -390,7 +388,7 @@ H5Pget_est_link_info(hid_t plist_id, unsigned *est_num_entries /*out*/, unsigned
 {
     herr_t ret_value = SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_est_link_info, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "ixx", plist_id, est_num_entries, est_name_len);
 
     /* Get values */
@@ -435,7 +433,7 @@ H5Pset_link_creation_order(hid_t plist_id, unsigned crt_order_flags)
     H5O_linfo_t linfo;                  /* Link information structure */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_API(H5Pset_link_creation_order, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "iIu", plist_id, crt_order_flags);
 
         /* Check for bad combination of flags */
@@ -451,8 +449,8 @@ H5Pset_link_creation_order(hid_t plist_id, unsigned crt_order_flags)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get link info")
 
     /* Update fields */
-    linfo.track_corder = (crt_order_flags & H5P_CRT_ORDER_TRACKED) ? TRUE : FALSE;
-    linfo.index_corder = (crt_order_flags & H5P_CRT_ORDER_INDEXED) ? TRUE : FALSE;
+    linfo.track_corder = (hbool_t)((crt_order_flags & H5P_CRT_ORDER_TRACKED) ? TRUE : FALSE);
+    linfo.index_corder = (hbool_t)((crt_order_flags & H5P_CRT_ORDER_INDEXED) ? TRUE : FALSE);
 
     /* Set link info */
     if(H5P_set(plist, H5G_CRT_LINK_INFO_NAME, &linfo) < 0)
@@ -480,7 +478,7 @@ H5Pget_link_creation_order(hid_t plist_id, unsigned *crt_order_flags /*out*/)
 {
     herr_t ret_value = SUCCEED;   /* return value */
 
-    FUNC_ENTER_API(H5Pget_link_creation_order, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "ix", plist_id, crt_order_flags);
 
     /* Get values */
