@@ -20,6 +20,7 @@
 
 
 #include "H5private.h"		/* Generic Functions			*/
+#include "H5ACprivate.h"        /* Metadata cache                       */
 #include "H5Dprivate.h"		/* Datasets				*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5Gprivate.h"		/* Groups				*/
@@ -37,7 +38,8 @@
 /* Static functions */
 static herr_t H5R_create(void *ref, H5G_loc_t *loc, const char *name,
         H5R_type_t ref_type, H5S_t *space, hid_t dxpl_id);
-static hid_t H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, const void *_ref);
+static hid_t H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type,
+    const void *_ref, hbool_t app_ref);
 static H5S_t * H5R_get_region(H5F_t *file, hid_t dxpl_id, const void *_ref);
 static ssize_t H5R_get_name(H5F_t *file, hid_t lapl_id, hid_t dxpl_id, hid_t id,
     H5R_type_t ref_type, const void *_ref, char *name, size_t size);
@@ -61,7 +63,7 @@ H5R_init(void)
 {
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5R_init, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
     /* FUNC_ENTER() does all the work */
 
 done:
@@ -86,14 +88,14 @@ H5R_init_interface(void)
 {
     herr_t      ret_value=SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5R_init_interface);
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Initialize the atom group for the file IDs */
     if(H5I_register_type(H5I_REFERENCE, (size_t)H5I_REFID_HASHSIZE, H5R_RESERVED_ATOMS, (H5I_free_t)NULL) < 0)
 	HGOTO_ERROR(H5E_REFERENCE, H5E_CANTINIT, FAIL, "unable to initialize interface");
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }
 
 
@@ -119,11 +121,11 @@ H5R_term_interface(void)
 {
     int	n=0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5R_term_interface);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     if (H5_interface_initialize_g) {
 	if ((n=H5I_nmembers(H5I_REFERENCE))) {
-	    H5I_clear_type(H5I_REFERENCE, FALSE);
+	    H5I_clear_type(H5I_REFERENCE, FALSE, FALSE);
 	} else {
 	    H5I_dec_type_ref(H5I_REFERENCE);
 	    H5_interface_initialize_g = 0;
@@ -131,7 +133,7 @@ H5R_term_interface(void)
 	}
     }
 
-    FUNC_LEAVE_NOAPI(n);
+    FUNC_LEAVE_NOAPI(n)
 }
 
 
@@ -171,7 +173,7 @@ H5R_create(void *_ref, H5G_loc_t *loc, const char *name, H5R_type_t ref_type, H5
     hbool_t     obj_found = FALSE;      /* Object location found */
     herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5R_create)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(_ref);
     HDassert(loc);
@@ -235,7 +237,7 @@ H5R_create(void *_ref, H5G_loc_t *loc, const char *name, H5R_type_t ref_type, H5
 
             /* Allocate the space to store the serialized information */
             H5_CHECK_OVERFLOW(buf_size, hssize_t, size_t);
-            if(NULL == (buf = H5MM_malloc((size_t)buf_size)))
+            if(NULL == (buf = (uint8_t *)H5MM_malloc((size_t)buf_size)))
                 HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed")
 
             /* Serialize information for dataset OID into heap buffer */
@@ -264,7 +266,7 @@ H5R_create(void *_ref, H5G_loc_t *loc, const char *name, H5R_type_t ref_type, H5
         case H5R_BADTYPE:
         case H5R_MAXTYPE:
         default:
-            assert("unknown reference type" && 0);
+            HDassert("unknown reference type" && 0);
             HGOTO_ERROR(H5E_REFERENCE, H5E_UNSUPPORTED, FAIL, "internal error (unknown reference type)")
     } /* end switch */
 
@@ -307,10 +309,10 @@ herr_t
 H5Rcreate(void *ref, hid_t loc_id, const char *name, H5R_type_t ref_type, hid_t space_id)
 {
     H5G_loc_t   loc;            /* File location */
-    H5S_t	*space = NULL;          /* Pointer to dataspace containing region */
+    H5S_t      *space = NULL;   /* Pointer to dataspace containing region */
     herr_t      ret_value;      /* Return value */
 
-    FUNC_ENTER_API(H5Rcreate, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE5("e", "*xi*sRti", ref, loc_id, name, ref_type, space_id);
 
     /* Check args */
@@ -324,7 +326,9 @@ H5Rcreate(void *ref, hid_t loc_id, const char *name, H5R_type_t ref_type, hid_t
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid reference type")
     if(ref_type != H5R_OBJECT && ref_type != H5R_DATASET_REGION)
         HGOTO_ERROR(H5E_ARGS, H5E_UNSUPPORTED, FAIL, "reference type not supported")
-    if(space_id != (-1) && (NULL == (space = H5I_object_verify(space_id, H5I_DATASPACE))))
+    if(space_id == (-1) && ref_type == H5R_DATASET_REGION)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "reference region dataspace id must be valid")
+    if(space_id != (-1) && (NULL == (space = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE))))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
     /* Create reference */
@@ -359,15 +363,16 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 static hid_t
-H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, const void *_ref)
+H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, const void *_ref, hbool_t app_ref)
 {
     H5O_loc_t oloc;             /* Object location */
     H5G_name_t path;            /* Path of object */
     H5G_loc_t loc;              /* Group location */
+    unsigned rc;		/* Reference count of object */
     H5O_type_t obj_type;        /* Type of object */
-    hid_t ret_value;
+    hid_t ret_value;            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5R_dereference)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(_ref);
     HDassert(ref_type > H5R_BADTYPE || ref_type < H5R_MAXTYPE);
@@ -394,7 +399,7 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, const void *_re
             INT32DECODE(p, hobjid.idx);
 
             /* Get the dataset region from the heap (allocate inside routine) */
-            if((buf = H5HG_read(oloc.file, dxpl_id, &hobjid, NULL, NULL)) == NULL)
+            if(NULL == (buf = (uint8_t *)H5HG_read(oloc.file, dxpl_id, &hobjid, NULL, NULL)))
                 HGOTO_ERROR(H5E_REFERENCE, H5E_READERROR, FAIL, "Unable to read dataset region information")
 
             /* Get the object oid for the dataset */
@@ -413,8 +418,9 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, const void *_re
             HGOTO_ERROR(H5E_REFERENCE, H5E_UNSUPPORTED, FAIL, "internal error (unknown reference type)")
     } /* end switch */
 
-    /* Check to make certain that this object hasn't been deleted since the reference was created */
-    if(H5O_link(&oloc, 0, dxpl_id) <= 0)
+    /* Get the # of links for object, and its type */
+    /* (To check to make certain that this object hasn't been deleted since the reference was created) */
+    if(H5O_get_rc_and_type(&oloc, dxpl_id, &rc, &obj_type) < 0 || 0 == rc)
         HGOTO_ERROR(H5E_REFERENCE, H5E_LINKCOUNT, FAIL, "dereferencing deleted object")
 
     /* Construct a group location for opening the object */
@@ -422,21 +428,17 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, const void *_re
     loc.oloc = &oloc;
     loc.path = &path;
 
-    /* Get the type of the object */
-    if(H5O_obj_type(&oloc, &obj_type, dxpl_id) < 0)
-        HGOTO_ERROR(H5E_REFERENCE, H5E_CANTGET, FAIL, "unable to get object type")
-
     /* Open the object */
     switch(obj_type) {
         case H5O_TYPE_GROUP:
             {
                 H5G_t *group;               /* Pointer to group to open */
 
-                if((group = H5G_open(&loc, dxpl_id)) == NULL)
+                if(NULL == (group = H5G_open(&loc, dxpl_id)))
                     HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "not found")
 
                 /* Create an atom for the group */
-                if((ret_value = H5I_register(H5I_GROUP, group)) < 0) {
+                if((ret_value = H5I_register(H5I_GROUP, group, app_ref)) < 0) {
                     H5G_close(group);
                     HGOTO_ERROR(H5E_SYM, H5E_CANTREGISTER, FAIL, "can't register group")
                 } /* end if */
@@ -447,11 +449,11 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, const void *_re
             {
                 H5T_t *type;                /* Pointer to datatype to open */
 
-                if((type = H5T_open(&loc, dxpl_id)) == NULL)
+                if(NULL == (type = H5T_open(&loc, dxpl_id)))
                     HGOTO_ERROR(H5E_DATATYPE, H5E_NOTFOUND, FAIL, "not found")
 
                 /* Create an atom for the datatype */
-                if((ret_value = H5I_register(H5I_DATATYPE, type)) < 0) {
+                if((ret_value = H5I_register(H5I_DATATYPE, type, app_ref)) < 0) {
                     H5T_close(type);
                     HGOTO_ERROR(H5E_DATATYPE, H5E_CANTREGISTER, FAIL, "can't register datatype")
                 } /* end if */
@@ -460,14 +462,15 @@ H5R_dereference(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type, const void *_re
 
         case H5O_TYPE_DATASET:
             {
+                hid_t dapl_id = H5P_DATASET_ACCESS_DEFAULT; /* dapl to use to open dataset */
                 H5D_t *dset;                /* Pointer to dataset to open */
 
                 /* Open the dataset */
-                if((dset = H5D_open(&loc, dxpl_id)) == NULL)
+                if(NULL == (dset = H5D_open(&loc, dapl_id, dxpl_id)))
                     HGOTO_ERROR(H5E_DATASET, H5E_NOTFOUND, FAIL, "not found")
 
                 /* Create an atom for the dataset */
-                if((ret_value = H5I_register(H5I_DATASET, dset)) < 0) {
+                if((ret_value = H5I_register(H5I_DATASET, dset, app_ref)) < 0) {
                     H5D_close(dset);
                     HGOTO_ERROR(H5E_DATASET, H5E_CANTREGISTER, FAIL, "can't register dataset")
                 } /* end if */
@@ -512,7 +515,7 @@ H5Rdereference(hid_t id, H5R_type_t ref_type, const void *_ref)
     H5F_t *file = NULL; /* File object */
     hid_t ret_value;
 
-    FUNC_ENTER_API(H5Rdereference, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("i", "iRt*x", id, ref_type, _ref);
 
     /* Check args */
@@ -527,7 +530,7 @@ H5Rdereference(hid_t id, H5R_type_t ref_type, const void *_ref)
     file = loc.oloc->file;
 
     /* Create reference */
-    if((ret_value = H5R_dereference(file, H5AC_dxpl_id, ref_type, _ref)) < 0)
+    if((ret_value = H5R_dereference(file, H5AC_dxpl_id, ref_type, _ref, TRUE)) < 0)
         HGOTO_ERROR(H5E_REFERENCE, H5E_CANTINIT, FAIL, "unable dereference object")
 
 done:
@@ -565,7 +568,7 @@ H5R_get_region(H5F_t *file, hid_t dxpl_id, const void *_ref)
     uint8_t *buf = NULL;        /* Buffer to store serialized selection in */
     H5S_t *ret_value;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5R_get_region)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(_ref);
     HDassert(file);
@@ -580,7 +583,7 @@ H5R_get_region(H5F_t *file, hid_t dxpl_id, const void *_ref)
     INT32DECODE(p, hobjid.idx);
 
     /* Get the dataset region from the heap (allocate inside routine) */
-    if((buf = H5HG_read(oloc.file, dxpl_id, &hobjid, NULL, NULL)) == NULL)
+    if((buf = (uint8_t *)H5HG_read(oloc.file, dxpl_id, &hobjid, NULL, NULL)) == NULL)
         HGOTO_ERROR(H5E_REFERENCE, H5E_READERROR, NULL, "Unable to read dataset region information")
 
     /* Get the object oid for the dataset */
@@ -634,7 +637,7 @@ H5Rget_region(hid_t id, H5R_type_t ref_type, const void *ref)
     H5S_t *space = NULL;    /* Dataspace object */
     hid_t ret_value;
 
-    FUNC_ENTER_API(H5Rget_region, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("i", "iRt*x", id, ref_type, ref);
 
     /* Check args */
@@ -650,7 +653,7 @@ H5Rget_region(hid_t id, H5R_type_t ref_type, const void *ref)
         HGOTO_ERROR(H5E_REFERENCE, H5E_CANTCREATE, FAIL, "unable to create dataspace")
 
     /* Atomize */
-    if((ret_value = H5I_register (H5I_DATASPACE, space)) < 0)
+    if((ret_value = H5I_register (H5I_DATASPACE, space, TRUE)) < 0)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register dataspace atom")
 
 done:
@@ -685,9 +688,10 @@ H5R_get_obj_type(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type,
     const void *_ref, H5O_type_t *obj_type)
 {
     H5O_loc_t oloc;             /* Object location */
+    unsigned rc;		/* Reference count of object    */
     herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5R_get_obj_type)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(file);
     HDassert(_ref);
@@ -714,7 +718,7 @@ H5R_get_obj_type(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type,
             INT32DECODE(p, hobjid.idx);
 
             /* Get the dataset region from the heap (allocate inside routine) */
-            if((buf = H5HG_read(oloc.file, dxpl_id, &hobjid, NULL, NULL)) == NULL)
+            if((buf = (uint8_t *)H5HG_read(oloc.file, dxpl_id, &hobjid, NULL, NULL)) == NULL)
                 HGOTO_ERROR(H5E_REFERENCE, H5E_READERROR, FAIL, "Unable to read dataset region information")
 
             /* Get the object oid for the dataset */
@@ -733,14 +737,11 @@ H5R_get_obj_type(H5F_t *file, hid_t dxpl_id, H5R_type_t ref_type,
             HGOTO_ERROR(H5E_REFERENCE, H5E_UNSUPPORTED, FAIL, "internal error (unknown reference type)")
     } /* end switch */
 
-    /* Check to make certain that this object hasn't been deleted since the reference was created */
-    if(H5O_link(&oloc, 0, dxpl_id) <= 0)
+    /* Get the # of links for object, and its type */
+    /* (To check to make certain that this object hasn't been deleted since the reference was created) */
+    if(H5O_get_rc_and_type(&oloc, dxpl_id, &rc, obj_type) < 0 || 0 == rc)
         HGOTO_ERROR(H5E_REFERENCE, H5E_LINKCOUNT, FAIL, "dereferencing deleted object")
 
-    /* Get the object type */
-    if(H5O_obj_type(&oloc, obj_type, dxpl_id) < 0)
-        HGOTO_ERROR(H5E_REFERENCE, H5E_CANTGET, FAIL, "unable to get object type")
-
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 }   /* end H5R_get_obj_type() */
@@ -776,7 +777,7 @@ H5Rget_obj_type2(hid_t id, H5R_type_t ref_type, const void *ref,
     H5G_loc_t loc;              /* Object location */
     herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_API(H5Rget_obj_type2, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "iRt*x*Ot", id, ref_type, ref, obj_type);
 
     /* Check args */
@@ -832,7 +833,7 @@ H5R_get_name(H5F_t *f, hid_t lapl_id, hid_t dxpl_id, hid_t id, H5R_type_t ref_ty
     H5O_loc_t oloc;             /* Object location describing object for reference */
     ssize_t ret_value;          /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5R_get_name)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     HDassert(f);
@@ -861,7 +862,7 @@ H5R_get_name(H5F_t *f, hid_t lapl_id, hid_t dxpl_id, hid_t id, H5R_type_t ref_ty
             INT32DECODE(p, hobjid.idx);
 
             /* Get the dataset region from the heap (allocate inside routine) */
-            if((buf = H5HG_read(oloc.file, dxpl_id, &hobjid, NULL, NULL)) == NULL)
+            if((buf = (uint8_t *)H5HG_read(oloc.file, dxpl_id, &hobjid, NULL, NULL)) == NULL)
                 HGOTO_ERROR(H5E_REFERENCE, H5E_READERROR, FAIL, "Unable to read dataset region information")
 
             /* Get the object oid for the dataset */
@@ -881,7 +882,7 @@ H5R_get_name(H5F_t *f, hid_t lapl_id, hid_t dxpl_id, hid_t id, H5R_type_t ref_ty
     } /* end switch */
 
     /* Retrieve file ID for name search */
-    if((file_id = H5I_get_file_id(id)) < 0)
+    if((file_id = H5I_get_file_id(id, FALSE)) < 0)
         HGOTO_ERROR(H5E_REFERENCE, H5E_CANTGET, FAIL, "can't retrieve file ID")
 
     /* Get name, length, etc. */
@@ -890,9 +891,8 @@ H5R_get_name(H5F_t *f, hid_t lapl_id, hid_t dxpl_id, hid_t id, H5R_type_t ref_ty
 
 done:
     /* Close file ID used for search */
-    if(file_id > 0)
-        if(H5I_dec_ref(file_id) < 0)
-            HDONE_ERROR(H5E_REFERENCE, H5E_CANTCLOSEFILE, FAIL, "can't determine name")
+    if(file_id > 0 && H5I_dec_ref(file_id) < 0)
+        HDONE_ERROR(H5E_REFERENCE, H5E_CANTDEC, FAIL, "can't decrement ref count of temp ID")
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5R_get_name() */
@@ -931,7 +931,7 @@ H5Rget_name(hid_t id, H5R_type_t ref_type, const void *_ref, char *name,
     H5F_t *file;        /* File object */
     ssize_t ret_value;  /* Return value */
 
-    FUNC_ENTER_API(H5Rget_name, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE5("Zs", "iRt*x*sz", id, ref_type, _ref, name, size);
 
     /* Check args */
