@@ -19,7 +19,6 @@
 #define H5_INTERFACE_INIT_FUNC	H5S_init_interface
 
 
-#define _H5S_IN_H5S_C
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5Fprivate.h"		/* Files				*/
@@ -76,7 +75,7 @@ H5S_init_interface(void)
 {
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5S_init_interface)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Initialize the atom group for the file IDs */
     if(H5I_register_type(H5I_DATASPACE, (size_t)H5I_DATASPACEID_HASHSIZE, H5S_RESERVED_ATOMS, (H5I_free_t)H5S_close) < 0)
@@ -87,7 +86,7 @@ H5S_init_interface(void)
         /* Allow MPI buf-and-file-type optimizations? */
         const char *s = HDgetenv ("HDF5_MPI_OPT_TYPES");
         if (s && HDisdigit(*s))
-            H5S_mpi_opt_types_g = (int)HDstrtol (s, NULL, 0);
+            H5S_mpi_opt_types_g = (hbool_t)HDstrtol (s, NULL, 0);
     }
 #endif /* H5_HAVE_PARALLEL */
 
@@ -118,7 +117,7 @@ H5S_term_interface(void)
 {
     int	n = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_term_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     if(H5_interface_initialize_g) {
 	if((n = H5I_nmembers(H5I_DATASPACE))) {
@@ -162,7 +161,7 @@ H5S_create(H5S_class_t type)
     H5S_t *new_ds = NULL;    /* New dataspace created */
     H5S_t *ret_value;           /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_create, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     /* Create a new dataspace */
     if(NULL == (new_ds = H5FL_MALLOC(H5S_t)))
@@ -187,6 +186,7 @@ H5S_create(H5S_class_t type)
             new_ds->extent.nelem = 0;
             break;
 
+        case H5S_NO_CLASS:
         default:
             HDassert("unknown dataspace (extent) type" && 0);
             break;
@@ -207,8 +207,10 @@ H5S_create(H5S_class_t type)
     ret_value = new_ds;
 
 done:
-    if(ret_value == NULL && new_ds)
-        H5S_close(new_ds);
+    if(ret_value == NULL) {
+        if(new_ds && H5S_close(new_ds) < 0)
+            HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, NULL, "unable to release dataspace")
+    } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5S_create() */
@@ -238,7 +240,7 @@ H5Screate(H5S_class_t type)
     H5S_t *new_ds=NULL;         /* New dataspace structure */
     hid_t ret_value;            /* Return value */
 
-    FUNC_ENTER_API(H5Screate, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "Sc", type);
 
     /* Check args */
@@ -253,10 +255,12 @@ H5Screate(H5S_class_t type)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register dataspace atom")
 
 done:
-    if(ret_value < 0 && new_ds)
-        H5S_close(new_ds);
+    if(ret_value < 0) {
+        if(new_ds && H5S_close(new_ds) < 0)
+            HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace")
+    } /* end if */
 
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 } /* end H5Screate() */
 
 
@@ -279,20 +283,20 @@ H5S_extent_release(H5S_extent_t *extent)
 {
     herr_t ret_value=SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_extent_release, FAIL);
+    FUNC_ENTER_NOAPI(FAIL)
 
     assert(extent);
 
     /* Release extent */
-    if(extent->type==H5S_SIMPLE) {
+    if(extent->type == H5S_SIMPLE) {
         if(extent->size)
-            H5FL_ARR_FREE(hsize_t,extent->size);
+            extent->size = H5FL_ARR_FREE(hsize_t, extent->size);
         if(extent->max)
-            H5FL_ARR_FREE(hsize_t,extent->max);
+            extent->max = H5FL_ARR_FREE(hsize_t, extent->max);
     } /* end if */
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }   /* end H5S_extent_release() */
 
 
@@ -311,20 +315,22 @@ done:
 herr_t
 H5S_close(H5S_t *ds)
 {
-    herr_t ret_value = SUCCEED;   /* Return value */
+    herr_t ret_value = SUCCEED;         /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_close, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     HDassert(ds);
 
     /* Release selection (this should come before the extent release) */
-    H5S_SELECT_RELEASE(ds);
+    if(H5S_SELECT_RELEASE(ds) < 0)
+        HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace selection")
 
     /* Release extent */
-    H5S_extent_release(&ds->extent);
+    if(H5S_extent_release(&ds->extent) < 0)
+        HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace extent")
 
     /* Release the main structure */
-    (void)H5FL_FREE(H5S_t, ds);
+    ds = H5FL_FREE(H5S_t, ds);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -350,9 +356,9 @@ done:
 herr_t
 H5Sclose(hid_t space_id)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
+    herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_API(H5Sclose, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", space_id);
 
     /* Check args */
@@ -360,12 +366,12 @@ H5Sclose(hid_t space_id)
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
     /* When the reference count reaches zero the resources are freed */
-    if (H5I_dec_ref(space_id, TRUE) < 0)
-        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "problem freeing id")
+    if(H5I_dec_app_ref(space_id) < 0)
+        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDEC, FAIL, "problem freeing id")
 
 done:
-    FUNC_LEAVE_API(ret_value);
-}
+    FUNC_LEAVE_API(ret_value)
+} /* end H5Sclose() */
 
 
 /*-------------------------------------------------------------------------
@@ -391,29 +397,29 @@ H5Scopy(hid_t space_id)
     H5S_t	*dst = NULL;
     hid_t	ret_value;
 
-    FUNC_ENTER_API(H5Scopy, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", space_id);
 
     /* Check args */
-    if (NULL==(src=(H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
+    if(NULL == (src = (H5S_t *)H5I_object_verify(space_id, H5I_DATASPACE)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataspace")
 
     /* Copy */
-    if (NULL == (dst = H5S_copy(src, FALSE, TRUE)))
+    if(NULL == (dst = H5S_copy(src, FALSE, TRUE)))
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to copy dataspace")
 
     /* Atomize */
-    if ((ret_value=H5I_register (H5I_DATASPACE, dst, TRUE))<0)
+    if((ret_value = H5I_register (H5I_DATASPACE, dst, TRUE)) < 0)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register dataspace atom")
 
 done:
-    if(ret_value<0) {
-        if(dst!=NULL)
-            H5S_close(dst);
+    if(ret_value < 0) {
+        if(dst && H5S_close(dst) < 0)
+            HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace")
     } /* end if */
 
-    FUNC_LEAVE_API(ret_value);
-}
+    FUNC_LEAVE_API(ret_value)
+} /* end H5Scopy() */
 
 
 /*-------------------------------------------------------------------------
@@ -437,7 +443,7 @@ H5Sextent_copy(hid_t dst_id,hid_t src_id)
     H5S_t	*dst;
     hid_t	ret_value = SUCCEED;
 
-    FUNC_ENTER_API(H5Sextent_copy, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "ii", dst_id, src_id);
 
     /* Check args */
@@ -475,7 +481,7 @@ H5S_extent_copy(H5S_extent_t *dst, const H5S_extent_t *src, hbool_t copy_max)
     unsigned u;
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_extent_copy, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Copy the regular fields */
     dst->type = src->type;
@@ -507,6 +513,7 @@ H5S_extent_copy(H5S_extent_t *dst, const H5S_extent_t *src, hbool_t copy_max)
                 dst->max = NULL;
             break;
 
+        case H5S_NO_CLASS:
         default:
             HDassert("unknown dataspace type" && 0);
             break;
@@ -548,7 +555,7 @@ H5S_copy(const H5S_t *src, hbool_t share_selection, hbool_t copy_max)
     H5S_t		   *dst = NULL;
     H5S_t		   *ret_value;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_copy, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     if(NULL == (dst = H5FL_MALLOC(H5S_t)))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
@@ -565,6 +572,10 @@ H5S_copy(const H5S_t *src, hbool_t share_selection, hbool_t copy_max)
     ret_value = dst;
 
 done:
+    if(NULL == ret_value)
+        if(dst)
+            dst = H5FL_FREE(H5S_t, dst);
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5S_copy() */
 
@@ -595,7 +606,7 @@ H5S_get_simple_extent_npoints(const H5S_t *ds)
 {
     hssize_t    ret_value;
 
-    FUNC_ENTER_NOAPI(H5S_get_simple_extent_npoints, -1)
+    FUNC_ENTER_NOAPI(-1)
 
     /* check args */
     HDassert(ds);
@@ -631,7 +642,7 @@ H5Sget_simple_extent_npoints(hid_t space_id)
     H5S_t		   *ds;
     hssize_t		    ret_value;
 
-    FUNC_ENTER_API(H5Sget_simple_extent_npoints, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Hs", "i", space_id);
 
     /* Check args */
@@ -672,7 +683,7 @@ H5S_get_npoints_max(const H5S_t *ds)
     hsize_t	    ret_value;
     unsigned	    u;
 
-    FUNC_ENTER_NOAPI(H5S_get_npoints_max, 0);
+    FUNC_ENTER_NOAPI(0)
 
     /* check args */
     assert(ds);
@@ -703,13 +714,14 @@ H5S_get_npoints_max(const H5S_t *ds)
             }
             break;
 
+        case H5S_NO_CLASS:
         default:
             assert("unknown dataspace class" && 0);
             HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, 0, "internal error (unknown dataspace class)")
     }
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }
 
 
@@ -735,7 +747,7 @@ H5Sget_simple_extent_ndims(hid_t space_id)
     H5S_t		   *ds;
     int		   ret_value;
 
-    FUNC_ENTER_API(H5Sget_simple_extent_ndims, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("Is", "i", space_id);
 
     /* Check args */
@@ -775,7 +787,7 @@ H5S_get_simple_extent_ndims(const H5S_t *ds)
 {
     int	ret_value;      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_get_simple_extent_ndims, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* check args */
     HDassert(ds);
@@ -787,6 +799,7 @@ H5S_get_simple_extent_ndims(const H5S_t *ds)
             ret_value = (int)ds->extent.rank;
             break;
 
+        case H5S_NO_CLASS:
         default:
             HDassert("unknown dataspace class" && 0);
             HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "internal error (unknown dataspace class)")
@@ -826,7 +839,7 @@ H5Sget_simple_extent_dims(hid_t space_id, hsize_t dims[]/*out*/,
     H5S_t		   *ds;
     int		   ret_value;
 
-    FUNC_ENTER_API(H5Sget_simple_extent_dims, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("Is", "ixx", space_id, dims, maxdims);
 
     /* Check args */
@@ -836,9 +849,65 @@ H5Sget_simple_extent_dims(hid_t space_id, hsize_t dims[]/*out*/,
     ret_value = H5S_get_simple_extent_dims(ds, dims, maxdims);
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5S_extent_get_dims
+ *
+ * Purpose:	Returns the size in each dimension of a dataspace.  This
+ *		function may not be meaningful for all types of dataspaces.
+ *
+ * Return:	Success:	Number of dimensions.  Zero implies scalar.
+ *		Failure:	Negative
+ *
+ * Programmer:	Quincey Koziol
+ *		Tuesday, June 30, 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+int
+H5S_extent_get_dims(const H5S_extent_t *ext, hsize_t dims[], hsize_t max_dims[])
+{
+    int	i;              /* Local index variable */
+    int	ret_value;      /* Return value */
+
+    FUNC_ENTER_NOAPI(FAIL)
+
+    /* check args */
+    HDassert(ext);
+
+    switch(ext->type) {
+        case H5S_NULL:
+        case H5S_SCALAR:
+            ret_value = 0;
+            break;
+
+        case H5S_SIMPLE:
+            ret_value = (int)ext->rank;
+            for(i = 0; i < ret_value; i++) {
+                if(dims)
+                    dims[i] = ext->size[i];
+                if(max_dims) {
+                    if(ext->max)
+                        max_dims[i] = ext->max[i];
+                    else
+                        max_dims[i] = ext->size[i];
+                } /* end if */
+            } /* end for */
+            break;
+
+        case H5S_NO_CLASS:
+        default:
+            HDassert("unknown dataspace class" && 0);
+            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "internal error (unknown dataspace class)")
+    } /* end switch */
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5S_extent_get_dims() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5S_get_simple_extent_dims
@@ -860,38 +929,16 @@ done:
 int
 H5S_get_simple_extent_dims(const H5S_t *ds, hsize_t dims[], hsize_t max_dims[])
 {
-    int	i;              /* Local index variable */
     int	ret_value;      /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_get_simple_extent_dims, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* check args */
     HDassert(ds);
 
-    switch(H5S_GET_EXTENT_TYPE(ds)) {
-        case H5S_NULL:
-        case H5S_SCALAR:
-            ret_value = 0;
-            break;
-
-        case H5S_SIMPLE:
-            ret_value = (int)ds->extent.rank;
-            for(i = 0; i < ret_value; i++) {
-                if(dims)
-                    dims[i] = ds->extent.size[i];
-                if(max_dims) {
-                    if(ds->extent.max)
-                        max_dims[i] = ds->extent.max[i];
-                    else
-                        max_dims[i] = ds->extent.size[i];
-                } /* end if */
-            } /* end for */
-            break;
-
-        default:
-            HDassert("unknown dataspace class" && 0);
-            HGOTO_ERROR(H5E_DATASPACE, H5E_UNSUPPORTED, FAIL, "internal error (unknown dataspace class)")
-    } /* end switch */
+    /* Get dims for extent */
+    if((ret_value = H5S_extent_get_dims(&ds->extent, dims, max_dims)) < 0)
+        HGOTO_ERROR(H5E_DATASPACE, H5E_CANTGET, FAIL, "can't retrieve dataspace extent dims")
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -916,7 +963,7 @@ H5S_write(H5F_t *f, hid_t dxpl_id, H5O_t *oh, unsigned update_flags, H5S_t *ds)
 {
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_write, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     HDassert(f);
     HDassert(oh);
@@ -957,7 +1004,7 @@ H5S_append(H5F_t *f, hid_t dxpl_id, H5O_t *oh, H5S_t *ds)
 {
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_append, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     HDassert(f);
     HDassert(oh);
@@ -993,7 +1040,7 @@ H5S_read(const H5O_loc_t *loc, hid_t dxpl_id)
     H5S_t	   *ds = NULL;          /* Dataspace to return */
     H5S_t	   *ret_value;          /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_read, NULL)
+    FUNC_ENTER_NOAPI(NULL)
 
     /* check args */
     HDassert(loc);
@@ -1014,7 +1061,7 @@ H5S_read(const H5O_loc_t *loc, hid_t dxpl_id)
 done:
     if(ret_value == NULL) {
         if(ds != NULL)
-            (void)H5FL_FREE(H5S_t, ds);
+            ds = H5FL_FREE(H5S_t, ds);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1040,7 +1087,7 @@ H5S_is_simple(const H5S_t *sdim)
 {
     htri_t		    ret_value;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_is_simple)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args and all the boring stuff. */
     HDassert(sdim);
@@ -1073,7 +1120,7 @@ H5Sis_simple(hid_t space_id)
     H5S_t		   *space;	/* dataspace to modify */
     htri_t		    ret_value;
 
-    FUNC_ENTER_API(H5Sis_simple, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("t", "i", space_id);
 
     /* Check args and all the boring stuff. */
@@ -1083,7 +1130,7 @@ H5Sis_simple(hid_t space_id)
     ret_value = H5S_is_simple(space);
 
   done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -1118,16 +1165,19 @@ H5Sis_simple(hid_t space_id)
     Christian Chilan 01/17/2007
     Verifies that each element of DIMS is not equal to H5S_UNLIMITED.
 
+    Raymond Lu 03/30/2011
+    We allow 0 dimension size for non-unlimited dimension starting from 1.8.7
+    release.
 --------------------------------------------------------------------------*/
 herr_t
 H5Sset_extent_simple(hid_t space_id, int rank, const hsize_t dims[/*rank*/],
 		      const hsize_t max[/*rank*/])
 {
     H5S_t	*space;	/* dataspace to modify */
-    int	u;	/* local counting variable */
-    herr_t ret_value=SUCCEED;   /* Return value */
+    int	        u;	/* local counting variable */
+    herr_t      ret_value=SUCCEED;   /* Return value */
 
-    FUNC_ENTER_API(H5Sset_extent_simple, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE4("e", "iIs*[a1]h*[a1]h", space_id, rank, dims, max);
 
     /* Check args */
@@ -1141,8 +1191,6 @@ H5Sset_extent_simple(hid_t space_id, int rank, const hsize_t dims[/*rank*/],
         for (u=0; u<rank; u++) {
             if (H5S_UNLIMITED==dims[u])
                 HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "current dimension must have a specific size, not H5S_UNLIMITED")
-            if (((max!=NULL && max[u]!=H5S_UNLIMITED) || max==NULL) && dims[u]==0)
-                HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid dimension size")
         }
     }
     if (max!=NULL) {
@@ -1159,7 +1207,7 @@ H5Sset_extent_simple(hid_t space_id, int rank, const hsize_t dims[/*rank*/],
 	HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to set simple extent")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -1185,7 +1233,7 @@ H5S_set_extent_simple(H5S_t *space, unsigned rank, const hsize_t *dims,
     unsigned u;                 /* Local index variable */
     herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5S_set_extent_simple)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     HDassert(rank <= H5S_MAX_RANK);
@@ -1216,13 +1264,15 @@ H5S_set_extent_simple(H5S_t *space, unsigned rank, const hsize_t *dims,
         } /* end for */
         space->extent.nelem = nelem;
 
-        /* Copy the maximum dimensions if specified */
+        /* Copy the maximum dimensions if specified. Otherwise, the maximal dimensions are the 
+         * same as the dimension */
+        space->extent.max = (hsize_t *)H5FL_ARR_MALLOC(hsize_t, (size_t)rank);
         if(max != NULL) {
-            space->extent.max = (hsize_t *)H5FL_ARR_MALLOC(hsize_t, (size_t)rank);
             HDmemcpy(space->extent.max, max, sizeof(hsize_t) * rank);
-        } /* end if */
-        else
-            space->extent.max = NULL;
+        } else {
+            for(u = 0; u < space->extent.rank; u++)
+                space->extent.max[u] = dims[u];
+        }
     } /* end else */
 
     /* Selection related cleanup */
@@ -1258,14 +1308,13 @@ done:
  *
  *		Failure:	Negative
  *
- * Errors:
- *
  * Programmer:	Quincey Koziol
  *		Tuesday, January  27, 1998
  *
- * Modifications: Christian Chilan 01/17/2007
- *                Verifies that each element of DIMS is not equal to
- *                H5S_UNLIMITED.
+ * Modification:
+ *              Raymond Lu 03/30/2011
+ *              We allow 0-dimension for non-unlimited dimension starting 
+ *              from 1.8.7 release.
  *
  *-------------------------------------------------------------------------
  */
@@ -1273,52 +1322,49 @@ hid_t
 H5Screate_simple(int rank, const hsize_t dims[/*rank*/],
 		  const hsize_t maxdims[/*rank*/])
 {
-    hid_t	ret_value;
     H5S_t	*space = NULL;
     int		i;
+    hid_t	ret_value;
 
-    FUNC_ENTER_API(H5Screate_simple, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("i", "Is*[a0]h*[a0]h", rank, dims, maxdims);
 
     /* Check arguments */
-    if (rank<0)
+    if(rank < 0)
         HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "dimensionality cannot be negative")
-    if (rank>H5S_MAX_RANK)
-	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "dimensionality is too large")
-    if (!dims && dims!=0)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no dimensions specified")
+    if(rank > H5S_MAX_RANK)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "dimensionality is too large")
+
+    /* We allow users to use this function to create scalar or null dataspace.
+     * Check DIMS isn't set when the RANK is 0.
+     */
+    if(!dims && rank != 0)
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid dataspace information")
+
     /* Check whether the current dimensions are valid */
-    for (i=0; i<rank; i++) {
-        if (H5S_UNLIMITED==dims[i])
+    for(i = 0; i < rank; i++) {
+        if(H5S_UNLIMITED == dims[i])
             HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "current dimension must have a specific size, not H5S_UNLIMITED")
-        if (maxdims) {
-            if (H5S_UNLIMITED!=maxdims[i] && maxdims[i]<dims[i])
-                HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "maxdims is smaller than dims")
-            if (H5S_UNLIMITED!=maxdims[i] && dims[i]==0)
-                HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "zero sized dimension for non-unlimited dimension")
-        }
-        else {
-            if (dims[i]==0)
-                HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "zero sized dimension for non-unlimited dimension")
-        }
-    }
+        if(maxdims && H5S_UNLIMITED != maxdims[i] && maxdims[i]<dims[i])
+            HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "maxdims is smaller than dims")
+    } /* end for */
 
     /* Create the space and set the extent */
-    if(NULL==(space=H5S_create_simple((unsigned)rank,dims,maxdims)))
+    if(NULL == (space = H5S_create_simple((unsigned)rank,dims,maxdims)))
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCREATE, FAIL, "can't create simple dataspace")
 
     /* Atomize */
-    if ((ret_value=H5I_register (H5I_DATASPACE, space, TRUE))<0)
+    if((ret_value = H5I_register (H5I_DATASPACE, space, TRUE)) < 0)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTREGISTER, FAIL, "unable to register dataspace ID")
 
 done:
-    if (ret_value<0) {
-        if (space!=NULL)
-            H5S_close(space);
+    if(ret_value < 0) {
+        if(space && H5S_close(space) < 0)
+            HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release dataspace")
     } /* end if */
 
-    FUNC_LEAVE_API(ret_value);
-}
+    FUNC_LEAVE_API(ret_value)
+} /* end H5Screate_simple() */
 
 
 /*-------------------------------------------------------------------------
@@ -1346,7 +1392,7 @@ H5S_create_simple(unsigned rank, const hsize_t dims[/*rank*/],
 {
     H5S_t	*ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_create_simple, NULL);
+    FUNC_ENTER_NOAPI(NULL)
 
     /* Check arguments */
     assert(rank <=H5S_MAX_RANK);
@@ -1358,7 +1404,7 @@ H5S_create_simple(unsigned rank, const hsize_t dims[/*rank*/],
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, NULL, "can't set dimensions")
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5S_create_simple() */
 
 
@@ -1386,7 +1432,7 @@ H5Sencode(hid_t obj_id, void *buf, size_t *nalloc)
     H5S_t       *dspace;
     herr_t      ret_value=SUCCEED;
 
-    FUNC_ENTER_API (H5Sencode, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE3("e", "i*x*z", obj_id, buf, nalloc);
 
     /* Check argument and retrieve object */
@@ -1397,7 +1443,7 @@ H5Sencode(hid_t obj_id, void *buf, size_t *nalloc)
 	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTENCODE, FAIL, "can't encode datatype")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -1426,10 +1472,10 @@ H5S_encode(H5S_t *obj, unsigned char *buf, size_t *nalloc)
     H5F_t       *f = NULL;      /* Fake file structure*/
     herr_t      ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5S_encode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Allocate "fake" file structure */
-    if(NULL == (f = H5F_fake_alloc((size_t)0)))
+    if(NULL == (f = H5F_fake_alloc((uint8_t)0)))
 	HGOTO_ERROR(H5E_DATASPACE, H5E_CANTALLOC, FAIL, "can't allocate fake file struct")
 
     /* Find out the size of buffer needed for extent */
@@ -1499,7 +1545,7 @@ H5Sdecode(const void *buf)
     H5S_t       *ds;
     hid_t       ret_value;
 
-    FUNC_ENTER_API (H5Sdecode, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "*x", buf);
 
     if(buf == NULL)
@@ -1540,10 +1586,10 @@ H5S_decode(const unsigned char *buf)
     H5S_extent_t *extent;
     size_t      extent_size;            /* size of the extent message*/
     H5F_t       *f = NULL;              /* Fake file structure*/
-    size_t      sizeof_size;            /* 'Size of sizes' for file */
+    uint8_t     sizeof_size;            /* 'Size of sizes' for file */
     H5S_t       *ret_value;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5S_decode)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Decode the type of the information */
     if(*buf++ != H5O_SDSPACE_ID)
@@ -1576,7 +1622,7 @@ H5S_decode(const unsigned char *buf)
 	HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCOPY, NULL, "can't copy object")
     if(H5S_extent_release(extent) < 0)
         HGOTO_ERROR(H5E_RESOURCE, H5E_CANTDELETE, NULL, "can't release previous dataspace")
-    (void)H5FL_FREE(H5S_extent_t, extent);
+    extent = H5FL_FREE(H5S_extent_t, extent);
 
     /* Initialize to "all" selection. Deserialization relies on valid existing selection. */
     if(H5S_select_all(ds, FALSE) < 0)
@@ -1625,14 +1671,14 @@ H5S_get_simple_extent_type(const H5S_t *space)
 {
     H5S_class_t	ret_value;
 
-    FUNC_ENTER_NOAPI(H5S_get_simple_extent_type, H5S_NO_CLASS);
+    FUNC_ENTER_NOAPI(H5S_NO_CLASS)
 
     assert(space);
 
     ret_value=H5S_GET_EXTENT_TYPE(space);
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 }
 
 
@@ -1660,7 +1706,7 @@ H5Sget_simple_extent_type(hid_t sid)
     H5S_t	*space;
     H5S_class_t	ret_value;
 
-    FUNC_ENTER_API(H5Sget_simple_extent_type, H5S_NO_CLASS);
+    FUNC_ENTER_API(H5S_NO_CLASS)
     H5TRACE1("Sc", "i", sid);
 
     /* Check arguments */
@@ -1670,7 +1716,7 @@ H5Sget_simple_extent_type(hid_t sid)
     ret_value=H5S_GET_EXTENT_TYPE(space);
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }
 
 
@@ -1694,7 +1740,7 @@ H5Sset_extent_none(hid_t space_id)
     H5S_t		   *space;	/* dataspace to modify */
     herr_t                  ret_value=SUCCEED;  /* Return value */
 
-    FUNC_ENTER_API(H5Sset_extent_none, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("e", "i", space_id);
 
     /* Check args */
@@ -1708,7 +1754,7 @@ H5Sset_extent_none(hid_t space_id)
     space->extent.type=H5S_NO_CLASS;
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* end H5Sset_extent_none() */
 
 
@@ -1734,7 +1780,7 @@ H5Soffset_simple(hid_t space_id, const hssize_t *offset)
     H5S_t		   *space;	/* dataspace to modify */
     herr_t                  ret_value=SUCCEED;  /* Return value */
 
-    FUNC_ENTER_API(H5Soffset_simple, FAIL);
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("e", "i*Hs", space_id, offset);
 
     /* Check args */
@@ -1751,32 +1797,30 @@ H5Soffset_simple(hid_t space_id, const hssize_t *offset)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "can't set offset")
 
 done:
-    FUNC_LEAVE_API(ret_value);
+    FUNC_LEAVE_API(ret_value)
 }   /* end H5Soffset_simple() */
 
 
 /*-------------------------------------------------------------------------
- * Function: H5S_set_extent
+ * Function:    H5S_set_extent
  *
- * Purpose: Modify the dimensions of a dataspace. Based on H5S_extend
+ * Purpose:     Modify the dimensions of a dataspace. Based on H5S_extend
  *
- * Return: Success: Non-negative
+ * Return:      Success: Non-negative
+ *              Failure: Negative
  *
- * Failure: Negative
- *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
- *
- * Date: March 13, 2002
+ * Programmer:  Pedro Vicente, pvn@ncsa.uiuc.edu
+ *              March 13, 2002
  *
  *-------------------------------------------------------------------------
  */
-int
+htri_t
 H5S_set_extent(H5S_t *space, const hsize_t *size)
 {
     unsigned u;                 /* Local index variable */
-    herr_t ret_value = 0;       /* Return value */
+    htri_t ret_value = FALSE;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_set_extent, FAIL);
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Check args */
     HDassert(space && H5S_SIMPLE == H5S_GET_EXTENT_TYPE(space));
@@ -1785,21 +1829,26 @@ H5S_set_extent(H5S_t *space, const hsize_t *size)
     /* Verify that the dimensions being changed are allowed to change */
     for(u = 0; u < space->extent.rank; u++) {
         if(space->extent.size[u] != size[u]) {
+            /* Check for invalid dimension size modification */
             if(space->extent.max && H5S_UNLIMITED != space->extent.max[u] &&
                      space->extent.max[u] < size[u])
-                 HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "dimension cannot be modified")
-            ret_value++;
+                 HGOTO_ERROR(H5E_DATASPACE, H5E_BADVALUE, FAIL, "dimension cannot exceed the existing maximal size (new: %llu max: %llu)", (unsigned long long)size[u], (unsigned long long)space->extent.max[u])
+
+            /* Indicate that dimension size can be modified */
+            ret_value = TRUE;
         } /* end if */
     } /* end for */
 
-    /* Update */
+    /* Update dimension size(s) */
     if(ret_value)
-        H5S_set_extent_real(space, size);
+        if(H5S_set_extent_real(space, size) < 0)
+             HGOTO_ERROR(H5E_DATASPACE, H5E_CANTSET, FAIL, "failed to change dimension size(s)")
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5S_set_extent() */
 
+
 /*-------------------------------------------------------------------------
  * Function: H5S_has_extent
  *
@@ -1818,60 +1867,57 @@ done:
 hbool_t
 H5S_has_extent(const H5S_t *ds)
 {
-    htri_t ret_value;
+    hbool_t ret_value;
 
-    FUNC_ENTER_NOAPI(H5S_has_extent, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     HDassert(ds);
 
-    if(ds->extent.rank==0 && ds->extent.nelem == 0 && ds->extent.type != H5S_NULL)
+    if(0 == ds->extent.rank && 0 == ds->extent.nelem && H5S_NULL != ds->extent.type)
         ret_value = FALSE;
     else
         ret_value = TRUE;
 
-done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5S_has_extent() */
 
 
 /*-------------------------------------------------------------------------
- * Function: H5S_set_extent_real
+ * Function:    H5S_set_extent_real
  *
- * Purpose: Modify the dimensions of a dataspace. Based on H5S_extend
+ * Purpose:     Modify the dimensions of a dataspace. Based on H5S_extend
  *
- * Return: Success: Non-negative
+ * Return:      Success: Non-negative
+ *              Failure: Negative
  *
- * Failure: Negative
- *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
- *
- * Date: March 13, 2002
+ * Programmer:  Pedro Vicente, pvn@ncsa.uiuc.edu
+ *              March 13, 2002
  *
  *-------------------------------------------------------------------------
  */
 herr_t
-H5S_set_extent_real( H5S_t *space, const hsize_t *size )
+H5S_set_extent_real(H5S_t *space, const hsize_t *size)
 {
     hsize_t nelem;      /* Number of elements in extent */
     unsigned u;         /* Local index variable */
-    herr_t ret_value=SUCCEED;   /* Return value */
+    herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI(H5S_set_extent_real, FAIL );
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Check args */
-    assert(space && H5S_SIMPLE==H5S_GET_EXTENT_TYPE(space));
-    assert(size);
+    HDassert(space && H5S_SIMPLE == H5S_GET_EXTENT_TYPE(space));
+    HDassert(size);
 
     /* Change the dataspace size & re-compute the number of elements in the extent */
-    for (u=0, nelem=1; u < space->extent.rank; u++ ) {
+    for(u = 0, nelem = 1; u < space->extent.rank; u++ ) {
         space->extent.size[u] = size[u];
-        nelem*=space->extent.size[u];
+        nelem *= size[u];
     } /* end for */
     space->extent.nelem = nelem;
 
     /* If the selection is 'all', update the number of elements selected */
-    if(H5S_GET_SELECT_TYPE(space)==H5S_SEL_ALL)
-        if(H5S_select_all(space, FALSE)<0)
+    if(H5S_SEL_ALL == H5S_GET_SELECT_TYPE(space))
+        if(H5S_select_all(space, FALSE) < 0)
             HGOTO_ERROR(H5E_DATASPACE, H5E_CANTDELETE, FAIL, "can't change selection")
 
     /* Mark the dataspace as no longer shared if it was before */
@@ -1879,7 +1925,7 @@ H5S_set_extent_real( H5S_t *space, const hsize_t *size )
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTRESET, FAIL, "can't stop sharing dataspace")
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5S_set_extent_real() */
 
 
@@ -1903,7 +1949,7 @@ H5Sextent_equal(hid_t space1_id, hid_t space2_id)
     const H5S_t	*ds1, *ds2;     /* Dataspaces to compare */
     htri_t	ret_value;
 
-    FUNC_ENTER_API(H5Sextent_equal, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE2("t", "ii", space1_id, space2_id);
 
     /* check args */
@@ -1939,7 +1985,7 @@ H5S_extent_equal(const H5S_t *ds1, const H5S_t *ds2)
     unsigned u;                 /* Local index variable */
     htri_t ret_value = TRUE;    /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_extent_equal)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     HDassert(ds1);
@@ -1997,7 +2043,7 @@ done:
 hsize_t
 H5S_extent_nelem(const H5S_extent_t *ext)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5S_extent_nelem)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check args */
     HDassert(ext);
@@ -2022,9 +2068,7 @@ H5S_extent_nelem(const H5S_extent_t *ext)
 herr_t
 H5S_set_latest_version(H5S_t *ds)
 {
-    herr_t ret_value = SUCCEED;         /* Return value */
-
-    FUNC_ENTER_NOAPI(H5S_set_latest_version, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Sanity check */
     HDassert(ds);
@@ -2032,8 +2076,7 @@ H5S_set_latest_version(H5S_t *ds)
     /* Set encoding of extent to latest version */
     ds->extent.version = H5O_SDSPACE_VERSION_LATEST;
 
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
+    FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5S_set_latest_version() */
 
 #ifndef H5_NO_DEPRECATED_SYMBOLS
@@ -2058,7 +2101,7 @@ H5S_extend(H5S_t *space, const hsize_t *size)
     unsigned	u;
     int	ret_value = 0;
 
-    FUNC_ENTER_NOAPI(H5S_extend, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     /* Check args */
     HDassert(space && H5S_SIMPLE == H5S_GET_EXTENT_TYPE(space));
