@@ -82,32 +82,42 @@
 
 
 /* Define a code template for comparing scalar keys for the "CMP" in the H5SL_LOCATE macro */
-#define H5SL_LOCATE_SCALAR_CMP(TYPE, PNODE, PKEY, HASHVAL)              \
+#define H5SL_LOCATE_SCALAR_CMP(SLIST, TYPE, PNODE, PKEY, HASHVAL)              \
         (*(TYPE *)((PNODE)->key) < *(TYPE *)PKEY)
 
 /* Define a code template for comparing string keys for the "CMP" in the H5SL_LOCATE macro */
-#define H5SL_LOCATE_STRING_CMP(TYPE, PNODE, PKEY, HASHVAL)              \
+#define H5SL_LOCATE_STRING_CMP(SLIST, TYPE, PNODE, PKEY, HASHVAL)              \
         (((PNODE)->hashval == HASHVAL) ?                                \
             (HDstrcmp((const char *)(PNODE)->key, (const char *)PKEY) < 0) : \
             ((PNODE)->hashval < HASHVAL))
 
 /* Define a code template for comparing H5_obj_t keys for the "CMP" in the H5SL_LOCATE macro */
-#define H5SL_LOCATE_OBJ_CMP(TYPE, PNODE, PKEY, HASHVAL)                 \
-        ((((TYPE *)((PNODE)->key))->fileno < ((TYPE *)PKEY)->fileno) ? TRUE : (((TYPE *)((PNODE)->key))->addr < ((TYPE *)PKEY)->addr))
+#define H5SL_LOCATE_OBJ_CMP(SLIST, TYPE, PNODE, PKEY, HASHVAL)                 \
+        ((((TYPE *)((PNODE)->key))->fileno == ((TYPE *)PKEY)->fileno)          \
+        ? (((TYPE *)((PNODE)->key))->addr < ((TYPE *)PKEY)->addr)              \
+        : (((TYPE *)((PNODE)->key))->fileno < ((TYPE *)PKEY)->fileno))
+
+/* Define a code template for comparing generic keys for the "CMP" in the H5SL_LOCATE macro */
+#define H5SL_LOCATE_GENERIC_CMP(SLIST, TYPE, PNODE, PKEY, HASHVAL)             \
+        ((SLIST)->cmp((TYPE *)((PNODE)->key), (TYPE *)PKEY) < 0)
 
 
 /* Define a code template for comparing scalar keys for the "EQ" in the H5SL_LOCATE macro */
-#define H5SL_LOCATE_SCALAR_EQ(TYPE, PNODE, PKEY, HASHVAL)               \
+#define H5SL_LOCATE_SCALAR_EQ(SLIST, TYPE, PNODE, PKEY, HASHVAL)               \
         (*(TYPE *)((PNODE)->key) == *(TYPE *)PKEY)
 
 /* Define a code template for comparing string keys for the "EQ" in the H5SL_LOCATE macro */
-#define H5SL_LOCATE_STRING_EQ(TYPE, PNODE, PKEY, HASHVAL)               \
+#define H5SL_LOCATE_STRING_EQ(SLIST, TYPE, PNODE, PKEY, HASHVAL)               \
         (((PNODE)->hashval == HASHVAL) && (HDstrcmp((const char *)(PNODE)->key, (const char *)PKEY) == 0))
 
 /* Define a code template for comparing H5_obj_t keys for the "EQ" in the H5SL_LOCATE macro */
-#define H5SL_LOCATE_OBJ_EQ(TYPE, PNODE, PKEY, HASHVAL)                  \
+#define H5SL_LOCATE_OBJ_EQ(SLIST, TYPE, PNODE, PKEY, HASHVAL)                  \
         ((((TYPE *)((PNODE)->key))->fileno == ((TYPE *)PKEY)->fileno) && (((TYPE *)((PNODE)->key))->addr == ((TYPE *)PKEY)->addr))
 
+/* Define a code template for comparing generic keys for the "EQ" in the H5SL_LOCATE macro */
+#define H5SL_LOCATE_GENERIC_EQ(SLIST, TYPE, PNODE, PKEY, HASHVAL)             \
+        ((SLIST)->cmp((TYPE *)((PNODE)->key), (TYPE *)PKEY) == 0)
+
 
 /* Define a code template for initializing the hash value for scalar keys for the "HASHINIT" in the H5SL_LOCATE macro */
 #define H5SL_LOCATE_SCALAR_HASHINIT(KEY, HASHVAL)
@@ -119,6 +129,9 @@
 /* Define a code template for initializing the hash value for H5_obj_t keys for the "HASHINIT" in the H5SL_LOCATE macro */
 #define H5SL_LOCATE_OBJ_HASHINIT(KEY, HASHVAL)
 
+/* Define a code template for initializing the hash value for generic keys for the "HASHINIT" in the H5SL_LOCATE macro */
+#define H5SL_LOCATE_GENERIC_HASHINIT(KEY, HASHVAL)
+
 
 /* Macro used to find node for operation */
 #define H5SL_LOCATE(OP, CMP, SLIST, X, TYPE, KEY, HASHVAL)      \
@@ -130,15 +143,15 @@
     for(_i = (int)SLIST->curr_level; _i >= 0; _i--) {                   \
         _count = 0;                                                     \
         while(_count < 3 && X->forward[_i] &&                           \
-                H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(TYPE, X->forward[_i], KEY, HASHVAL) ) { \
+                H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(SLIST, TYPE, X->forward[_i], KEY, HASHVAL) ) { \
             X = X->forward[_i];                                         \
             _count++;                                                   \
         } /* end while */                                               \
     } /* end for */                                                     \
     X = X->forward[0];                                                  \
-    if(X != NULL && H5_GLUE3(H5SL_LOCATE_,CMP,_EQ)(TYPE, X, KEY, HASHVAL) ) {    \
+    if(X != NULL && H5_GLUE3(H5SL_LOCATE_,CMP,_EQ)(SLIST, TYPE, X, KEY, HASHVAL) ) { \
         /* What to do when a node is found */				\
-        H5_GLUE3(H5SL_LOCATE_,OP,_FOUND)(SLIST, X, _i)          \
+        H5_GLUE3(H5SL_LOCATE_,OP,_FOUND)(SLIST, X, _i)                  \
     } /* end if */                                                      \
 }
 
@@ -176,7 +189,7 @@
         if(NULL == (_tmp = (H5SL_node_t **)H5FL_FAC_MALLOC(H5SL_fac_g[X->log_nalloc]))) \
             HGOTO_ERROR(H5E_SLIST, H5E_NOSPACE, NULL, "memory allocation failed") \
         HDmemcpy((void *)_tmp, (const void *)X->forward, (LVL + 1) * sizeof(H5SL_node_t *)); \
-        (void)H5FL_FAC_FREE(H5SL_fac_g[X->log_nalloc-1], (void *)X->forward);  \
+        X->forward = (H5SL_node_t **)H5FL_FAC_FREE(H5SL_fac_g[X->log_nalloc-1], (void *)X->forward);  \
         X->forward = _tmp;                                                     \
     } /* end if */                                                             \
                                                                                \
@@ -198,7 +211,7 @@
         if(NULL == (_tmp = (H5SL_node_t **)H5FL_FAC_MALLOC(H5SL_fac_g[X->log_nalloc]))) \
             HGOTO_ERROR(H5E_SLIST, H5E_NOSPACE, NULL, "memory allocation failed") \
         HDmemcpy((void *)_tmp, (const void *)X->forward, (LVL) * sizeof(H5SL_node_t *)); \
-        (void)H5FL_FAC_FREE(H5SL_fac_g[X->log_nalloc+1], (void *)X->forward);  \
+        X->forward = (H5SL_node_t **)H5FL_FAC_FREE(H5SL_fac_g[X->log_nalloc+1], (void *)X->forward);  \
         X->forward = _tmp;                                                     \
     } /* end if */                                                             \
                                                                                \
@@ -266,7 +279,7 @@
             } /* end if */                                                     \
                                                                                \
             /* Check if this node is the start of the next gap */              \
-            if(!_drop && !H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(TYPE, X->forward[_i], KEY, HASHVAL)) \
+            if(!_drop && !H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(SLIST, TYPE, X->forward[_i], KEY, HASHVAL)) \
                 _drop = X;                                                     \
                                                                                \
             /* No need to check the last node in the gap if there are 3, as */ \
@@ -280,7 +293,7 @@
             X = X->forward[_i];                                                \
         } /* end for */                                                        \
         HDassert(!_drop->forward[_i] ||                                        \
-                !H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(TYPE, _drop->forward[_i], KEY, HASHVAL)); \
+                !H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(SLIST, TYPE, _drop->forward[_i], KEY, HASHVAL)); \
                                                                                \
         /* Promote the middle node if necessary */                             \
         if(_count == 3) {                                                      \
@@ -293,7 +306,7 @@
         _next = _drop->forward[_i];                                            \
     } /* end for */                                                            \
                                                                                \
-    if(_next && H5_GLUE3(H5SL_LOCATE_,CMP,_EQ)(TYPE, _next, KEY, HASHVAL))     \
+    if(_next && H5_GLUE3(H5SL_LOCATE_,CMP,_EQ)(SLIST, TYPE, _next, KEY, HASHVAL)) \
         HGOTO_ERROR(H5E_SLIST, H5E_CANTINSERT, NULL, "can't insert duplicate key") \
 }
 
@@ -316,7 +329,7 @@
     H5_GLUE3(H5SL_LOCATE_,CMP,_HASHINIT)(KEY, HASHVAL)                         \
                                                                                \
     /* Find the gap to drop in to at the highest level */                      \
-    while(X && (!X->key || H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(TYPE, X, KEY, HASHVAL))) { \
+    while(X && (!X->key || H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(SLIST, TYPE, X, KEY, HASHVAL))) { \
         _llast = _last;                                                        \
         _last = X;                                                             \
         X = X->forward[_i];                                                    \
@@ -347,7 +360,7 @@
                 break;                                                         \
             } else { /* !_drop */                                              \
                 /* Check if this node is the start of the next gap */          \
-                if (!H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(TYPE, X->forward[_i], KEY, HASHVAL)) { \
+                if (!H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(SLIST, TYPE, X->forward[_i], KEY, HASHVAL)) { \
                     _drop = X;                                                 \
                     /* Again check if we can stop searching */                 \
                     if(_count) {                                               \
@@ -370,7 +383,7 @@
         } /* end for */                                                        \
         HDassert(_count >= 1 && _count <= 3);                                  \
         HDassert(!_drop->forward[_i] ||                                        \
-                !H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(TYPE, _drop->forward[_i], KEY, HASHVAL)); \
+                !H5_GLUE3(H5SL_LOCATE_,CMP,_CMP)(SLIST, TYPE, _drop->forward[_i], KEY, HASHVAL)); \
                                                                                \
         /* Check if we need to adjust node heights */                          \
         if(_count == 1) {                                                      \
@@ -431,7 +444,7 @@
     } /* end for */                                                            \
                                                                                \
     /* Check if we've found the node */                                        \
-    if(_next && H5_GLUE3(H5SL_LOCATE_,CMP,_EQ)(TYPE, _next, KEY, HASHVAL)) {   \
+    if(_next && H5_GLUE3(H5SL_LOCATE_,CMP,_EQ)(SLIST, TYPE, _next, KEY, HASHVAL)) { \
         void *tmp = _next->item;                                               \
         X = _next;                                                             \
                                                                                \
@@ -451,8 +464,8 @@
         else                                                                   \
             X->forward[0]->backward = X->backward;                             \
         SLIST->nobjs--;                                                        \
-        (void)H5FL_FAC_FREE(H5SL_fac_g[0], X->forward);                        \
-        (void)H5FL_FREE(H5SL_node_t, X);                                       \
+        X->forward = (H5SL_node_t **)H5FL_FAC_FREE(H5SL_fac_g[0], X->forward);                        \
+        X = H5FL_FREE(H5SL_node_t, X);                                       \
                                                                                \
         HGOTO_DONE(tmp);                                                       \
     } /* end if */                                                             \
@@ -485,6 +498,7 @@ struct H5SL_node_t {
 struct H5SL_t {
     /* Static values for each list */
     H5SL_type_t type;   /* Type of skip list */
+    H5SL_cmp_t cmp;     /* Comparison callback, if type is H5SL_TYPE_GENERIC */
 
     /* Dynamic values for each list */
     int curr_level;             /* Current top level used in list */
@@ -531,7 +545,7 @@ static size_t H5SL_fac_nalloc_g;
 static herr_t
 H5SL_init_interface(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_init_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Allocate space for array of factories */
     H5SL_fac_g = (H5FL_fac_head_t **)H5MM_malloc(sizeof(H5FL_fac_head_t *));
@@ -572,7 +586,7 @@ H5SL_new_node(void *item, const void *key, uint32_t hashval)
 {
     H5SL_node_t *ret_value;      /* New skip list node */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5SL_new_node)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Allocate the node */
     if(NULL == (ret_value = (H5SL_node_t *)H5FL_MALLOC(H5SL_node_t)))
@@ -583,8 +597,10 @@ H5SL_new_node(void *item, const void *key, uint32_t hashval)
     ret_value->item = item;
     ret_value->level = 0;
     ret_value->hashval = hashval;
-    if(NULL == (ret_value->forward = (H5SL_node_t **)H5FL_FAC_MALLOC(H5SL_fac_g[0])))
+    if(NULL == (ret_value->forward = (H5SL_node_t **)H5FL_FAC_MALLOC(H5SL_fac_g[0]))) {
+        ret_value = H5FL_FREE(H5SL_node_t, ret_value);
         HGOTO_ERROR(H5E_SLIST, H5E_NOSPACE, NULL, "memory allocation failed")
+    } /* end if */
     ret_value->log_nalloc = 0;
 
 done:
@@ -621,7 +637,7 @@ H5SL_insert_common(H5SL_t *slist, void *item, const void *key)
     uint32_t hashval = 0;                       /* Hash value for key */
     H5SL_node_t *ret_value;                     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5SL_insert_common);
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     HDassert(slist);
@@ -665,6 +681,10 @@ H5SL_insert_common(H5SL_t *slist, void *item, const void *key)
             H5SL_INSERT(OBJ, slist, prev, const H5_obj_t, key, -)
             break;
 
+        case H5SL_TYPE_GENERIC:
+            H5SL_INSERT(GENERIC, slist, prev, const void, key, -)
+            break;
+
         default:
             HDassert(0 && "Unknown skiplist type!");
     } /* end switch */
@@ -697,7 +717,7 @@ H5SL_insert_common(H5SL_t *slist, void *item, const void *key)
     ret_value=x;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_insert_common() */
 
 
@@ -731,7 +751,7 @@ H5SL_release_common(H5SL_t *slist, H5SL_operator_t op, void *op_data)
     H5SL_node_t *node, *next_node;      /* Pointers to skip list nodes */
     herr_t ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5SL_release_common);
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     assert(slist);
@@ -749,13 +769,13 @@ H5SL_release_common(H5SL_t *slist, H5SL_operator_t op, void *op_data)
             /* Casting away const OK -QAK */
             (void)(op)(node->item,(void *)node->key,op_data);
 
-        (void)H5FL_FAC_FREE(H5SL_fac_g[node->log_nalloc], node->forward);
-        (void)H5FL_FREE(H5SL_node_t, node);
-        node=next_node;
+        node->forward = (H5SL_node_t **)H5FL_FAC_FREE(H5SL_fac_g[node->log_nalloc], node->forward);
+        node = H5FL_FREE(H5SL_node_t, node);
+        node = next_node;
     } /* end while */
 
     /* Reset the header pointers */
-    (void)H5FL_FAC_FREE(H5SL_fac_g[slist->header->log_nalloc], slist->header->forward);
+    slist->header->forward = (H5SL_node_t **)H5FL_FAC_FREE(H5SL_fac_g[slist->header->log_nalloc], slist->header->forward);
     if(NULL == (slist->header->forward = (H5SL_node_t **) H5FL_FAC_MALLOC(H5SL_fac_g[0])))
         HGOTO_ERROR(H5E_SLIST, H5E_NOSPACE, FAIL, "memory allocation failed")
     slist->header->forward[0] = NULL;
@@ -770,7 +790,7 @@ H5SL_release_common(H5SL_t *slist, H5SL_operator_t op, void *op_data)
     slist->nobjs=0;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_release_common() */
 
 
@@ -802,7 +822,7 @@ H5SL_close_common(H5SL_t *slist, H5SL_operator_t op, void *op_data)
 {
     herr_t ret_value = SUCCEED;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5SL_close_common)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     HDassert(slist);
@@ -815,11 +835,11 @@ H5SL_close_common(H5SL_t *slist, H5SL_operator_t op, void *op_data)
         HGOTO_ERROR(H5E_SLIST, H5E_CANTFREE, FAIL, "can't release skip list nodes")
 
     /* Release header node */
-    (void)H5FL_FAC_FREE(H5SL_fac_g[slist->header->log_nalloc], slist->header->forward);
-    (void)H5FL_FREE(H5SL_node_t, slist->header);
+    slist->header->forward = (H5SL_node_t **)H5FL_FAC_FREE(H5SL_fac_g[slist->header->log_nalloc], slist->header->forward);
+    slist->header = H5FL_FREE(H5SL_node_t, slist->header);
 
     /* Free skip list object */
-    (void)H5FL_FREE(H5SL_t, slist);
+    slist = H5FL_FREE(H5SL_t, slist);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -832,7 +852,7 @@ done:
  PURPOSE
     Create a skip list
  USAGE
-    H5SL_t *H5SL_create(H5SL_type_t type)
+    H5SL_t *H5SL_create(H5SL_type_t type, H5SL_cmp_t cmp)
 
  RETURNS
     Returns a pointer to a skip list on success, NULL on failure.
@@ -844,16 +864,16 @@ done:
  REVISION LOG
 --------------------------------------------------------------------------*/
 H5SL_t *
-H5SL_create(H5SL_type_t type)
+H5SL_create(H5SL_type_t type, H5SL_cmp_t cmp)
 {
     H5SL_t *new_slist = NULL;   /* Pointer to new skip list object created */
     H5SL_node_t *header;        /* Pointer to skip list header node */
     H5SL_t *ret_value;          /* Return value */
 
-    FUNC_ENTER_NOAPI(H5SL_create,NULL);
+    FUNC_ENTER_NOAPI(NULL)
 
     /* Check args */
-    HDassert(type>=H5SL_TYPE_INT && type<=H5SL_TYPE_OBJ);
+    HDassert(type>=H5SL_TYPE_INT && type<=H5SL_TYPE_GENERIC);
 
     /* Allocate skip list structure */
     if(NULL == (new_slist = H5FL_MALLOC(H5SL_t)))
@@ -861,6 +881,8 @@ H5SL_create(H5SL_type_t type)
 
     /* Set the static internal fields */
     new_slist->type = type;
+    HDassert((type == H5SL_TYPE_GENERIC) == !!cmp);
+    new_slist->cmp = cmp;
 
     /* Set the dynamic internal fields */
     new_slist->curr_level = -1;
@@ -887,7 +909,7 @@ done:
     /* Error cleanup */
     if(ret_value == NULL) {
         if(new_slist != NULL)
-            (void)H5FL_FREE(H5SL_t, new_slist);
+            new_slist = H5FL_FREE(H5SL_t, new_slist);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -915,7 +937,7 @@ done:
 size_t
 H5SL_count(H5SL_t *slist)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_count);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -923,7 +945,7 @@ H5SL_count(H5SL_t *slist)
     /* Check internal consistency */
     /* (Pre-condition) */
 
-    FUNC_LEAVE_NOAPI(slist->nobjs);
+    FUNC_LEAVE_NOAPI(slist->nobjs)
 } /* end H5SL_count() */
 
 
@@ -953,7 +975,7 @@ H5SL_insert(H5SL_t *slist, void *item, const void *key)
 {
     herr_t ret_value=SUCCEED;                   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5SL_insert);
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     assert(slist);
@@ -967,7 +989,7 @@ H5SL_insert(H5SL_t *slist, void *item, const void *key)
         HGOTO_ERROR(H5E_SLIST,H5E_CANTINSERT,FAIL,"can't create new skip list node")
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_insert() */
 
 
@@ -1000,7 +1022,7 @@ H5SL_add(H5SL_t *slist, void *item, const void *key)
 {
     H5SL_node_t *ret_value;                   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5SL_add);
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     assert(slist);
@@ -1014,7 +1036,7 @@ H5SL_add(H5SL_t *slist, void *item, const void *key)
         HGOTO_ERROR(H5E_SLIST,H5E_CANTINSERT,NULL,"can't create new skip list node")
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_add() */
 
 
@@ -1044,7 +1066,7 @@ H5SL_remove(H5SL_t *slist, const void *key)
     uint32_t hashval = 0;                       /* Hash value for key */
     void *ret_value = NULL;                     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5SL_remove)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     HDassert(slist);
@@ -1088,6 +1110,10 @@ H5SL_remove(H5SL_t *slist, const void *key)
             H5SL_REMOVE(OBJ, slist, x, const H5_obj_t, key, -)
             break;
 
+        case H5SL_TYPE_GENERIC:
+            H5SL_REMOVE(GENERIC, slist, x, const void, key, -)
+            break;
+
         default:
             HDassert(0 && "Unknown skiplist type!");
     } /* end switch */
@@ -1125,7 +1151,7 @@ H5SL_remove_first(H5SL_t *slist)
     size_t      level = slist->curr_level;      /* Skip list level */
     size_t      i;                              /* Index */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5SL_remove_first)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     HDassert(slist);
@@ -1151,8 +1177,8 @@ H5SL_remove_first(H5SL_t *slist)
             tmp->forward[0]->backward = head;
         slist->nobjs--;
         /* Free memory */
-        (void)H5FL_FAC_FREE(H5SL_fac_g[0], tmp->forward);
-        (void)H5FL_FREE(H5SL_node_t, tmp);
+        tmp->forward = (H5SL_node_t **)H5FL_FAC_FREE(H5SL_fac_g[0], tmp->forward);
+        tmp = H5FL_FREE(H5SL_node_t, tmp);
 
         /* Reshape the skip list as necessary to maintain 1-2-3 condition */
         for(i=0; i < level; i++) {
@@ -1223,7 +1249,7 @@ H5SL_search(H5SL_t *slist, const void *key)
     uint32_t hashval = 0;                       /* Hash value for key */
     void *ret_value;                            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_search);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -1267,6 +1293,10 @@ H5SL_search(H5SL_t *slist, const void *key)
             H5SL_SEARCH(OBJ, slist, x, const H5_obj_t, key, -)
             break;
 
+        case H5SL_TYPE_GENERIC:
+            H5SL_SEARCH(GENERIC, slist, x, const void, key, -)
+            break;
+
         default:
             HDassert(0 && "Unknown skiplist type!");
     } /* end switch */
@@ -1275,7 +1305,7 @@ H5SL_search(H5SL_t *slist, const void *key)
     ret_value=NULL;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_search() */
 
 
@@ -1308,7 +1338,7 @@ H5SL_less(H5SL_t *slist, const void *key)
     uint32_t hashval = 0;                       /* Hash value for key */
     void *ret_value;                            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_less);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -1352,6 +1382,10 @@ H5SL_less(H5SL_t *slist, const void *key)
             H5SL_SEARCH(OBJ, slist, x, const H5_obj_t, key, -)
             break;
 
+        case H5SL_TYPE_GENERIC:
+            H5SL_SEARCH(GENERIC, slist, x, const void, key, -)
+            break;
+
         default:
             HDassert(0 && "Unknown skiplist type!");
     } /* end switch */
@@ -1373,7 +1407,7 @@ H5SL_less(H5SL_t *slist, const void *key)
     } /* end else */
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_less() */
 
 
@@ -1406,7 +1440,7 @@ H5SL_greater(H5SL_t *slist, const void *key)
     uint32_t hashval = 0;                       /* Hash value for key */
     void *ret_value;                            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_greater);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     HDassert(slist);
@@ -1450,6 +1484,10 @@ H5SL_greater(H5SL_t *slist, const void *key)
             H5SL_SEARCH(OBJ, slist, x, const H5_obj_t, key, -)
             break;
 
+        case H5SL_TYPE_GENERIC:
+            H5SL_SEARCH(GENERIC, slist, x, const void, key, -)
+            break;
+
         default:
             HDassert(0 && "Unknown skiplist type!");
     } /* end switch */
@@ -1462,7 +1500,7 @@ H5SL_greater(H5SL_t *slist, const void *key)
         ret_value = NULL;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_greater() */
 
 
@@ -1494,7 +1532,7 @@ H5SL_find(H5SL_t *slist, const void *key)
     uint32_t hashval = 0;                       /* Hash value for key */
     H5SL_node_t *ret_value;                     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_find);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -1538,6 +1576,10 @@ H5SL_find(H5SL_t *slist, const void *key)
             H5SL_FIND(OBJ, slist, x, const H5_obj_t, key, -)
             break;
 
+        case H5SL_TYPE_GENERIC:
+            H5SL_FIND(GENERIC, slist, x, const void, key, -)
+            break;
+
         default:
             HDassert(0 && "Unknown skiplist type!");
     } /* end switch */
@@ -1546,7 +1588,7 @@ H5SL_find(H5SL_t *slist, const void *key)
     ret_value=NULL;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_find() */
 
 
@@ -1579,7 +1621,7 @@ H5SL_below(H5SL_t *slist, const void *key)
     uint32_t hashval = 0;                       /* Hash value for key */
     H5SL_node_t *ret_value;                     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_below)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     HDassert(slist);
@@ -1622,6 +1664,10 @@ H5SL_below(H5SL_t *slist, const void *key)
         case H5SL_TYPE_OBJ:
             H5SL_FIND(OBJ, slist, x, const H5_obj_t, key, -)
             break;
+
+        case H5SL_TYPE_GENERIC:
+            H5SL_FIND(GENERIC, slist, x, const void, key, -)
+            break;
     } /* end switch */
 
     /* An exact match for 'key' must not have been found in list, if we get here */
@@ -1674,7 +1720,7 @@ H5SL_above(H5SL_t *slist, const void *key)
     uint32_t hashval = 0;                       /* Hash value for key */
     H5SL_node_t *ret_value;                     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_above)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     HDassert(slist);
@@ -1717,6 +1763,10 @@ H5SL_above(H5SL_t *slist, const void *key)
         case H5SL_TYPE_OBJ:
             H5SL_FIND(OBJ, slist, x, const H5_obj_t, key, -)
             break;
+
+        case H5SL_TYPE_GENERIC:
+            H5SL_FIND(GENERIC, slist, x, const void, key, -)
+            break;
     } /* end switch */
 
     /* An exact match for 'key' must not have been found in list, if we get here */
@@ -1753,7 +1803,7 @@ done:
 H5SL_node_t *
 H5SL_first(H5SL_t *slist)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_first);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -1761,7 +1811,7 @@ H5SL_first(H5SL_t *slist)
     /* Check internal consistency */
     /* (Pre-condition) */
 
-    FUNC_LEAVE_NOAPI(slist->header->forward[0]);
+    FUNC_LEAVE_NOAPI(slist->header->forward[0])
 } /* end H5SL_first() */
 
 
@@ -1787,7 +1837,7 @@ H5SL_first(H5SL_t *slist)
 H5SL_node_t *
 H5SL_next(H5SL_node_t *slist_node)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_next);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist_node);
@@ -1795,7 +1845,7 @@ H5SL_next(H5SL_node_t *slist_node)
     /* Check internal consistency */
     /* (Pre-condition) */
 
-    FUNC_LEAVE_NOAPI(slist_node->forward[0]);
+    FUNC_LEAVE_NOAPI(slist_node->forward[0])
 } /* end H5SL_next() */
 
 
@@ -1821,7 +1871,7 @@ H5SL_next(H5SL_node_t *slist_node)
 H5SL_node_t *
 H5SL_prev(H5SL_node_t *slist_node)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_prev);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist_node);
@@ -1830,7 +1880,7 @@ H5SL_prev(H5SL_node_t *slist_node)
     /* (Pre-condition) */
 
     /* Walk backward, detecting the header node (which has it's key set to NULL) */
-    FUNC_LEAVE_NOAPI(slist_node->backward->key==NULL ? NULL : slist_node->backward);
+    FUNC_LEAVE_NOAPI(slist_node->backward->key==NULL ? NULL : slist_node->backward)
 } /* end H5SL_prev() */
 
 
@@ -1856,7 +1906,7 @@ H5SL_prev(H5SL_node_t *slist_node)
 H5SL_node_t *
 H5SL_last(H5SL_t *slist)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_last);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -1865,7 +1915,7 @@ H5SL_last(H5SL_t *slist)
     /* (Pre-condition) */
 
     /* Find last node, avoiding the header node */
-    FUNC_LEAVE_NOAPI(slist->last==slist->header ? NULL : slist->last);
+    FUNC_LEAVE_NOAPI(slist->last==slist->header ? NULL : slist->last)
 } /* end H5SL_last() */
 
 
@@ -1890,7 +1940,7 @@ H5SL_last(H5SL_t *slist)
 void *
 H5SL_item(H5SL_node_t *slist_node)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_item);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist_node);
@@ -1898,7 +1948,7 @@ H5SL_item(H5SL_node_t *slist_node)
     /* Check internal consistency */
     /* (Pre-condition) */
 
-    FUNC_LEAVE_NOAPI(slist_node->item);
+    FUNC_LEAVE_NOAPI(slist_node->item)
 } /* end H5SL_item() */
 
 
@@ -1941,7 +1991,7 @@ H5SL_iterate(H5SL_t *slist, H5SL_operator_t op, void *op_data)
     H5SL_node_t *node;  /* Pointers to skip list nodes */
     herr_t ret_value = 0; /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_iterate)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     HDassert(slist);
@@ -1987,7 +2037,7 @@ H5SL_iterate(H5SL_t *slist, H5SL_operator_t op, void *op_data)
 herr_t
 H5SL_release(H5SL_t *slist)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_release);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -1998,7 +2048,7 @@ H5SL_release(H5SL_t *slist)
     /* Free skip list nodes */
     H5SL_release_common(slist,NULL,NULL); /* always succeeds */
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
+    FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5SL_release() */
 
 
@@ -2033,7 +2083,7 @@ H5SL_release(H5SL_t *slist)
 herr_t
 H5SL_free(H5SL_t *slist, H5SL_operator_t op, void *op_data)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_free);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -2044,7 +2094,7 @@ H5SL_free(H5SL_t *slist, H5SL_operator_t op, void *op_data)
     /* Free skip list nodes */
     H5SL_release_common(slist,op,op_data); /* always succeeds */
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
+    FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5SL_free() */
 
 
@@ -2079,7 +2129,7 @@ H5SL_destroy(H5SL_t *slist, H5SL_operator_t op, void *op_data)
 {
     herr_t ret_value=SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_destroy);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -2090,7 +2140,7 @@ H5SL_destroy(H5SL_t *slist, H5SL_operator_t op, void *op_data)
     /* Close skip list */
     (void)H5SL_close_common(slist,op,op_data); /* always succeeds */
 
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5SL_destroy() */
 
 
@@ -2116,7 +2166,7 @@ H5SL_destroy(H5SL_t *slist, H5SL_operator_t op, void *op_data)
 herr_t
 H5SL_close(H5SL_t *slist)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_close);
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Check args */
     assert(slist);
@@ -2127,7 +2177,7 @@ H5SL_close(H5SL_t *slist)
     /* Close skip list */
     (void)H5SL_close_common(slist,NULL,NULL); /* always succeeds */
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
+    FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5SL_close() */
 
 
@@ -2156,7 +2206,7 @@ int H5SL_term_interface(void)
     herr_t  ret;
     int     n = H5_interface_initialize_g ? 1 : 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5SL_term_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     if(n) {
         /* Terminate all the factories */
@@ -2167,13 +2217,13 @@ int H5SL_term_interface(void)
         H5SL_fac_nused_g = 0;
 
         /* Free the list of factories */
-        H5SL_fac_g = (H5FL_reg_head_t **)H5MM_xfree((void *)H5SL_fac_g);
+        H5SL_fac_g = (H5FL_fac_head_t **)H5MM_xfree((void *)H5SL_fac_g);
         H5SL_fac_nalloc_g = 0;
 
         /* Mark the interface as uninitialized */
         H5_interface_initialize_g = 0;
     } /* end if */
 
-    FUNC_LEAVE_NOAPI(n);
+    FUNC_LEAVE_NOAPI(n)
 } /* H5SL_term_interface() */
 
