@@ -76,7 +76,9 @@ const H5O_obj_class_t H5O_OBJ_DATATYPE[1] = {{
     H5O_dtype_isa, 		/* "isa" 			*/
     H5O_dtype_open, 		/* open an object of this class */
     H5O_dtype_create, 		/* create an object of this class */
-    H5O_dtype_get_oloc 		/* get an object header location for an object */
+    H5O_dtype_get_oloc,		/* get an object header location for an object */
+    NULL,			/* get the index & heap info for an object */
+    NULL 			/* flush an opened object of this class */
 }};
 
 
@@ -97,12 +99,12 @@ const H5O_obj_class_t H5O_OBJ_DATATYPE[1] = {{
  *
  *-------------------------------------------------------------------------
  */
-htri_t
+static htri_t
 H5O_dtype_isa(struct H5O_t *oh)
 {
     htri_t	ret_value;              /* Return value */
 
-    FUNC_ENTER_NOAPI(H5O_dtype_isa, FAIL)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(oh);
 
@@ -133,7 +135,7 @@ H5O_dtype_open(const H5G_loc_t *obj_loc, hid_t UNUSED lapl_id, hid_t dxpl_id, hb
     H5T_t       *type = NULL;           /* Datatype opened */
     hid_t	ret_value;              /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_dtype_open)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(obj_loc);
 
@@ -173,7 +175,7 @@ H5O_dtype_create(H5F_t *f, void *_crt_info, H5G_loc_t *obj_loc, hid_t dxpl_id)
     H5T_obj_create_t *crt_info = (H5T_obj_create_t *)_crt_info; /* Named datatype creation parameters */
     void *ret_value;            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_dtype_create)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Sanity checks */
     HDassert(f);
@@ -217,7 +219,7 @@ H5O_dtype_get_oloc(hid_t obj_id)
     H5T_t       *type;                  /* Datatype opened */
     H5O_loc_t	*ret_value;             /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_dtype_get_oloc)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Get the datatype */
     if(NULL == (type = (H5T_t *)H5I_object(obj_id)))
