@@ -81,7 +81,7 @@ DESCRIPTION
 static herr_t
 H5T_init_vlen_interface(void)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_init_vlen_interface)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     FUNC_LEAVE_NOAPI(H5T_init())
 } /* H5T_init_vlen_interface() */
@@ -100,8 +100,6 @@ H5T_init_vlen_interface(void)
  * Programmer:	Quincey Koziol
  *              Thursday, May 20, 1999
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 hid_t
@@ -111,7 +109,7 @@ H5Tvlen_create(hid_t base_id)
     H5T_t	*dt = NULL;		/*new datatype	*/
     hid_t	ret_value;	        /*return value			*/
 
-    FUNC_ENTER_API(H5Tvlen_create, FAIL)
+    FUNC_ENTER_API(FAIL)
     H5TRACE1("i", "i", base_id);
 
     /* Check args */
@@ -144,8 +142,6 @@ done:
  * Programmer:	Quincey Koziol
  *              Tuesday, November 20, 2001
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 H5T_t *
@@ -154,14 +150,14 @@ H5T_vlen_create(const H5T_t *base)
     H5T_t	*dt = NULL;		/*new VL datatype	*/
     H5T_t	*ret_value;	/*return value			*/
 
-    FUNC_ENTER_NOAPI_NOINIT(H5T_vlen_create)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* Check args */
     HDassert(base);
 
     /* Build new type */
     if(NULL == (dt = H5T_alloc()))
-        HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+        HGOTO_ERROR(H5E_DATATYPE, H5E_CANTALLOC, NULL, "memory allocation failed")
     dt->shared->type = H5T_VLEN;
 
     /*
@@ -169,7 +165,8 @@ H5T_vlen_create(const H5T_t *base)
      * data, not point to the same VL sequences)
      */
     dt->shared->force_conv = TRUE;
-    dt->shared->parent = H5T_copy(base, H5T_COPY_ALL);
+    if(NULL == (dt->shared->parent = H5T_copy(base, H5T_COPY_ALL)))
+        HGOTO_ERROR(H5E_DATATYPE, H5E_CANTCOPY, NULL, "can't copy base datatype")
 
     /* Inherit encoding version from base type */
     dt->shared->version = base->shared->version;
@@ -185,6 +182,10 @@ H5T_vlen_create(const H5T_t *base)
     ret_value = dt;
 
 done:
+    if(!ret_value)
+        if(dt && H5T_close(dt) < 0)
+            HDONE_ERROR(H5E_DATATYPE, H5E_CANTRELEASE, NULL, "unable to release datatype info")
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5T_vlen_create() */
 
@@ -210,11 +211,11 @@ H5T_vlen_set_loc(const H5T_t *dt, H5F_t *f, H5T_loc_t loc)
 {
     htri_t ret_value = FALSE;   /* Indicate success, but no location change */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5T_vlen_set_loc)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check parameters */
     HDassert(dt);
-    HDassert(loc > H5T_LOC_BADLOC && loc < H5T_LOC_MAXLOC);
+    HDassert(loc >= H5T_LOC_BADLOC && loc < H5T_LOC_MAXLOC);
 
     /* Only change the location if it's different */
     if(loc != dt->shared->u.vlen.loc || f != dt->shared->u.vlen.f) {
@@ -281,6 +282,12 @@ H5T_vlen_set_loc(const H5T_t *dt, H5F_t *f, H5T_loc_t loc)
                 dt->shared->u.vlen.f = f;
                 break;
 
+            case H5T_LOC_BADLOC:
+                /* Allow undefined location. In H5Odtype.c, H5O_dtype_decode sets undefined
+                 * location for VL type and leaves it for the caller to decide.
+                 */
+                break;
+
             default:
                 HGOTO_ERROR(H5E_DATATYPE, H5E_BADRANGE, FAIL, "invalid VL datatype location")
         } /* end switch */ /*lint !e788 All appropriate cases are covered */
@@ -304,11 +311,6 @@ done:
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:   Neil Fortner
- *      Friday, August 22, 2008
- *      Changed function to be tolerant of an hvl_t that is not aligned
- *      properly in _vl.
- *
  *-------------------------------------------------------------------------
  */
 static ssize_t
@@ -320,7 +322,7 @@ H5T_vlen_seq_mem_getlen(const void *_vl)
     hvl_t vl;             /* User's hvl_t information */
 #endif
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_seq_mem_getlen)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters, return result */
 #ifdef H5_NO_ALIGNMENT_RESTRICTIONS
@@ -346,11 +348,6 @@ H5T_vlen_seq_mem_getlen(const void *_vl)
  * Programmer:	Quincey Koziol
  *		Saturday, June 12, 2004
  *
- * Modifications:   Neil Fortner
- *      Friday, August 22, 2008
- *      Changed function to be tolerant of an hvl_t that is not aligned
- *      properly in _vl.
- *
  *-------------------------------------------------------------------------
  */
 static void *
@@ -362,7 +359,7 @@ H5T_vlen_seq_mem_getptr(void *_vl)
     hvl_t vl;             /* User's hvl_t information */
 #endif
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_seq_mem_getptr)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters, return result */
 #ifdef H5_NO_ALIGNMENT_RESTRICTIONS
@@ -388,11 +385,6 @@ H5T_vlen_seq_mem_getptr(void *_vl)
  * Programmer:	Quincey Koziol
  *		Saturday, November 8, 2003
  *
- * Modifications:   Neil Fortner
- *      Friday, August 22, 2008
- *      Changed function to be tolerant of an hvl_t that is not aligned
- *      properly in _vl.
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -405,7 +397,7 @@ H5T_vlen_seq_mem_isnull(const H5F_t UNUSED *f, void *_vl)
     hvl_t vl;             /* User's hvl_t information */
 #endif
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_seq_mem_isnull)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters, return result */
 #ifdef H5_NO_ALIGNMENT_RESTRICTIONS
@@ -431,11 +423,6 @@ H5T_vlen_seq_mem_isnull(const H5F_t UNUSED *f, void *_vl)
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:   Neil Fortner
- *      Friday, August 22, 2008
- *      Changed function to be tolerant of an hvl_t that is not aligned
- *      properly in _vl.
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -448,7 +435,7 @@ H5T_vlen_seq_mem_read(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, void *_vl, void *bu
     hvl_t vl;             /* User's hvl_t information */
 #endif
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_seq_mem_read)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters, copy data */
     HDassert(buf);
@@ -478,8 +465,6 @@ H5T_vlen_seq_mem_read(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, void *_vl, void *bu
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -490,7 +475,7 @@ H5T_vlen_seq_mem_write(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const H5T_vlen_all
     size_t len;
     herr_t      ret_value=SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5T_vlen_seq_mem_write)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check parameters */
     HDassert(_vl);
@@ -537,8 +522,6 @@ done:
  * Programmer:	Quincey Koziol
  *		Saturday, November 8, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -547,7 +530,7 @@ H5T_vlen_seq_mem_setnull(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, void *_vl, void
 {
     hvl_t vl;                       /* Temporary hvl_t to use during operation */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_seq_mem_setnull)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters */
     HDassert(_vl);
@@ -573,11 +556,6 @@ H5T_vlen_seq_mem_setnull(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, void *_vl, void
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:   Neil Fortner
- *      Friday, August 22, 2008
- *      Changed function to be tolerant of a char * that is not aligned
- *      properly in _vl.
- *
  *-------------------------------------------------------------------------
  */
 static ssize_t
@@ -589,7 +567,7 @@ H5T_vlen_str_mem_getlen(const void *_vl)
     const char *s;      /* Pointer to the user's string information */
 #endif
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_str_mem_getlen)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters */
 #ifdef H5_NO_ALIGNMENT_RESTRICTIONS
@@ -613,12 +591,6 @@ H5T_vlen_str_mem_getlen(const void *_vl)
  * Programmer:	Quincey Koziol
  *		Saturday, June 12, 2004
  *
- * Modifications:   Neil Fortner
- *      Friday, August 22, 2008
- *      Changed function to be tolerant of a char * that is not aligned
- *      properly in _vl.
- *      Added assertion on _vl.
- *
  *-------------------------------------------------------------------------
  */
 static void *
@@ -630,7 +602,7 @@ H5T_vlen_str_mem_getptr(void *_vl)
     char *s;      /* Pointer to the user's string information */
 #endif
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_str_mem_getptr)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters */
 #ifdef H5_NO_ALIGNMENT_RESTRICTIONS
@@ -654,11 +626,6 @@ H5T_vlen_str_mem_getptr(void *_vl)
  * Programmer:	Quincey Koziol
  *		Saturday, November 8, 2003
  *
- * Modifications:   Neil Fortner
- *      Friday, August 22, 2008
- *      Changed function to be tolerant of a char * that is not aligned
- *      properly in _vl.
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -671,7 +638,7 @@ H5T_vlen_str_mem_isnull(const H5F_t UNUSED *f, void *_vl)
     char *s;      /* Pointer to the user's string information */
 #endif
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_str_mem_isnull)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
 #ifndef H5_NO_ALIGNMENT_RESTRICTIONS
     HDmemcpy(&s, _vl, sizeof(char *));
@@ -691,11 +658,6 @@ H5T_vlen_str_mem_isnull(const H5F_t UNUSED *f, void *_vl)
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:   Neil Fortner
- *      Friday, August 22, 2008
- *      Changed function to be tolerant of a char * that is not aligned
- *      properly in _vl.
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -708,7 +670,7 @@ H5T_vlen_str_mem_read(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, void *_vl, void *bu
     char *s;      /* Pointer to the user's string information */
 #endif
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_str_mem_read)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     if(len>0) {
         /* check parameters */
@@ -737,8 +699,6 @@ H5T_vlen_str_mem_read(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, void *_vl, void *bu
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -749,7 +709,7 @@ H5T_vlen_str_mem_write(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const H5T_vlen_all
     size_t len;                     /* Maximum length of the string to copy */
     herr_t      ret_value=SUCCEED;  /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5T_vlen_str_mem_write)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check parameters */
     HDassert(buf);
@@ -786,8 +746,6 @@ done:
  * Programmer:	Quincey Koziol
  *		Saturday, November 8, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -796,7 +754,7 @@ H5T_vlen_str_mem_setnull(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, void *_vl, void
 {
     char *t=NULL;                   /* Pointer to temporary buffer allocated */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_str_mem_setnull)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* Set pointer in user's buffer with memcpy, to avoid alignment issues */
     HDmemcpy(_vl,&t,sizeof(char *));
@@ -815,8 +773,6 @@ H5T_vlen_str_mem_setnull(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, void *_vl, void
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static ssize_t
@@ -825,7 +781,7 @@ H5T_vlen_disk_getlen(const void *_vl)
     const uint8_t *vl=(const uint8_t *)_vl; /* Pointer to the disk VL information */
     size_t	seq_len;        /* Sequence length */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_disk_getlen)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters */
     HDassert(vl);
@@ -846,15 +802,13 @@ H5T_vlen_disk_getlen(const void *_vl)
  * Programmer:	Quincey Koziol
  *		Saturday, June 12, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
 static void *
 H5T_vlen_disk_getptr(void UNUSED *vl)
 {
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_disk_getptr)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters */
     HDassert(vl);
@@ -873,8 +827,6 @@ H5T_vlen_disk_getptr(void UNUSED *vl)
  * Programmer:	Quincey Koziol
  *		Saturday, November 8, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static htri_t
@@ -883,7 +835,7 @@ H5T_vlen_disk_isnull(const H5F_t *f, void *_vl)
     uint8_t *vl=(uint8_t *)_vl; /* Pointer to the disk VL information */
     haddr_t addr;               /* Sequence's heap address */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5T_vlen_disk_isnull)
+    FUNC_ENTER_NOAPI_NOINIT_NOERR
 
     /* check parameters */
     HDassert(vl);
@@ -908,8 +860,6 @@ H5T_vlen_disk_isnull(const H5F_t *f, void *_vl)
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
@@ -920,7 +870,7 @@ H5T_vlen_disk_read(H5F_t *f, hid_t dxpl_id, void *_vl, void *buf, size_t UNUSED
     H5HG_t hobjid;
     herr_t      ret_value=SUCCEED;       /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5T_vlen_disk_read)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check parameters */
     HDassert(vl);
@@ -956,29 +906,24 @@ done:
  * Programmer:	Quincey Koziol
  *		Wednesday, June 2, 1999
  *
- * Modifications:
- *
- *		Raymond Lu
- *		Thursday, June 26, 2002
- *		Free heap objects storing old data.
- *
  *-------------------------------------------------------------------------
  */
 /* ARGSUSED */
 static herr_t
-H5T_vlen_disk_write(H5F_t *f, hid_t dxpl_id, const H5T_vlen_alloc_info_t UNUSED *vl_alloc_info, void *_vl, void *buf, void *_bg, size_t seq_len, size_t base_size)
+H5T_vlen_disk_write(H5F_t *f, hid_t dxpl_id, const H5T_vlen_alloc_info_t UNUSED *vl_alloc_info,
+    void *_vl, void *buf, void *_bg, size_t seq_len, size_t base_size)
 {
-    uint8_t *vl=(uint8_t *)_vl; /*Pointer to the user's hvl_t information*/
-    uint8_t *bg=(uint8_t *)_bg; /*Pointer to the old data hvl_t          */
+    uint8_t *vl = (uint8_t *)_vl; /*Pointer to the user's hvl_t information*/
+    uint8_t *bg = (uint8_t *)_bg; /*Pointer to the old data hvl_t          */
     H5HG_t hobjid;              /* New VL sequence's heap ID */
     size_t len;                 /* Size of new sequence on disk (in bytes) */
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5T_vlen_disk_write)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check parameters */
     HDassert(vl);
-    HDassert(seq_len==0 || buf);
+    HDassert(seq_len == 0 || buf);
     HDassert(f);
 
     /* Free heap object for old data.  */
@@ -993,9 +938,9 @@ H5T_vlen_disk_write(H5F_t *f, hid_t dxpl_id, const H5T_vlen_alloc_info_t UNUSED
         INT32DECODE(bg, bg_hobjid.idx);
 
         /* Free heap object for old data */
-        if(bg_hobjid.addr>0) {
+        if(bg_hobjid.addr > 0) {
             /* Free heap object */
-            if(H5HG_remove(f, dxpl_id, &bg_hobjid)<0)
+            if(H5HG_remove(f, dxpl_id, &bg_hobjid) < 0)
                 HGOTO_ERROR(H5E_DATATYPE, H5E_WRITEERROR, FAIL, "Unable to remove heap object")
         } /* end if */
     } /* end if */
@@ -1004,13 +949,13 @@ H5T_vlen_disk_write(H5F_t *f, hid_t dxpl_id, const H5T_vlen_alloc_info_t UNUSED
     UINT32ENCODE(vl, seq_len);
 
     /* Write the VL information to disk (allocates space also) */
-    len=(seq_len*base_size);
-    if(H5HG_insert(f,dxpl_id,len,buf,&hobjid)<0)
+    len = (seq_len*base_size);
+    if(H5HG_insert(f, dxpl_id, len, buf, &hobjid) < 0)
         HGOTO_ERROR(H5E_DATATYPE, H5E_WRITEERROR, FAIL, "Unable to write VL information")
 
     /* Encode the heap information */
-    H5F_addr_encode(f,&vl,hobjid.addr);
-    INT32ENCODE(vl,hobjid.idx);
+    H5F_addr_encode(f, &vl, hobjid.addr);
+    INT32ENCODE(vl, hobjid.idx);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1027,26 +972,24 @@ done:
  * Programmer:	Quincey Koziol
  *		Saturday, November 8, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
 H5T_vlen_disk_setnull(H5F_t *f, hid_t dxpl_id, void *_vl, void *_bg)
 {
-    uint8_t *vl=(uint8_t *)_vl; /*Pointer to the user's hvl_t information*/
-    uint8_t *bg=(uint8_t *)_bg; /*Pointer to the old data hvl_t          */
-    uint32_t seq_len=0;         /* Sequence length */
-    herr_t      ret_value=SUCCEED;       /* Return value */
+    uint8_t *vl = (uint8_t *)_vl; /*Pointer to the user's hvl_t information*/
+    uint8_t *bg = (uint8_t *)_bg; /*Pointer to the old data hvl_t          */
+    uint32_t seq_len = 0;         /* Sequence length */
+    herr_t ret_value = SUCCEED;   /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5T_vlen_disk_setnull)
+    FUNC_ENTER_NOAPI_NOINIT
 
     /* check parameters */
     HDassert(f);
     HDassert(vl);
 
     /* Free heap object for old data.  */
-    if(bg!=NULL) {
+    if(bg != NULL) {
         H5HG_t bg_hobjid;       /* "Background" VL info sequence's ID info */
 
         /* Skip the length of the sequence and heap object ID from background data. */
@@ -1057,9 +1000,9 @@ H5T_vlen_disk_setnull(H5F_t *f, hid_t dxpl_id, void *_vl, void *_bg)
         INT32DECODE(bg, bg_hobjid.idx);
 
         /* Free heap object for old data */
-        if(bg_hobjid.addr>0) {
+        if(bg_hobjid.addr > 0) {
             /* Free heap object */
-            if(H5HG_remove(f, dxpl_id, &bg_hobjid)<0)
+            if(H5HG_remove(f, dxpl_id, &bg_hobjid) < 0)
                 HGOTO_ERROR(H5E_DATATYPE, H5E_WRITEERROR, FAIL, "Unable to remove heap object")
          } /* end if */
     } /* end if */
@@ -1068,8 +1011,8 @@ H5T_vlen_disk_setnull(H5F_t *f, hid_t dxpl_id, void *_vl, void *_bg)
     UINT32ENCODE(vl, seq_len);
 
     /* Encode the "nil" heap pointer information */
-    H5F_addr_encode(f,&vl,(haddr_t)0);
-    INT32ENCODE(vl,0);
+    H5F_addr_encode(f, &vl, (haddr_t)0);
+    INT32ENCODE(vl, 0);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1100,10 +1043,10 @@ done:
 static herr_t
 H5T_vlen_reclaim_recurse(void *elem, const H5T_t *dt, H5MM_free_t free_func, void *free_info)
 {
-    unsigned i;     /* local index variable */
-    herr_t ret_value = SUCCEED;
+    unsigned u;                     /* Local index variable */
+    herr_t ret_value = SUCCEED;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5T_vlen_reclaim_recurse)
+    FUNC_ENTER_NOAPI_NOINIT
 
     HDassert(elem);
     HDassert(dt);
@@ -1116,9 +1059,9 @@ H5T_vlen_reclaim_recurse(void *elem, const H5T_t *dt, H5MM_free_t free_func, voi
                 void *off;     /* offset of field */
 
                 /* Calculate the offset member and recurse on it */
-                for(i=0; i<dt->shared->u.array.nelem; i++) {
-                    off=((uint8_t *)elem)+i*(dt->shared->parent->shared->size);
-                    if(H5T_vlen_reclaim_recurse(off,dt->shared->parent,free_func,free_info)<0)
+                for(u = 0; u < dt->shared->u.array.nelem; u++) {
+                    off = ((uint8_t *)elem) + u * (dt->shared->parent->shared->size);
+                    if(H5T_vlen_reclaim_recurse(off, dt->shared->parent, free_func, free_info) < 0)
                         HGOTO_ERROR(H5E_DATATYPE, H5E_CANTFREE, FAIL, "Unable to free array element")
                 } /* end for */
             } /* end if */
@@ -1126,14 +1069,14 @@ H5T_vlen_reclaim_recurse(void *elem, const H5T_t *dt, H5MM_free_t free_func, voi
 
         case H5T_COMPOUND:
             /* Check each field and recurse on VL, compound, enum or array ones */
-            for (i=0; i<dt->shared->u.compnd.nmembs; i++) {
+            for(u = 0; u < dt->shared->u.compnd.nmembs; u++) {
                 /* Recurse if it's VL, compound, enum or array */
-                if(H5T_IS_COMPLEX(dt->shared->u.compnd.memb[i].type->shared->type)) {
+                if(H5T_IS_COMPLEX(dt->shared->u.compnd.memb[u].type->shared->type)) {
                     void *off;     /* offset of field */
 
                     /* Calculate the offset member and recurse on it */
-                    off=((uint8_t *)elem)+dt->shared->u.compnd.memb[i].offset;
-                    if(H5T_vlen_reclaim_recurse(off,dt->shared->u.compnd.memb[i].type,free_func,free_info)<0)
+                    off = ((uint8_t *)elem) + dt->shared->u.compnd.memb[u].offset;
+                    if(H5T_vlen_reclaim_recurse(off, dt->shared->u.compnd.memb[u].type, free_func, free_info) < 0)
                         HGOTO_ERROR(H5E_DATATYPE, H5E_CANTFREE, FAIL, "Unable to free compound field")
                 } /* end if */
             } /* end for */
@@ -1141,8 +1084,8 @@ H5T_vlen_reclaim_recurse(void *elem, const H5T_t *dt, H5MM_free_t free_func, voi
 
         case H5T_VLEN:
             /* Recurse on the VL information if it's VL, compound, enum or array, then free VL sequence */
-            if(dt->shared->u.vlen.type==H5T_VLEN_SEQUENCE) {
-                hvl_t *vl=(hvl_t *)elem;    /* Temp. ptr to the vl info */
+            if(dt->shared->u.vlen.type == H5T_VLEN_SEQUENCE) {
+                hvl_t *vl = (hvl_t *)elem;    /* Temp. ptr to the vl info */
 
                 /* Check if there is anything actually in this sequence */
                 if(vl->len!=0) {
@@ -1151,24 +1094,24 @@ H5T_vlen_reclaim_recurse(void *elem, const H5T_t *dt, H5MM_free_t free_func, voi
                         void *off;     /* offset of field */
 
                         /* Calculate the offset of each array element and recurse on it */
-                        while(vl->len>0) {
-                            off=((uint8_t *)vl->p)+(vl->len-1)*dt->shared->parent->shared->size;
-                            if(H5T_vlen_reclaim_recurse(off,dt->shared->parent,free_func,free_info)<0)
+                        while(vl->len > 0) {
+                            off = ((uint8_t *)vl->p) + (vl->len - 1) * dt->shared->parent->shared->size;
+                            if(H5T_vlen_reclaim_recurse(off, dt->shared->parent, free_func, free_info) < 0)
                                 HGOTO_ERROR(H5E_DATATYPE, H5E_CANTFREE, FAIL, "Unable to free VL element")
                             vl->len--;
                         } /* end while */
                     } /* end if */
 
                     /* Free the VL sequence */
-                    if(free_func!=NULL)
-                        (*free_func)(vl->p,free_info);
+                    if(free_func != NULL)
+                        (*free_func)(vl->p, free_info);
                     else
                         H5MM_xfree(vl->p);
                 } /* end if */
-            } else if(dt->shared->u.vlen.type==H5T_VLEN_STRING) {
+            } else if(dt->shared->u.vlen.type == H5T_VLEN_STRING) {
                 /* Free the VL string */
-                if(free_func!=NULL)
-                    (*free_func)(*(char **)elem,free_info);
+                if(free_func != NULL)
+                    (*free_func)(*(char **)elem, free_info);
                 else
                     H5MM_xfree(*(char **)elem);
             } else {
@@ -1215,20 +1158,21 @@ H5T_vlen_reclaim(void *elem, hid_t type_id, unsigned UNUSED ndim, const hsize_t
 {
     H5T_vlen_alloc_info_t *vl_alloc_info = (H5T_vlen_alloc_info_t *)op_data; /* VL allocation info from iterator */
     H5T_t	*dt;
-    herr_t ret_value;
+    herr_t ret_value = SUCCEED;     /* Return value */
 
-    FUNC_ENTER_NOAPI(H5T_vlen_reclaim, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     HDassert(elem);
     HDassert(vl_alloc_info);
     HDassert(H5I_DATATYPE == H5I_get_type(type_id));
 
     /* Check args */
-    if (NULL==(dt=H5I_object_verify(type_id,H5I_DATATYPE)))
+    if(NULL == (dt = H5I_object_verify(type_id, H5I_DATATYPE)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a datatype")
 
     /* Pull the free function and free info pointer out of the op_data and call the recurse datatype free function */
-    ret_value=H5T_vlen_reclaim_recurse(elem,dt,vl_alloc_info->free_func,vl_alloc_info->free_info);
+    if(H5T_vlen_reclaim_recurse(elem, dt, vl_alloc_info->free_func, vl_alloc_info->free_info) < 0)
+        HGOTO_ERROR(H5E_DATATYPE, H5E_CANTFREE, FAIL, "can't reclaim vlen elements")
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1264,7 +1208,7 @@ H5T_vlen_get_alloc_info(hid_t dxpl_id, H5T_vlen_alloc_info_t **vl_alloc_info)
     H5P_genplist_t *plist;              /* DX property list */
     herr_t ret_value=SUCCEED;
 
-    FUNC_ENTER_NOAPI(H5T_vlen_get_alloc_info, FAIL)
+    FUNC_ENTER_NOAPI(FAIL)
 
     HDassert(H5I_GENPROP_LST == H5I_get_type(dxpl_id));
     HDassert(vl_alloc_info);
@@ -1278,13 +1222,13 @@ H5T_vlen_get_alloc_info(hid_t dxpl_id, H5T_vlen_alloc_info_t **vl_alloc_info)
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a dataset transfer property list")
 
         /* Get the allocation functions & information */
-        if (H5P_get(plist,H5D_XFER_VLEN_ALLOC_NAME,&(*vl_alloc_info)->alloc_func)<0)
+        if (H5P_get(plist,H5D_XFER_VLEN_ALLOC_NAME,&(*vl_alloc_info)->alloc_func) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "unable to get value")
-        if (H5P_get(plist,H5D_XFER_VLEN_ALLOC_INFO_NAME,&(*vl_alloc_info)->alloc_info)<0)
+        if (H5P_get(plist,H5D_XFER_VLEN_ALLOC_INFO_NAME,&(*vl_alloc_info)->alloc_info) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "unable to get value")
-        if (H5P_get(plist,H5D_XFER_VLEN_FREE_NAME,&(*vl_alloc_info)->free_func)<0)
+        if (H5P_get(plist,H5D_XFER_VLEN_FREE_NAME,&(*vl_alloc_info)->free_func) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "unable to get value")
-        if (H5P_get(plist,H5D_XFER_VLEN_FREE_INFO_NAME,&(*vl_alloc_info)->free_info)<0)
+        if (H5P_get(plist,H5D_XFER_VLEN_FREE_INFO_NAME,&(*vl_alloc_info)->free_info) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "unable to get value")
     } /* end else */
 
@@ -1292,3 +1236,44 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 }   /* end H5T_vlen_get_alloc_info() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5T_vlen_reclaim_elmt
+ *
+ * Purpose: Alternative method to reclaim any VL data for a buffer element.
+ * 
+ *          Use this function when the datatype is already available, but 
+ *          the allocation info is needed from the dxpl_id before jumping
+ *          into recursion.
+ *
+ * Return:	Non-negative on success/Negative on failure
+ *
+ * Programmer:	Mike McGreevy
+ *              May 11, 2010
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5T_vlen_reclaim_elmt(void *elem, H5T_t *dt, hid_t dxpl_id)
+{
+    H5T_vlen_alloc_info_t _vl_alloc_info;       /* VL allocation info buffer */
+    H5T_vlen_alloc_info_t *vl_alloc_info = &_vl_alloc_info;   /* VL allocation info */
+    herr_t ret_value = SUCCEED;                  /* return value */
+
+    HDassert(dt);
+    HDassert(elem);
+
+    FUNC_ENTER_NOAPI(FAIL)
+
+    /* Get VL allocation info */
+    if(H5T_vlen_get_alloc_info(dxpl_id, &vl_alloc_info) < 0)
+        HGOTO_ERROR(H5E_DATATYPE, H5E_CANTGET, FAIL, "unable to retrieve VL allocation info")
+
+    /* Recurse on buffer to free dynamic fields */
+    if(H5T_vlen_reclaim_recurse(elem, dt, vl_alloc_info->free_func, vl_alloc_info->free_info) < 0)
+        HGOTO_ERROR(H5E_DATATYPE, H5E_CANTFREE, FAIL, "can't reclaim vlen elements")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* H5T_vlen_reclaim_elmt */
+
