@@ -502,6 +502,13 @@ int main(int argc, char **argv)
             "test actual io mode proprerty",
             PARATESTFILE);
 
+    if((mpi_size < 2) && MAINPROCESS) {
+        printf("File Image Ops daisy chain test needs at least 2 processes.\n");
+        printf("File Image Ops daisy chain test will be skipped \n");
+    }
+    AddTest((mpi_size < 2)? "-fiodc" : "fiodc", file_image_daisy_chain_test, NULL,
+            "file image ops daisy chain", NULL);
+
     /* Display testing information */
     TestInfo(argv[0]);
 
@@ -512,6 +519,18 @@ int main(int argc, char **argv)
     /* Parse command line arguments */
     TestParseCmdLine(argc, argv);
 
+    if((mpi_size < 2)&& MAINPROCESS ) {
+	printf("Atomicity tests need at least 2 processes to participate\n");
+	printf("8 is more recommended.. Atomicity tests will be skipped \n");
+    }
+    else if (facc_type != FACC_MPIO && MAINPROCESS) {
+	printf("Atomicity tests will not work with a non MPIO VFD\n");        
+    }
+    else if(mpi_size >= 2 && facc_type == FACC_MPIO){
+        AddTest("atomicity", dataset_atomicity, NULL,
+                "dataset atomic updates", PARATESTFILE);
+    }
+
     if (facc_type == FACC_MPIPOSIX && MAINPROCESS){
   printf("===================================\n"
          "   Using MPIPOSIX driver\n"
