@@ -111,6 +111,8 @@ $SRC_H5DIFF_TESTFILES/h5diff_exclude1-1.h5
 $SRC_H5DIFF_TESTFILES/h5diff_exclude1-2.h5
 $SRC_H5DIFF_TESTFILES/h5diff_exclude2-1.h5
 $SRC_H5DIFF_TESTFILES/h5diff_exclude2-2.h5
+$SRC_H5DIFF_TESTFILES/h5diff_exclude3-1.h5
+$SRC_H5DIFF_TESTFILES/h5diff_exclude3-2.h5
 $SRC_H5DIFF_TESTFILES/h5diff_comp_vl_strs.h5
 $SRC_H5DIFF_TESTFILES/compounds_array_vlen1.h5
 $SRC_H5DIFF_TESTFILES/compounds_array_vlen2.h5
@@ -154,6 +156,8 @@ $SRC_H5DIFF_TESTFILES/h5diff_208.txt
 $SRC_H5DIFF_TESTFILES/h5diff_220.txt
 $SRC_H5DIFF_TESTFILES/h5diff_221.txt
 $SRC_H5DIFF_TESTFILES/h5diff_222.txt
+$SRC_H5DIFF_TESTFILES/h5diff_223.txt
+$SRC_H5DIFF_TESTFILES/h5diff_224.txt
 $SRC_H5DIFF_TESTFILES/h5diff_21.txt
 $SRC_H5DIFF_TESTFILES/h5diff_22.txt
 $SRC_H5DIFF_TESTFILES/h5diff_23.txt
@@ -200,11 +204,19 @@ $SRC_H5DIFF_TESTFILES/h5diff_456.txt
 $SRC_H5DIFF_TESTFILES/h5diff_457.txt
 $SRC_H5DIFF_TESTFILES/h5diff_458.txt
 $SRC_H5DIFF_TESTFILES/h5diff_459.txt
+$SRC_H5DIFF_TESTFILES/h5diff_465.txt
+$SRC_H5DIFF_TESTFILES/h5diff_466.txt
+$SRC_H5DIFF_TESTFILES/h5diff_467.txt
+$SRC_H5DIFF_TESTFILES/h5diff_468.txt
+$SRC_H5DIFF_TESTFILES/h5diff_469.txt
 $SRC_H5DIFF_TESTFILES/h5diff_480.txt
 $SRC_H5DIFF_TESTFILES/h5diff_481.txt
 $SRC_H5DIFF_TESTFILES/h5diff_482.txt
 $SRC_H5DIFF_TESTFILES/h5diff_483.txt
 $SRC_H5DIFF_TESTFILES/h5diff_484.txt
+$SRC_H5DIFF_TESTFILES/h5diff_485.txt
+$SRC_H5DIFF_TESTFILES/h5diff_486.txt
+$SRC_H5DIFF_TESTFILES/h5diff_487.txt
 $SRC_H5DIFF_TESTFILES/h5diff_50.txt
 $SRC_H5DIFF_TESTFILES/h5diff_51.txt
 $SRC_H5DIFF_TESTFILES/h5diff_52.txt
@@ -810,13 +822,20 @@ TOOLTEST h5diff_221.txt -c non_comparables1.h5 non_comparables2.h5 /g2
 
 # entire file
 # All the comparables should display differences.
-if test -n "$pmode" -a "$mydomainname" = hdfgroup.uiuc.edu; then
+if test -n "$pmode"; then
     # parallel mode: 
-    # skip due to ph5diff hangs on koala (linux64-LE) randomly.    
+    # skip due to ph5diff hangs on koala (linux64-LE) and ember intermittently.
+    # (HDFFV-8003 - TBD)
     SKIP -c non_comparables1.h5 non_comparables2.h5
 else
     TOOLTEST h5diff_222.txt -c non_comparables1.h5 non_comparables2.h5
 fi    
+
+# non-comparable test for common objects (same name) with different object types
+# (HDFFV-7644)
+TOOLTEST h5diff_223.txt -c non_comparables1.h5 non_comparables2.h5 /diffobjtypes
+# swap files
+TOOLTEST h5diff_224.txt -c non_comparables2.h5 non_comparables1.h5 /diffobjtypes
     
 # ##############################################################################
 # # Links compare without --follow-symlinks nor --no-dangling-links
@@ -939,6 +958,18 @@ TOOLTEST h5diff_458.txt  --follow-symlinks -v --no-dangling-links  h5diff_extlin
 # dangling link found for ext links (obj to obj). Both dangle links
 TOOLTEST h5diff_459.txt  --follow-symlinks -v --no-dangling-links  h5diff_extlink_src.h5 h5diff_extlink_src.h5 /ext_link_noexist1 /ext_link_noexist2
 
+# dangling link --follow-symlinks (obj vs obj)
+# (HDFFV-7836)
+TOOLTEST h5diff_465.txt --follow-symlinks h5diff_danglelinks1.h5 h5diff_danglelinks2.h5 /soft_link1
+# (HDFFV-7835)
+# soft dangling vs. soft dangling
+TOOLTEST h5diff_466.txt -v --follow-symlinks h5diff_danglelinks1.h5 h5diff_danglelinks2.h5 /soft_link1
+# soft link  vs. soft dangling
+TOOLTEST h5diff_467.txt -v --follow-symlinks h5diff_danglelinks1.h5 h5diff_danglelinks2.h5 /soft_link2
+# ext dangling vs. ext dangling
+TOOLTEST h5diff_468.txt -v --follow-symlinks h5diff_danglelinks1.h5 h5diff_danglelinks2.h5 /ext_link4 
+# ext link vs. ext dangling
+TOOLTEST h5diff_469.txt -v --follow-symlinks h5diff_danglelinks1.h5 h5diff_danglelinks2.h5 /ext_link2
 
 # ##############################################################################
 # # test for group diff recursivly
@@ -1010,6 +1041,15 @@ TOOLTEST h5diff_483.txt -v --exclude-path "/group1" h5diff_exclude2-1.h5 h5diff_
 # Exclude from group compare
 TOOLTEST h5diff_484.txt -v --exclude-path "/dset3" h5diff_exclude1-1.h5 h5diff_exclude1-2.h5 /group1
 
+#
+# Only one file contains unique objs. Common objs are same.
+# (HDFFV-7837)
+#
+TOOLTEST h5diff_485.txt -v --exclude-path "/group1" h5diff_exclude3-1.h5 h5diff_exclude3-2.h5
+TOOLTEST h5diff_486.txt -v --exclude-path "/group1" h5diff_exclude3-2.h5 h5diff_exclude3-1.h5
+TOOLTEST h5diff_487.txt -v --exclude-path "/group1/dset" h5diff_exclude3-1.h5 h5diff_exclude3-2.h5
+
+
 # ##############################################################################
 # # diff various multiple vlen and fixed strings in a compound type dataset
 # ##############################################################################
