@@ -15,6 +15,7 @@
 
 #include "hdf5.h"
 #include "H5private.h"
+#include "h5tools.h"
 #include "h5tools_utils.h"
 
 /* Name of tool */
@@ -139,7 +140,7 @@ leave(int ret)
 
     h5tools_close();
 
-    exit(ret);
+    HDexit(ret);
 }
 
 /*-------------------------------------------------------------------------
@@ -237,6 +238,9 @@ main (int argc, const char *argv[])
     H5Eget_auto2(H5E_DEFAULT, &func, &edata);
     H5Eset_auto2(H5E_DEFAULT, NULL, NULL);
 
+    /* Initialize h5tools lib */
+    h5tools_init();
+
     parse_command_line (argc, argv);
 
     if (ub_file == NULL) {
