@@ -51,7 +51,7 @@ typedef struct ohdr_info_t {
 /* Info to pass to the iteration functions */
 typedef struct iter_t {
     hid_t fid;                          /* File ID */
-
+    hsize_t filesize;      /* Size of the file */
     unsigned long uniq_groups;          /* Number of unique groups */
     unsigned long uniq_dsets;           /* Number of unique datasets */
     unsigned long uniq_dtypes;          /* Number of unique named datatypes */
@@ -90,8 +90,8 @@ typedef struct iter_t {
     hsize_t attrs_heap_storage_size;    /* fractal heap size for attributes (1.8) */
     hsize_t SM_hdr_storage_size;        /* header size for SOHM table (1.8) */
     hsize_t SM_index_storage_size;      /* index (btree & list) size for SOHM table (1.8) */
-    hsize_t SM_heap_storage_size;  /* fractal heap size for SOHM table (1.8) */
-    hsize_t super_ext_size;       /* superblock extension size */
+    hsize_t SM_heap_storage_size;       /* fractal heap size for SOHM table (1.8) */
+    hsize_t super_ext_size;             /* superblock extension size */
     hsize_t ublk_size;                  /* user block size (if exists) */
     hsize_t datasets_index_storage_size;/* meta size for chunked dataset's indexing type */
     hsize_t datasets_heap_storage_size; /* heap size for dataset with external storage */
@@ -101,15 +101,20 @@ typedef struct iter_t {
 
 
 static int        display_all = TRUE;
-static int        display_file = FALSE;
-static int        display_file_metadata = FALSE;
-static int        display_group = FALSE;
-static int        display_group_metadata = FALSE;
-static int        display_dset = FALSE;
-static int        display_dset_metadata = FALSE;
-static int        display_dset_dtype_meta = FALSE;
-static int        display_object = FALSE;
-static int        display_attr = FALSE;
+
+/* Enable the printing of selected statistics */
+static int        display_file = FALSE;     /* display file information */
+static int        display_group = FALSE;    /* display groups information */
+static int        display_dset = FALSE;     /* display datasets information */
+static int        display_dset_dtype_meta = FALSE;  /* display datasets' datatype information */
+static int        display_attr = FALSE;     /* display attributes information */
+static int        display_summary = FALSE;  /* display summary of file space information */
+
+static int        display_file_metadata = FALSE;    /* display file space info for file's metadata */
+static int        display_group_metadata = FALSE;   /* display file space info for groups' metadata */
+static int        display_dset_metadata = FALSE;    /* display file space info for datasets' metadata */
+
+static int        display_object = FALSE;  /* not implemented yet */
 
 /* a structure for handling the order command-line parameters come in */
 struct handler_t {
@@ -117,13 +122,12 @@ struct handler_t {
 };
 
 
-static const char *s_opts ="ADdFfhGgTO:V";
+static const char *s_opts ="ADdFfhGgSTO:V";
 /* e.g. "filemetadata" has to precedue "file"; "groupmetadata" has to precede "group" etc. */
 static struct long_options l_opts[] = {
     {"help", no_arg, 'h'},
     {"hel", no_arg, 'h'},
     {"he", no_arg, 'h'},
-
     {"filemetadata", no_arg, 'F'},
     {"filemetadat", no_arg, 'F'},
     {"filemetada", no_arg, 'F'},
@@ -135,7 +139,6 @@ static struct long_options l_opts[] = {
     {"file", no_arg, 'f'},
     {"fil", no_arg, 'f'},
     {"fi", no_arg, 'f'},
-
     {"groupmetadata", no_arg, 'G'},
     {"groupmetadat", no_arg, 'G'},
     {"groupmetada", no_arg, 'G'},
@@ -148,7 +151,6 @@ static struct long_options l_opts[] = {
     {"grou", no_arg, 'g'},
     {"gro", no_arg, 'g'},
     {"gr", no_arg, 'g'},
-
     {"dsetmetadata", no_arg, 'D'},
     {"dsetmetadat", no_arg, 'D'},
     {"dsetmetada", no_arg, 'D'},
@@ -191,6 +193,12 @@ static struct long_options l_opts[] = {
     { "attr", no_arg, 'A' },
     { "att", no_arg, 'A' },
     { "at", no_arg, 'A' },
+    { "summary", no_arg, 'S' },
+    { "summar", no_arg, 'S' },
+    { "summa", no_arg, 'S' },
+    { "summ", no_arg, 'S' },
+    { "sum", no_arg, 'S' },
+    { "su", no_arg, 'S' },
     { NULL, 0, '\0' }
 };
 
@@ -198,28 +206,28 @@ static void
 leave(int ret)
 {
    h5tools_close();
-   exit(ret);
-} /* leave() */
+   HDexit(ret);
+}
 
 
 static void usage(const char *prog)
 {
-     fflush(stdout);
-     fprintf(stdout, "Usage: %s [OPTIONS] file\n", prog);
-     fprintf(stdout, "\n");
-     fprintf(stdout, "      OPTIONS\n");
-     fprintf(stdout, "     -h, --help            Print a usage message and exit\n");
-     fprintf(stdout, "     -V, --version         Print version number and exit\n");
-     fprintf(stdout, "     -f, --file            Print file information\n");
-     fprintf(stdout, "     -F, --filemetadata    Print file space information for file's metadata\n");
-     fprintf(stdout, "     -g, --group           Print group information\n");
-     fprintf(stdout, "     -G, --groupmetadata   Print file space information for groups' metadata\n");
-     fprintf(stdout, "     -d, --dset            Print dataset information\n");
-     fprintf(stdout, "     -D, --dsetmetadata    Print file space information for datasets' metadata\n");
-     fprintf(stdout, "     -T, --dtypemetadata   Print datasets' datatype metadata\n");
-     fprintf(stdout, "     -A, --attribute       Print attribute information\n");
-     fprintf(stdout, "\n");
-} /* usage() */
+     HDfflush(stdout);
+     HDfprintf(stdout, "Usage: %s [OPTIONS] file\n", prog);
+     HDfprintf(stdout, "\n");
+     HDfprintf(stdout, "      OPTIONS\n");
+     HDfprintf(stdout, "     -h, --help            Print a usage message and exit\n");
+     HDfprintf(stdout, "     -V, --version         Print version number and exit\n");
+     HDfprintf(stdout, "     -f, --file            Print file information\n");
+     HDfprintf(stdout, "     -F, --filemetadata    Print file space information for file's metadata\n");
+     HDfprintf(stdout, "     -g, --group           Print group information\n");
+     HDfprintf(stdout, "     -G, --groupmetadata   Print file space information for groups' metadata\n");
+     HDfprintf(stdout, "     -d, --dset            Print dataset information\n");
+     HDfprintf(stdout, "     -D, --dsetmetadata    Print file space information for datasets' metadata\n");
+     HDfprintf(stdout, "     -T, --dtypemetadata   Print datasets' datatype information\n");
+     HDfprintf(stdout, "     -A, --attribute       Print attribute information\n");
+     HDfprintf(stdout, "     -S, --summary         Print summary of file space information\n");
+}
 
 
 /*-------------------------------------------------------------------------
@@ -283,8 +291,8 @@ attribute_stats(iter_t *iter, const H5O_info_t *oi)
     /* Add attribute count to proper bin */
     bin = ceil_log10((unsigned long)oi->num_attrs);
     if((bin + 1) > iter->attr_nbins) {
-  iter->attr_bins = (unsigned long *)realloc(iter->attr_bins, (bin + 1) * sizeof(unsigned long));
-        assert(iter->attr_bins);
+  iter->attr_bins = (unsigned long *)HDrealloc(iter->attr_bins, (bin + 1) * sizeof(unsigned long));
+        HDassert(iter->attr_bins);
 
   /* Initialize counts for intermediate bins */
         while(iter->attr_nbins < bin)
@@ -343,7 +351,7 @@ group_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
 
     /* Get group information */
     ret = H5Gget_info_by_name(iter->fid, name, &ginfo, H5P_DEFAULT);
-    assert(ret >= 0);
+    HDassert(ret >= 0);
 
     /* Update link stats */
     if(ginfo.nlinks < SIZE_SMALL_GROUPS)
@@ -355,8 +363,8 @@ group_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
     bin = ceil_log10((unsigned long)ginfo.nlinks);
     if((bin + 1) > iter->group_nbins) {
         /* Allocate more storage for info about dataset's datatype */
-        iter->group_bins = (unsigned long *)realloc(iter->group_bins, (bin + 1) * sizeof(unsigned long));
-        assert(iter->group_bins);
+        iter->group_bins = (unsigned long *)HDrealloc(iter->group_bins, (bin + 1) * sizeof(unsigned long));
+        HDassert(iter->group_bins);
 
   /* Initialize counts for intermediate bins */
         while(iter->group_nbins < bin)
@@ -375,7 +383,7 @@ group_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
 
     /* Update attribute metadata info */
     ret = attribute_stats(iter, oi);
-    assert(ret >= 0);
+    HDassert(ret >= 0);
 
     return 0;
 } /* end group_stats() */
@@ -393,10 +401,6 @@ group_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
  * Programmer:    Quincey Koziol
  *                Tuesday, August 16, 2005
  *
- * Modifications:
- *      2/2010; Vailin Choi
- *      Handle external data (brought over from trunk h5stat)
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
@@ -427,7 +431,7 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
     iter->dset_ohdr_info.free_size += oi->hdr.space.free;
 
     did = H5Dopen2(iter->fid, name, H5P_DEFAULT);
-    assert(did > 0);
+    HDassert(did > 0);
 
     /* Update dataset metadata info */
     iter->datasets_index_storage_size += oi->meta_size.obj.index_size;
@@ -435,17 +439,17 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
 
     /* Update attribute metadata info */
     ret = attribute_stats(iter, oi);
-    assert(ret >= 0);
+    HDassert(ret >= 0);
 
     /* Get storage info */
     storage = H5Dget_storage_size(did);
 
     /* Gather layout statistics */
     dcpl = H5Dget_create_plist(did);
-    assert(dcpl > 0);
+    HDassert(dcpl > 0);
 
     lout = H5Pget_layout(dcpl);
-    assert(lout >= 0);
+    HDassert(lout >= 0);
 
     /* Object header's total size for H5D_COMPACT layout includes raw data size */
     /* "storage" also includes H5D_COMPACT raw data size */
@@ -468,10 +472,10 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
 
     /* Gather dataspace statistics */
     sid = H5Dget_space(did);
-    assert(sid > 0);
+    HDassert(sid > 0);
 
     ndims = H5Sget_simple_extent_dims(sid, dims, NULL);
-    assert(ndims >= 0);
+    HDassert(ndims >= 0);
 
     /* Check for larger rank of dataset */
     if((unsigned)ndims > iter->max_dset_rank)
@@ -490,8 +494,8 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
        bin = ceil_log10((unsigned long)dims[0]);
        if((bin + 1) > iter->dset_dim_nbins) {
           /* Allocate more storage for info about dataset's datatype */
-          iter->dset_dim_bins = (unsigned long *)realloc(iter->dset_dim_bins, (bin + 1) * sizeof(unsigned long));
-          assert(iter->dset_dim_bins);
+          iter->dset_dim_bins = (unsigned long *)HDrealloc(iter->dset_dim_bins, (bin + 1) * sizeof(unsigned long));
+          HDassert(iter->dset_dim_bins);
 
           /* Initialize counts for intermediate bins */
           while(iter->dset_dim_nbins < bin)
@@ -506,11 +510,11 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
     } /* end if */
 
     ret = H5Sclose(sid);
-    assert(ret >= 0);
+    HDassert(ret >= 0);
 
     /* Gather datatype statistics */
     tid = H5Dget_type(did);
-    assert(tid > 0);
+    HDassert(tid > 0);
 
     type_found = FALSE;
     for(u = 0; u < iter->dset_ntypes; u++)
@@ -527,12 +531,12 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
         iter->dset_ntypes++;
 
         /* Allocate more storage for info about dataset's datatype */
-        iter->dset_type_info = (dtype_info_t *)realloc(iter->dset_type_info, iter->dset_ntypes * sizeof(dtype_info_t));
-        assert(iter->dset_type_info);
+        iter->dset_type_info = (dtype_info_t *)HDrealloc(iter->dset_type_info, iter->dset_ntypes * sizeof(dtype_info_t));
+        HDassert(iter->dset_type_info);
 
         /* Initialize information about datatype */
         iter->dset_type_info[curr_ntype].tid = H5Tcopy(tid);
-        assert(iter->dset_type_info[curr_ntype].tid > 0);
+        HDassert(iter->dset_type_info[curr_ntype].tid > 0);
         iter->dset_type_info[curr_ntype].count = 1;
         iter->dset_type_info[curr_ntype].named = 0;
 
@@ -545,7 +549,7 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
         (iter->dset_type_info[u].named)++;
 
     ret = H5Tclose(tid);
-    assert(ret >= 0);
+    HDassert(ret >= 0);
 
     /* Track different filters */
     if((nfltr = H5Pget_nfilters(dcpl)) >= 0) {
@@ -559,15 +563,14 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
                 else
                     iter->dset_comptype[H5_NFILTERS_IMPL - 1]++; /*other filters*/
             } /* end if */
-
         } /* end for */
     } /* endif nfltr */
 
      ret = H5Pclose(dcpl);
-     assert(ret >= 0);
+     HDassert(ret >= 0);
 
      ret = H5Dclose(did);
-     assert(ret >= 0);
+     HDassert(ret >= 0);
 
      return 0;
 }  /* end dataset_stats() */
@@ -583,16 +586,12 @@ dataset_stats(iter_t *iter, const char *name, const H5O_info_t *oi)
  *
  * Programmer:    Vailin Choi; July 7th, 2009
  *
- * Modifications:
- *      2/2010; Vailin Choi
- *      Gather attribute info for named datatype
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
 datatype_stats(iter_t *iter, const H5O_info_t *oi)
 {
-     herr_t              ret;
+    herr_t ret;
 
     /* Gather statistics about this type of object */
     iter->uniq_dtypes++;
@@ -603,7 +602,7 @@ datatype_stats(iter_t *iter, const H5O_info_t *oi)
 
     /* Update attribute metadata info */
     ret = attribute_stats(iter, oi);
-    assert(ret >= 0);
+    HDassert(ret >= 0);
 
      return 0;
 }  /* end datatype_stats() */
@@ -706,10 +705,6 @@ lnk_stats(const char UNUSED *path, const H5L_info_t *li, void *_iter)
  * Programmer: Elena Pourmal
  *             Saturday, August 12, 2006
  *
- * Modifications:
- *      2/2010; Vailin Choi
- *      Handle -G and -T options
- *
  *-------------------------------------------------------------------------
  */
 static struct handler_t *
@@ -719,7 +714,7 @@ parse_command_line(int argc, const char *argv[])
     struct handler_t   *hand = NULL;
 
     /* Allocate space to hold the command line info */
-    if((hand = (struct handler_t *)calloc((size_t)argc, sizeof(struct handler_t)))==NULL) {
+    if((hand = (struct handler_t *)HDcalloc((size_t)argc, sizeof(struct handler_t)))==NULL) {
         error_msg("unable to parse command line arguments \n");
         goto error;
     }
@@ -729,34 +724,34 @@ parse_command_line(int argc, const char *argv[])
         switch ((char)opt) {
             case 'h':
                 usage(h5tools_getprogname());
-            h5tools_setstatus(EXIT_SUCCESS);
-            if (hand) {
-                for (i = 0; i < argc; i++)
-                    if(hand[i].obj) {
-                        free(hand[i].obj);
-                        hand[i].obj=NULL;
-                    }
+                h5tools_setstatus(EXIT_SUCCESS);
+                if (hand) {
+                    for (i = 0; i < argc; i++)
+                        if(hand[i].obj) {
+                            free(hand[i].obj);
+                            hand[i].obj=NULL;
+                        }
 
-                free(hand);
-                hand = NULL;
-            }
-            goto done;
-            break;
+                    free(hand);
+                    hand = NULL;
+                }
+                goto done;
+                break;
 
             case 'V':
                 print_version(h5tools_getprogname());
-            h5tools_setstatus(EXIT_SUCCESS);
-            if (hand) {
-                for (i = 0; i < argc; i++)
-                    if(hand[i].obj) {
-                        free(hand[i].obj);
-                        hand[i].obj=NULL;
-                    }
+                h5tools_setstatus(EXIT_SUCCESS);
+                if (hand) {
+                    for (i = 0; i < argc; i++)
+                        if(hand[i].obj) {
+                            free(hand[i].obj);
+                            hand[i].obj=NULL;
+                        }
 
-                free(hand);
-                hand = NULL;
-            }
-            goto done;
+                    free(hand);
+                    hand = NULL;
+                }
+                goto done;
                 break;
 
             case 'F':
@@ -771,7 +766,7 @@ parse_command_line(int argc, const char *argv[])
 
             case 'G':
                 display_all = FALSE;
-          display_group_metadata = TRUE;
+                display_group_metadata = TRUE;
                 break;
 
             case 'g':
@@ -781,7 +776,7 @@ parse_command_line(int argc, const char *argv[])
 
             case 'D':
                 display_all = FALSE;
-    display_dset_metadata = TRUE;
+                display_dset_metadata = TRUE;
                 break;
 
             case 'd':
@@ -791,7 +786,7 @@ parse_command_line(int argc, const char *argv[])
 
             case 'T':
                 display_all = FALSE;
-    display_dset_dtype_meta = TRUE;
+                display_dset_dtype_meta = TRUE;
                 break;
 
             case 'A':
@@ -799,7 +794,13 @@ parse_command_line(int argc, const char *argv[])
                 display_attr = TRUE;
                 break;
 
+            case 'S':
+                display_all = FALSE;
+                display_summary = TRUE;
+                break;
+
             case 'O':
+                display_all = FALSE;
                 display_object = TRUE;
                 for(i = 0; i < argc; i++)
                     if(!hand[i].obj) {
@@ -810,8 +811,8 @@ parse_command_line(int argc, const char *argv[])
 
             default:
                 usage(h5tools_getprogname());
-            h5tools_setstatus(EXIT_FAILURE);
-            goto error;
+                h5tools_setstatus(EXIT_FAILURE);
+                goto error;
         } /* end switch */
     } /* end while */
 
@@ -940,7 +941,7 @@ print_file_info(const iter_t *iter)
 /*-------------------------------------------------------------------------
  * Function: print_file_metadata
  *
- * Purpose: Prints metadata information about file
+ * Purpose: Prints file space information for file's metadata
  *
  * Return: Success: 0
  *
@@ -949,11 +950,6 @@ print_file_info(const iter_t *iter)
  * Programmer: Elena Pourmal
  *             Saturday, August 12, 2006
  *
- * Modifications:
- *      2/2010; Vailin Choi
- *      Brought over from trunk h5stat
- *      (Note: couldn't get superblock size because of H5F_info_t in H5public.h)
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
@@ -1073,7 +1069,7 @@ print_group_metadata(const iter_t *iter)
     printf("File space information for groups' metadata (in bytes):\n");
 
     HDfprintf(stdout, "\tObject headers (total/unused): %Hu/%Hu\n",
-                iter->group_ohdr_info.total_size, iter->group_ohdr_info.free_size);
+            iter->group_ohdr_info.total_size, iter->group_ohdr_info.free_size);
 
     HDfprintf(stdout, "\tB-tree/List: %Hu\n", iter->groups_btree_storage_size);
     HDfprintf(stdout, "\tHeap: %Hu\n", iter->groups_heap_storage_size);
@@ -1094,10 +1090,6 @@ print_group_metadata(const iter_t *iter)
  * Programmer: Elena Pourmal
  *             Saturday, August 12, 2006
  *
- * Modifications:
- *      2/2010; Vailin Choi
- *      Moved coding for datatype to another routine.
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
@@ -1150,7 +1142,7 @@ print_dataset_info(const iter_t *iter)
 
         printf("Dataset storage information:\n");
         HDfprintf(stdout, "\tTotal raw data size: %Hu\n", iter->dset_storage_size);
-  HDfprintf(stdout, "\tTotal external raw data size: %Hu\n", iter->dset_external_storage_size);
+        HDfprintf(stdout, "\tTotal external raw data size: %Hu\n", iter->dset_external_storage_size);
 
         printf("Dataset layout information:\n");
         for(u = 0; u < H5D_NLAYOUTS; u++)
@@ -1168,7 +1160,6 @@ print_dataset_info(const iter_t *iter)
         printf("\t\tNBIT filter: %lu\n", iter->dset_comptype[H5Z_FILTER_NBIT]);
         printf("\t\tSCALEOFFSET filter: %lu\n", iter->dset_comptype[H5Z_FILTER_SCALEOFFSET]);
         printf("\t\tUSER-DEFINED filter: %lu\n", iter->dset_comptype[H5_NFILTERS_IMPL-1]);
-
     } /* end if */
 
     return 0;
@@ -1185,7 +1176,6 @@ print_dataset_info(const iter_t *iter)
  * Failure: Never fails
  *
  * Programmer:  Vailin Choi; October 2009
- *      Brought over from trunk h5stat.
  *
  *-------------------------------------------------------------------------
  */
@@ -1195,10 +1185,10 @@ print_dset_metadata(const iter_t *iter)
     printf("File space information for datasets' metadata (in bytes):\n");
 
     HDfprintf(stdout, "\tObject headers (total/unused): %Hu/%Hu\n",
-                iter->dset_ohdr_info.total_size, iter->dset_ohdr_info.free_size);
+            iter->dset_ohdr_info.total_size, iter->dset_ohdr_info.free_size);
 
     HDfprintf(stdout, "\tIndex for Chunked datasets: %Hu\n",
-                iter->datasets_index_storage_size);
+            iter->datasets_index_storage_size);
     HDfprintf(stdout, "\tHeap: %Hu\n", iter->datasets_heap_storage_size);
 
     return 0;
@@ -1214,8 +1204,7 @@ print_dset_metadata(const iter_t *iter)
  *
  * Failure: Never fails
  *
- * Modifications:
- *      Moved from print_dataset_info()
+ * Programmer: Vailin Choi; October 2009
  *
  *-------------------------------------------------------------------------
  */
@@ -1234,14 +1223,14 @@ print_dset_dtype_meta(const iter_t *iter)
             H5Tencode(iter->dset_type_info[u].tid, NULL, &dtype_size);
             printf("\tDataset datatype #%u:\n", u);
             printf("\t\tCount (total/named) = (%lu/%lu)\n",
-                iter->dset_type_info[u].count, iter->dset_type_info[u].named);
+                    iter->dset_type_info[u].count, iter->dset_type_info[u].named);
             printf("\t\tSize (desc./elmt) = (%lu/%lu)\n", (unsigned long)dtype_size,
-                (unsigned long)H5Tget_size(iter->dset_type_info[u].tid));
+                    (unsigned long)H5Tget_size(iter->dset_type_info[u].tid));
             H5Tclose(iter->dset_type_info[u].tid);
             total += iter->dset_type_info[u].count;
         } /* end for */
         printf("\tTotal dataset datatype count: %lu\n", total);
-    }
+    } /* end if */
 
     return 0;
 } /* print_dset_dtype_meta() */
@@ -1297,6 +1286,63 @@ print_attr_info(const iter_t *iter)
     return 0;
 } /* print_attr_info() */
 
+
+/*-------------------------------------------------------------------------
+ * Function: print_storage_summary
+ *
+ * Purpose: Prints file space information for the file
+ *
+ * Return: Success: 0
+ *
+ * Failure: Never fails
+ *
+ * Programmer: Vailin Choi; August 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+print_storage_summary(const iter_t *iter)
+{
+    hsize_t total_meta = 0;
+    hsize_t unaccount = 0;
+    float   percent = 0.0;
+
+    printf("Summary of file space information:\n");
+    total_meta =
+            iter->super_ext_size + iter->ublk_size +
+            iter->group_ohdr_info.total_size +
+            iter->dset_ohdr_info.total_size +
+            iter->dtype_ohdr_info.total_size +
+            iter->groups_btree_storage_size +
+            iter->groups_heap_storage_size +
+            iter->attrs_btree_storage_size +
+            iter->attrs_heap_storage_size +
+            iter->datasets_index_storage_size +
+            iter->datasets_heap_storage_size +
+            iter->SM_hdr_storage_size +
+            iter->SM_index_storage_size +
+            iter->SM_heap_storage_size;
+
+    HDfprintf(stdout, "  File metadata: %Hu bytes\n", total_meta);
+    HDfprintf(stdout, "  Raw data: %Hu bytes\n", iter->dset_storage_size);
+
+    if(iter->filesize < (total_meta + iter->dset_storage_size)) {
+        unaccount = (total_meta + iter->dset_storage_size ) - iter->filesize;
+        HDfprintf(stdout, "  ??? File has %Hu more bytes accounted for than its size! ???\n", unaccount);
+    }
+    else {
+        unaccount = iter->filesize - (total_meta + iter->dset_storage_size);
+        HDfprintf(stdout, "  Unaccounted space: %Hu bytes\n", unaccount);
+    }
+
+    HDfprintf(stdout, "Total space: %Hu bytes\n", total_meta + iter->dset_storage_size + unaccount);
+
+    if(iter->nexternal)
+        HDfprintf(stdout, "External raw data: %Hu bytes\n", iter->dset_external_storage_size);
+
+    return 0;
+} /* print_storage_summary() */
+
 
 /*-------------------------------------------------------------------------
  * Function: print_file_statistics
@@ -1321,23 +1367,25 @@ print_file_statistics(const iter_t *iter)
         display_dset = TRUE;
         display_dset_dtype_meta = TRUE;
         display_attr = TRUE;
+        display_summary = TRUE;
 
         display_file_metadata = TRUE;
         display_group_metadata = TRUE;
         display_dset_metadata = TRUE;
     }
 
-    if(display_file)          print_file_info(iter);
-    if(display_file_metadata) print_file_metadata(iter);
+    if(display_file)            print_file_info(iter);
+    if(display_file_metadata)   print_file_metadata(iter);
 
-    if(display_group)         print_group_info(iter);
-    if(!display_all && display_group_metadata)  print_group_metadata(iter);
+    if(display_group)           print_group_info(iter);
+    if(!display_all && display_group_metadata)   print_group_metadata(iter);
 
-    if(display_dset)          print_dataset_info(iter);
+    if(display_dset)            print_dataset_info(iter);
     if(display_dset_dtype_meta) print_dset_dtype_meta(iter);
     if(!display_all && display_dset_metadata)   print_dset_metadata(iter);
 
-    if(display_attr)          print_attr_info(iter);
+    if(display_attr)            print_attr_info(iter);
+    if(display_summary)         print_storage_summary(iter);
 } /* print_file_statistics() */
 
 
@@ -1402,12 +1450,12 @@ print_statistics(const char *name, const iter_t *iter)
 int
 main(int argc, const char *argv[])
 {
-    iter_t            iter;
-    const char       *fname = NULL;
-    hid_t             fid;
+    iter_t              iter;
+    const char         *fname = NULL;
+    hid_t               fid;
     hid_t               fcpl;
     struct handler_t   *hand = NULL;
-    H5F_info_t        finfo;
+    H5F_info_t         finfo;
     int                 i;
 
     h5tools_setprogname(PROGRAMNAME);
@@ -1418,6 +1466,7 @@ main(int argc, const char *argv[])
 
     /* Initialize h5tools lib */
     h5tools_init();
+    
     if((hand = parse_command_line(argc, argv))==NULL) {
         goto done;
     }
@@ -1436,14 +1485,18 @@ main(int argc, const char *argv[])
     /* Initialize iter structure */
     iter_init(&iter, fid);
 
-    /* Get storge info for SOHM's btree/list/heap and superblock extension */
+    if(H5Fget_filesize(fid, &iter.filesize) < 0)
+        warn_msg("Unable to retrieve file size\n");
+    HDassert(iter.filesize != 0);
+
+    /* Get storge info for file-level structures */
     if(H5Fget_info(fid, &finfo) < 0)
-    warn_msg("Unable to retrieve SOHM info\n");
+        warn_msg("Unable to retrieve file info\n");
     else {
-    iter.super_ext_size = finfo.super_ext_size;
-    iter.SM_hdr_storage_size = finfo.sohm.hdr_size;
-    iter.SM_index_storage_size = finfo.sohm.msgs_info.index_size;
-    iter.SM_heap_storage_size = finfo.sohm.msgs_info.heap_size;
+        iter.super_ext_size = finfo.super_ext_size;
+        iter.SM_hdr_storage_size = finfo.sohm.hdr_size;
+        iter.SM_index_storage_size = finfo.sohm.msgs_info.index_size;
+        iter.SM_heap_storage_size = finfo.sohm.msgs_info.heap_size;
     } /* end else */
 
     if((fcpl = H5Fget_create_plist(fid)) < 0)
@@ -1459,15 +1512,15 @@ main(int argc, const char *argv[])
         u = 0;
         while(hand[u].obj) {
             if (h5trav_visit(fid, hand[u].obj, TRUE, TRUE, obj_stats, lnk_stats, &iter) < 0)
-    warn_msg("Unable to traverse object \"%s\"\n", hand[u].obj);
-      else
-    print_statistics(hand[u].obj, &iter);
+                warn_msg("Unable to traverse object \"%s\"\n", hand[u].obj);
+            else
+                print_statistics(hand[u].obj, &iter);
             u++;
         } /* end while */
     } /* end if */
     else {
         if (h5trav_visit(fid, "/", TRUE, TRUE, obj_stats, lnk_stats, &iter) < 0)
-      warn_msg("Unable to traverse objects/links in file \"%s\"\n", fname);
+            warn_msg("Unable to traverse objects/links in file \"%s\"\n", fname);
   else
       print_statistics("/", &iter);
     } /* end else */
@@ -1486,8 +1539,8 @@ done:
         /* Free iter structure */
         iter_free(&iter);
     
-    if(H5Fclose(fid) < 0) {
-        error_msg("unable to close file \"%s\"\n", fname);
+        if(H5Fclose(fid) < 0) {
+            error_msg("unable to close file \"%s\"\n", fname);
             h5tools_setstatus(EXIT_FAILURE);
         }
     }
