@@ -409,7 +409,7 @@ hsize_t diff_datasetid( hid_t did1,
                 options, name1, name2, m_tid1, did1, did2);
 
             /* reclaim any VL memory, if necessary */
-       	    if(vl_data) {
+            if(vl_data) {
                 H5Dvlen_reclaim(m_tid1, sid1, H5P_DEFAULT, buf1);
                 H5Dvlen_reclaim(m_tid2, sid2, H5P_DEFAULT, buf2);
             } /* end if */
@@ -445,27 +445,27 @@ hsize_t diff_datasetid( hid_t did1,
                     size = 1;
                 sm_size[i - 1] = MIN(dims1[i - 1], size);
                 sm_nbytes *= sm_size[i - 1];
-                assert(sm_nbytes > 0);
+                HDassert(sm_nbytes > 0);
             } /* end for */
 
-	    /* malloc return code should be verified.
+      /* malloc return code should be verified.
              * If fail, need to handle the error.
              * This else branch should be recoded as a separate function.
              * Note that there are many "goto error" within this branch
              * that fails to address freeing other objects created here.
-	     * E.g., sm_space.
-	     */
-            sm_buf1 = malloc((size_t)sm_nbytes);
-	    assert(sm_buf1);
-            sm_buf2 = malloc((size_t)sm_nbytes);
-	    assert(sm_buf2);
+       * E.g., sm_space.
+       */
+            sm_buf1 = HDmalloc((size_t)sm_nbytes);
+            HDassert(sm_buf1);
+            sm_buf2 = HDmalloc((size_t)sm_nbytes);
+            HDassert(sm_buf2);
 
             sm_nelmts = sm_nbytes / p_type_nbytes;
             sm_space = H5Screate_simple(1, &sm_nelmts, NULL);
 
             /* the stripmine loop */
-            memset(hs_offset, 0, sizeof hs_offset);
-            memset(zero, 0, sizeof zero);
+            HDmemset(hs_offset, 0, sizeof hs_offset);
+            HDmemset(zero, 0, sizeof zero);
 
             for(elmtno = 0; elmtno < p_nelmts; elmtno += hs_nelmts) {
                 /* calculate the hyperslab size */
@@ -529,19 +529,19 @@ hsize_t diff_datasetid( hid_t did1,
 
     /* free */
     if(buf1 != NULL) {
-        free(buf1);
+        HDfree(buf1);
         buf1 = NULL;
     } /* end if */
     if(buf2 != NULL) {
-        free(buf2);
+        HDfree(buf2);
         buf2 = NULL;
     } /* end if */
     if(sm_buf1 != NULL) {
-        free(sm_buf1);
+        HDfree(sm_buf1);
         sm_buf1 = NULL;
     } /* end if */
     if(sm_buf2 != NULL) {
-        free(sm_buf2);
+        HDfree(sm_buf2);
         sm_buf2 = NULL;
     } /* end if */
 
@@ -565,7 +565,7 @@ error:
         /* reclaim any VL memory, if necessary */
         if(vl_data)
             H5Dvlen_reclaim(m_tid1, sid1, H5P_DEFAULT, buf1);
-        free(buf1);
+        HDfree(buf1);
         buf1=NULL;
     }
     if (buf2!=NULL)
@@ -573,7 +573,7 @@ error:
         /* reclaim any VL memory, if necessary */
         if(vl_data)
             H5Dvlen_reclaim(m_tid2, sid2, H5P_DEFAULT, buf2);
-        free(buf2);
+        HDfree(buf2);
         buf2=NULL;
     }
     if (sm_buf1!=NULL)
@@ -581,7 +581,7 @@ error:
         /* reclaim any VL memory, if necessary */
         if(vl_data)
             H5Dvlen_reclaim(m_tid1, sm_space, H5P_DEFAULT, sm_buf1);
-        free(sm_buf1);
+        HDfree(sm_buf1);
         sm_buf1=NULL;
     }
     if (sm_buf2!=NULL)
@@ -589,7 +589,7 @@ error:
         /* reclaim any VL memory, if necessary */
         if(vl_data)
             H5Dvlen_reclaim(m_tid1, sm_space, H5P_DEFAULT, sm_buf2);
-        free(sm_buf2);
+        HDfree(sm_buf2);
         sm_buf2=NULL;
     }
 
@@ -695,7 +695,7 @@ int diff_can_type( hid_t       f_tid1, /* file data type */
     *-------------------------------------------------------------------------
     */
 
-    assert(tclass1==tclass2);
+    HDassert(tclass1==tclass2);
     switch (tclass1)
     {
     case H5T_INTEGER:
@@ -784,7 +784,7 @@ int diff_can_type( hid_t       f_tid1, /* file data type */
     *-------------------------------------------------------------------------
     */
 
-    assert(rank1==rank2);
+    HDassert(rank1==rank2);
     for ( i=0; i<rank1; i++)
     {
         if (maxdim1 && maxdim2)
